@@ -21,7 +21,10 @@
 #  include "config.h"
 #endif
 
+#include "defs.h"
+
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -31,11 +34,8 @@
 #  include <locale.h>
 #endif
 
-#if HAVE_ICONV
-#  include <iconv.h>
-#endif
+#include <iconv.h>
 
-#include "intl.h"
 #include "codeconv.h"
 #include "unmime.h"
 #include "base64.h"
@@ -104,11 +104,40 @@ typedef enum
 		state = JIS_AUXKANJI;	\
 	}
 
-void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_jistoeuc(const gchar *inbuf, gint *error);
+static gchar *conv_euctojis(const gchar *inbuf, gint *error);
+static gchar *conv_sjistoeuc(const gchar *inbuf, gint *error);
+
+static gchar *conv_jistoutf8(const gchar *inbuf, gint *error);
+static gchar *conv_sjistoutf8(const gchar *inbuf, gint *error);
+static gchar *conv_euctoutf8(const gchar *inbuf, gint *error);
+static gchar *conv_anytoutf8(const gchar *inbuf, gint *error);
+
+static gchar *conv_utf8toeuc(const gchar *inbuf, gint *error);
+static gchar *conv_utf8tojis(const gchar *inbuf, gint *error);
+
+/* static void conv_unreadable_eucjp(gchar *str); */
+static void conv_unreadable_8bit(gchar *str);
+/* static void conv_unreadable_latin(gchar *str); */
+
+static gchar *conv_jistodisp(const gchar *inbuf, gint *error);
+static gchar *conv_sjistodisp(const gchar *inbuf, gint *error);
+static gchar *conv_euctodisp(const gchar *inbuf, gint *error);
+
+static gchar *conv_anytodisp(const gchar *inbuf, gint *error);
+static gchar *conv_ustodisp(const gchar *inbuf, gint *error);
+static gchar *conv_noconv(const gchar *inbuf, gint *error);
+
+static gchar *conv_jistoeuc(const gchar *inbuf, gint *error)
 {
+	gchar *outbuf;
 	const guchar *in = inbuf;
-	guchar *out = outbuf;
+	guchar *out;
 	JISState state = JIS_ASCII;
+	gint error_ = 0;
+
+	outbuf = g_malloc(strlen(inbuf) * 2 + 1);
+	out = outbuf;
 
 	while (*in != '\0') {
 		if (*in == ESC) {
@@ -123,6 +152,7 @@ void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 					in += 3;
 				} else {
 					/* unknown escape sequence */
+					error_ = -1;
 					state = JIS_ASCII;
 				}
 			} else if (*in == '(') {
@@ -134,10 +164,12 @@ void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 					in += 2;
 				} else {
 					/* unknown escape sequence */
+					error_ = -1;
 					state = JIS_ASCII;
 				}
 			} else {
 				/* unknown escape sequence */
+				error_ = -1;
 				state = JIS_ASCII;
 			}
 		} else if (*in == 0x0e) {
@@ -171,6 +203,11 @@ void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 	}
 
 	*out = '\0';
+
+	if (error)
+		*error = error_;
+
+	return outbuf;
 }
 
 #define JIS_HWDAKUTEN		0x5e
@@ -239,11 +276,16 @@ static gint conv_jis_hantozen(guchar *outbuf, guchar jis_code, guchar sound_sym)
 	return 1;
 }
 
-void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_euctojis(const gchar *inbuf, gint *error)
 {
+	gchar *outbuf;
 	const guchar *in = inbuf;
-	guchar *out = outbuf;
+	guchar *out;
 	JISState state = JIS_ASCII;
+	gint error_ = 0;
+ 
+	outbuf = g_malloc(strlen(inbuf) * 3 + 4);
+	out = outbuf;
 
 	while (*in != '\0') {
 		if (isascii(*in)) {
@@ -255,6 +297,7 @@ void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 				*out++ = *in++ & 0x7f;
 				*out++ = *in++ & 0x7f;
 			} else {
+				error_ = -1;
 				K_OUT();
 				*out++ = SUBST_CHAR;
 				in++;
@@ -292,6 +335,7 @@ void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 					}
 				}
 			} else {
+				error_ = -1;
 				K_OUT();
 				in++;
 				if (*in != '\0' && !isascii(*in)) {
@@ -306,6 +350,7 @@ void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 				*out++ = *in++ & 0x7f;
 				*out++ = *in++ & 0x7f;
 			} else {
+				error_ = -1;
 				K_OUT();
 				if (*in != '\0' && !isascii(*in)) {
 					*out++ = SUBST_CHAR;
@@ -317,6 +362,7 @@ void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 				}
 			}
 		} else {
+			error_ = -1;
 			K_OUT();
 			*out++ = SUBST_CHAR;
 			in++;
@@ -325,12 +371,22 @@ void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 
 	K_OUT();
 	*out = '\0';
+
+	if (error)
+		*error = error_;
+
+	return outbuf;
 }
 
-void conv_sjistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_sjistoeuc(const gchar *inbuf, gint *error)
 {
+	gchar *outbuf;
 	const guchar *in = inbuf;
-	guchar *out = outbuf;
+	guchar *out;
+	gint error_ = 0;
+
+	outbuf = g_malloc(strlen(inbuf) * 2 + 1);
+	out = outbuf;
 
 	while (*in != '\0') {
 		if (isascii(*in)) {
@@ -354,6 +410,7 @@ void conv_sjistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 				*out++ = out2 | 0x80;
 				in += 2;
 			} else {
+				error_ = -1;
 				*out++ = SUBST_CHAR;
 				in++;
 				if (*in != '\0' && !isascii(*in)) {
@@ -365,41 +422,136 @@ void conv_sjistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 			*out++ = 0x8e;
 			*out++ = *in++;
 		} else {
+			error_ = -1;
 			*out++ = SUBST_CHAR;
 			in++;
 		}
 	}
 
 	*out = '\0';
+
+	if (error)
+		*error = error_;
+
+	return outbuf;
 }
 
-void conv_anytoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_jistoutf8(const gchar *inbuf, gint *error)
+{
+	gchar *eucstr, *utf8str;
+	gint e_error = 0, u_error = 0;
+
+	eucstr = conv_jistoeuc(inbuf, &e_error);
+	utf8str = conv_euctoutf8(eucstr, &u_error);
+	g_free(eucstr);
+
+	if (error)
+		*error = (e_error | u_error);
+
+	return utf8str;
+}
+
+static gchar *conv_sjistoutf8(const gchar *inbuf, gint *error)
+{
+	gchar *utf8str;
+
+	utf8str = conv_iconv_strdup(inbuf, CS_SHIFT_JIS, CS_UTF_8, error);
+	if (!utf8str)
+		utf8str = g_strdup(inbuf);
+
+	return utf8str;
+}
+
+static gchar *conv_euctoutf8(const gchar *inbuf, gint *error)
+{
+	static iconv_t cd = (iconv_t)-1;
+	static gboolean iconv_ok = TRUE;
+
+	if (cd == (iconv_t)-1) {
+		if (!iconv_ok) {
+			if (error)
+				*error = -1;
+			return g_strdup(inbuf);
+		}
+
+		cd = iconv_open(CS_UTF_8, CS_EUC_JP_MS);
+		if (cd == (iconv_t)-1) {
+			cd = iconv_open(CS_UTF_8, CS_EUC_JP);
+			if (cd == (iconv_t)-1) {
+				g_warning("conv_euctoutf8(): %s\n",
+					  g_strerror(errno));
+				iconv_ok = FALSE;
+				if (error)
+					*error = -1;
+				return g_strdup(inbuf);
+			}
+		}
+	}
+
+	return conv_iconv_strdup_with_cd(inbuf, cd, error);
+}
+
+static gchar *conv_anytoutf8(const gchar *inbuf, gint *error)
 {
 	switch (conv_guess_ja_encoding(inbuf)) {
 	case C_ISO_2022_JP:
-		conv_jistoeuc(outbuf, outlen, inbuf);
-		break;
+		return conv_jistoutf8(inbuf, error);
 	case C_SHIFT_JIS:
-		conv_sjistoeuc(outbuf, outlen, inbuf);
-		break;
-	default:
-		strncpy2(outbuf, inbuf, outlen);
-		break;
-	}
-}
-
-void conv_anytojis(gchar *outbuf, gint outlen, const gchar *inbuf)
-{
-	switch (conv_guess_ja_encoding(inbuf)) {
+		return conv_sjistoutf8(inbuf, error);
 	case C_EUC_JP:
-		conv_euctojis(outbuf, outlen, inbuf);
-		break;
+		return conv_euctoutf8(inbuf, error);
 	default:
-		strncpy2(outbuf, inbuf, outlen);
-		break;
+		if (error)
+			*error = 0;
+		return g_strdup(inbuf);
 	}
 }
 
+static gchar *conv_utf8toeuc(const gchar *inbuf, gint *error)
+{
+	static iconv_t cd = (iconv_t)-1;
+	static gboolean iconv_ok = TRUE;
+
+	if (cd == (iconv_t)-1) {
+		if (!iconv_ok) {
+			if (error)
+				*error = -1;
+			return g_strdup(inbuf);
+		}
+
+		cd = iconv_open(CS_EUC_JP_MS, CS_UTF_8);
+		if (cd == (iconv_t)-1) {
+			cd = iconv_open(CS_EUC_JP, CS_UTF_8);
+			if (cd == (iconv_t)-1) {
+				g_warning("conv_utf8toeuc(): %s\n",
+					  g_strerror(errno));
+				iconv_ok = FALSE;
+				if (error)
+					*error = -1;
+				return g_strdup(inbuf);
+			}
+		}
+	}
+
+	return conv_iconv_strdup_with_cd(inbuf, cd, error);
+}
+
+static gchar *conv_utf8tojis(const gchar *inbuf, gint *error)
+{
+	gchar *eucstr, *jisstr;
+	gint e_error = 0, j_error = 0;
+
+	eucstr = conv_utf8toeuc(inbuf, &e_error);
+	jisstr = conv_euctojis(eucstr, &j_error);
+	g_free(eucstr);
+
+	if (error)
+		*error = (e_error | j_error);
+
+	return jisstr;
+}
+
+#if 0
 static gchar valid_eucjp_tbl[][96] = {
 	/* 0xa2a0 - 0xa2ff */
 	{ 0, 1, 1, 1, 1, 1, 1, 1,  1, 1, 1, 1, 1, 1, 1, 0,
@@ -482,7 +634,7 @@ static gboolean isprintableeuckanji(guchar c1, guchar c2)
 	return TRUE;
 }
 
-void conv_unreadable_eucjp(gchar *str)
+static void conv_unreadable_eucjp(gchar *str)
 {
 	register guchar *p = str;
 
@@ -524,8 +676,9 @@ void conv_unreadable_eucjp(gchar *str)
 			*p++ = SUBST_CHAR;
 	}
 }
+#endif
 
-void conv_unreadable_8bit(gchar *str)
+static void conv_unreadable_8bit(gchar *str)
 {
 	register guchar *p = str;
 
@@ -538,7 +691,8 @@ void conv_unreadable_8bit(gchar *str)
 	}
 }
 
-void conv_unreadable_latin(gchar *str)
+#if 0
+static void conv_unreadable_latin(gchar *str)
 {
 	register guchar *p = str;
 
@@ -551,34 +705,7 @@ void conv_unreadable_latin(gchar *str)
 		p++;
 	}
 }
-
-void conv_unreadable_locale(gchar *str)
-{
-	switch (conv_get_current_charset()) {
-	case C_US_ASCII:
-	case C_ISO_8859_1:
-	case C_ISO_8859_2:
-	case C_ISO_8859_3:
-	case C_ISO_8859_4:
-	case C_ISO_8859_5:
-	case C_ISO_8859_6:
-	case C_ISO_8859_7:
-	case C_ISO_8859_8:
-	case C_ISO_8859_9:
-	case C_ISO_8859_10:
-	case C_ISO_8859_11:
-	case C_ISO_8859_13:
-	case C_ISO_8859_14:
-	case C_ISO_8859_15:
-		conv_unreadable_latin(str);
-		break;
-	case C_EUC_JP:
-		conv_unreadable_eucjp(str);
-		break;
-	default:
-		break;
-	}
-}
+#endif
 
 #define NCV	'\0'
 
@@ -688,162 +815,163 @@ CharSet conv_guess_ja_encoding(const gchar *str)
 	return guessed;
 }
 
-void conv_jistodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_jistodisp(const gchar *inbuf, gint *error)
 {
-	conv_jistoeuc(outbuf, outlen, inbuf);
-	conv_unreadable_eucjp(outbuf);
+	return conv_jistoutf8(inbuf, error);
 }
 
-void conv_sjistodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_sjistodisp(const gchar *inbuf, gint *error)
 {
-	conv_sjistoeuc(outbuf, outlen, inbuf);
-	conv_unreadable_eucjp(outbuf);
+	return conv_sjistoutf8(inbuf, error);
 }
 
-void conv_euctodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_euctodisp(const gchar *inbuf, gint *error)
 {
-	strncpy2(outbuf, inbuf, outlen);
-	conv_unreadable_eucjp(outbuf);
+	return conv_euctoutf8(inbuf, error);
 }
 
-void conv_anytodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+gchar *conv_utf8todisp(const gchar *inbuf, gint *error)
 {
-	conv_anytoeuc(outbuf, outlen, inbuf);
-	conv_unreadable_eucjp(outbuf);
+	if (g_utf8_validate(inbuf, -1, NULL) == TRUE) {
+		if (error)
+			*error = 0;
+		return g_strdup(inbuf);
+	} else
+		return conv_ustodisp(inbuf, error);
 }
 
-void conv_ustodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_anytodisp(const gchar *inbuf, gint *error)
 {
-	strncpy2(outbuf, inbuf, outlen);
+	gchar *outbuf;
+
+	outbuf = conv_anytoutf8(inbuf, error);
+	if (g_utf8_validate(outbuf, -1, NULL) != TRUE) {
+		if (error)
+			*error = -1;
+		conv_unreadable_8bit(outbuf);
+	}
+
+	return outbuf;
+}
+
+static gchar *conv_ustodisp(const gchar *inbuf, gint *error)
+{
+	gchar *outbuf;
+
+	outbuf = g_strdup(inbuf);
 	conv_unreadable_8bit(outbuf);
+	if (error)
+		*error = 0;
+
+	return outbuf;
 }
 
-void conv_latintodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+gchar *conv_localetodisp(const gchar *inbuf, gint *error)
 {
-	strncpy2(outbuf, inbuf, outlen);
-	conv_unreadable_latin(outbuf);
+	gchar *str;
+
+	str = conv_iconv_strdup(inbuf, conv_get_locale_charset_str(),
+				CS_INTERNAL, error);
+	if (!str)
+		str = conv_utf8todisp(inbuf, NULL);
+
+	return str;
 }
 
-void conv_localetodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static gchar *conv_noconv(const gchar *inbuf, gint *error)
 {
-	strncpy2(outbuf, inbuf, outlen);
-	conv_unreadable_locale(outbuf);
+	if (error)
+		*error = 0;
+	return g_strdup(inbuf);
 }
 
-void conv_noconv(gchar *outbuf, gint outlen, const gchar *inbuf)
+static const gchar *
+conv_get_fallback_for_private_encoding(const gchar *encoding)
 {
-	strncpy2(outbuf, inbuf, outlen);
+	if (encoding && (encoding[0] == 'X' || encoding[0] == 'x') &&
+	    encoding[1] == '-') {
+		if (!g_strcasecmp(encoding, CS_X_GBK))
+			return CS_GBK;
+	}
+
+	return encoding;
 }
 
-CodeConverter *conv_code_converter_new(const gchar *charset)
+CodeConverter *conv_code_converter_new(const gchar *src_encoding,
+				       const gchar *dest_encoding)
 {
 	CodeConverter *conv;
 
+	src_encoding = conv_get_fallback_for_private_encoding(src_encoding);
+
 	conv = g_new0(CodeConverter, 1);
-	conv->code_conv_func = conv_get_code_conv_func(charset, NULL);
-	conv->charset_str = g_strdup(charset);
-	conv->charset = conv_get_charset_from_str(charset);
+	conv->code_conv_func =
+		conv_get_code_conv_func(src_encoding, dest_encoding);
+	conv->src_encoding = g_strdup(src_encoding);
+	conv->dest_encoding = g_strdup(dest_encoding);
 
 	return conv;
 }
 
 void conv_code_converter_destroy(CodeConverter *conv)
 {
-	g_free(conv->charset_str);
+	g_free(conv->src_encoding);
+	g_free(conv->dest_encoding);
 	g_free(conv);
 }
 
-gint conv_convert(CodeConverter *conv, gchar *outbuf, gint outlen,
-		  const gchar *inbuf)
+gchar *conv_convert(CodeConverter *conv, const gchar *inbuf)
 {
-#if HAVE_ICONV
 	if (conv->code_conv_func != conv_noconv)
-		conv->code_conv_func(outbuf, outlen, inbuf);
-	else {
-		gchar *str;
-
-		str = conv_iconv_strdup(inbuf, conv->charset_str, NULL);
-		if (!str)
-			return -1;
-		else {
-			strncpy2(outbuf, str, outlen);
-			g_free(str);
-		}
-	}
-#else /* !HAVE_ICONV */
-	conv->code_conv_func(outbuf, outlen, inbuf);
-#endif
-
-	return 0;
+		return conv->code_conv_func(inbuf, NULL);
+	else
+		return conv_iconv_strdup
+			(inbuf, conv->src_encoding, conv->dest_encoding, NULL);
 }
 
-gchar *conv_codeset_strdup(const gchar *inbuf,
-			   const gchar *src_code, const gchar *dest_code)
+gchar *conv_codeset_strdup_full(const gchar *inbuf,
+				const gchar *src_encoding,
+				const gchar *dest_encoding,
+				gint *error)
 {
-	gchar *buf;
-	size_t len;
 	CodeConvFunc conv_func;
 
-	conv_func = conv_get_code_conv_func(src_code, dest_code);
-	if (conv_func != conv_noconv) {
-		len = (strlen(inbuf) + 1) * 3;
-		buf = g_malloc(len);
-		if (!buf) return NULL;
+	src_encoding = conv_get_fallback_for_private_encoding(src_encoding);
 
-		conv_func(buf, len, inbuf);
-		return g_realloc(buf, strlen(buf) + 1);
-	}
+	conv_func = conv_get_code_conv_func(src_encoding, dest_encoding);
+	if (conv_func != conv_noconv)
+		return conv_func(inbuf, error);
 
-#if HAVE_ICONV
-	return conv_iconv_strdup(inbuf, src_code, dest_code);
-#else
-	return g_strdup(inbuf);
-#endif /* HAVE_ICONV */
+	return conv_iconv_strdup(inbuf, src_encoding, dest_encoding, error);
 }
 
-CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
-				     const gchar *dest_charset_str)
+CodeConvFunc conv_get_code_conv_func(const gchar *src_encoding,
+				     const gchar *dest_encoding)
 {
 	CodeConvFunc code_conv = conv_noconv;
 	CharSet src_charset;
 	CharSet dest_charset;
 
-	if (!src_charset_str)
-		src_charset = conv_get_current_charset();
+	if (!src_encoding)
+		src_charset = conv_get_locale_charset();
 	else
-		src_charset = conv_get_charset_from_str(src_charset_str);
+		src_charset = conv_get_charset_from_str(src_encoding);
 
 	/* auto detection mode */
-	if (!src_charset_str && !dest_charset_str) {
+	if (!src_encoding && !dest_encoding) {
 		if (src_charset == C_EUC_JP || src_charset == C_SHIFT_JIS)
 			return conv_anytodisp;
 		else
 			return conv_noconv;
 	}
 
-	dest_charset = conv_get_charset_from_str(dest_charset_str);
+	dest_charset = conv_get_charset_from_str(dest_encoding);
 
 	if (dest_charset == C_US_ASCII)
 		return conv_ustodisp;
-	else if (dest_charset == C_UTF_8 ||
-		 (dest_charset == C_AUTO &&
-		  conv_get_current_charset() == C_UTF_8))
-		return conv_noconv;
 
 	switch (src_charset) {
-	case C_ISO_2022_JP:
-	case C_ISO_2022_JP_2:
-	case C_ISO_2022_JP_3:
-		if (dest_charset == C_AUTO &&
-		    conv_get_current_charset() == C_EUC_JP)
-			code_conv = conv_jistodisp;
-		else if (dest_charset == C_EUC_JP)
-			code_conv = conv_jistoeuc;
-		break;
 	case C_US_ASCII:
-		if (dest_charset == C_AUTO)
-			code_conv = conv_ustodisp;
-		break;
 	case C_ISO_8859_1:
 	case C_ISO_8859_2:
 	case C_ISO_8859_3:
@@ -858,26 +986,42 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
 	case C_ISO_8859_13:
 	case C_ISO_8859_14:
 	case C_ISO_8859_15:
-		if (dest_charset == C_AUTO &&
-		    (conv_get_current_charset() == src_charset ||
-		     MB_CUR_MAX > 1))
-			code_conv = conv_latintodisp;
+		break;
+	case C_ISO_2022_JP:
+	case C_ISO_2022_JP_2:
+	case C_ISO_2022_JP_3:
+		if (dest_charset == C_AUTO)
+			code_conv = conv_jistodisp;
+		else if (dest_charset == C_EUC_JP)
+			code_conv = conv_jistoeuc;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_jistoutf8;
 		break;
 	case C_SHIFT_JIS:
-		if (dest_charset == C_AUTO &&
-		    conv_get_current_charset() == C_EUC_JP)
+		if (dest_charset == C_AUTO)
 			code_conv = conv_sjistodisp;
 		else if (dest_charset == C_EUC_JP)
 			code_conv = conv_sjistoeuc;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_sjistoutf8;
 		break;
 	case C_EUC_JP:
-		if (dest_charset == C_AUTO &&
-		    conv_get_current_charset() == C_EUC_JP)
+		if (dest_charset == C_AUTO)
 			code_conv = conv_euctodisp;
 		else if (dest_charset == C_ISO_2022_JP   ||
 			 dest_charset == C_ISO_2022_JP_2 ||
 			 dest_charset == C_ISO_2022_JP_3)
 			code_conv = conv_euctojis;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_euctoutf8;
+		break;
+	case C_UTF_8:
+		if (dest_charset == C_EUC_JP)
+			code_conv = conv_utf8toeuc;
+		else if (dest_charset == C_ISO_2022_JP   ||
+			 dest_charset == C_ISO_2022_JP_2 ||
+			 dest_charset == C_ISO_2022_JP_3)
+			code_conv = conv_utf8tojis;
 		break;
 	default:
 		break;
@@ -886,11 +1030,34 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
 	return code_conv;
 }
 
-#if HAVE_ICONV
 gchar *conv_iconv_strdup(const gchar *inbuf,
-			 const gchar *src_code, const gchar *dest_code)
+			 const gchar *src_code, const gchar *dest_code,
+			 gint *error)
 {
 	iconv_t cd;
+	gchar *outbuf;
+
+	if (!src_code)
+		src_code = conv_get_locale_charset_str();
+	if (!dest_code)
+		dest_code = CS_INTERNAL;
+
+	cd = iconv_open(dest_code, src_code);
+	if (cd == (iconv_t)-1) {
+		if (error)
+			*error = -1;
+		return NULL;
+	}
+
+	outbuf = conv_iconv_strdup_with_cd(inbuf, cd, error);
+
+	iconv_close(cd);
+
+	return outbuf;
+}
+
+gchar *conv_iconv_strdup_with_cd(const gchar *inbuf, iconv_t cd, gint *error)
+{
 	const gchar *inbuf_p;
 	gchar *outbuf;
 	gchar *outbuf_p;
@@ -900,23 +1067,7 @@ gchar *conv_iconv_strdup(const gchar *inbuf,
 	size_t out_left;
 	size_t n_conv;
 	size_t len;
-
-	if (!src_code)
-		src_code = conv_get_outgoing_charset_str();
-	if (!dest_code)
-		dest_code = conv_get_current_charset_str();
-
-	/* don't convert if current codeset is US-ASCII */
-	if (!strcasecmp(dest_code, CS_US_ASCII))
-		return g_strdup(inbuf);
-
-	/* don't convert if src and dest codeset are identical */
-	if (!strcasecmp(src_code, dest_code))
-		return g_strdup(inbuf);
-
-	cd = iconv_open(dest_code, src_code);
-	if (cd == (iconv_t)-1)
-		return NULL;
+	gint error_ = 0;
 
 	inbuf_p = inbuf;
 	in_size = strlen(inbuf);
@@ -938,6 +1089,8 @@ gchar *conv_iconv_strdup(const gchar *inbuf,
 	while ((n_conv = iconv(cd, (ICONV_CONST gchar **)&inbuf_p, &in_left,
 			       &outbuf_p, &out_left)) == (size_t)-1) {
 		if (EILSEQ == errno) {
+			/* g_print("iconv(): at %d: %s\n", in_size - in_left, g_strerror(errno)); */
+			error_ = -1;
 			inbuf_p++;
 			in_left--;
 			if (out_left == 0) {
@@ -946,12 +1099,14 @@ gchar *conv_iconv_strdup(const gchar *inbuf,
 			*outbuf_p++ = SUBST_CHAR;
 			out_left--;
 		} else if (EINVAL == errno) {
+			error_ = -1;
 			break;
 		} else if (E2BIG == errno) {
 			EXPAND_BUF();
 		} else {
 			g_warning("conv_iconv_strdup(): %s\n",
 				  g_strerror(errno));
+			error_ = -1;
 			break;
 		}
 	}
@@ -963,6 +1118,7 @@ gchar *conv_iconv_strdup(const gchar *inbuf,
 		} else {
 			g_warning("conv_iconv_strdup(): %s\n",
 				  g_strerror(errno));
+			error_ = -1;
 			break;
 		}
 	}
@@ -973,11 +1129,11 @@ gchar *conv_iconv_strdup(const gchar *inbuf,
 	outbuf = g_realloc(outbuf, len + 1);
 	outbuf[len] = '\0';
 
-	iconv_close(cd);
+	if (error)
+		*error = error_;
 
 	return outbuf;
 }
-#endif /* HAVE_ICONV */
 
 static const struct {
 	CharSet charset;
@@ -1028,6 +1184,7 @@ static const struct {
 	{C_ISO_2022_JP_3,	CS_ISO_2022_JP_3},
 	{C_EUC_JP,		CS_EUC_JP},
 	{C_EUC_JP,		CS_EUCJP},
+	{C_EUC_JP_MS,		CS_EUC_JP_MS},
 	{C_SHIFT_JIS,		CS_SHIFT_JIS},
 	{C_SHIFT_JIS,		CS_SHIFT__JIS},
 	{C_SHIFT_JIS,		CS_SJIS},
@@ -1061,7 +1218,7 @@ static const struct {
 	{"ko_KR.EUC-KR"	, C_EUC_KR	, C_EUC_KR},
 	{"ko_KR"	, C_EUC_KR	, C_EUC_KR},
 	{"zh_CN.GB2312"	, C_GB2312	, C_GB2312},
-	{"zh_CN.GBK"	, C_GBK		, C_GB2312},
+	{"zh_CN.GBK"	, C_GBK		, C_GBK},
 	{"zh_CN"	, C_GB2312	, C_GB2312},
 	{"zh_HK"	, C_BIG5_HKSCS	, C_BIG5_HKSCS},
 	{"zh_TW.eucTW"	, C_EUC_TW	, C_BIG5},
@@ -1284,7 +1441,7 @@ CharSet conv_get_charset_from_str(const gchar *charset)
 	return GPOINTER_TO_UINT(g_hash_table_lookup(table, charset));
 }
 
-CharSet conv_get_current_charset(void)
+CharSet conv_get_locale_charset(void)
 {
 	static CharSet cur_charset = -1;
 	const gchar *cur_locale;
@@ -1333,14 +1490,24 @@ CharSet conv_get_current_charset(void)
 	return cur_charset;
 }
 
-const gchar *conv_get_current_charset_str(void)
+const gchar *conv_get_locale_charset_str(void)
 {
 	static const gchar *codeset = NULL;
 
 	if (!codeset)
-		codeset = conv_get_charset_str(conv_get_current_charset());
+		codeset = conv_get_charset_str(conv_get_locale_charset());
 
-	return codeset ? codeset : CS_US_ASCII;
+	return codeset ? codeset : CS_INTERNAL;
+}
+
+CharSet conv_get_internal_charset(void)
+{
+	return C_INTERNAL;
+}
+
+const gchar *conv_get_internal_charset_str(void)
+{
+	return CS_INTERNAL;
 }
 
 CharSet conv_get_outgoing_charset(void)
@@ -1381,15 +1548,6 @@ CharSet conv_get_outgoing_charset(void)
 		}
 	}
 
-#if !HAVE_ICONV
-	/* encoding conversion without iconv() is only supported
-	   on Japanese locale for now */
-	if (out_charset == C_ISO_2022_JP)
-		return out_charset;
-	else
-		return conv_get_current_charset();
-#endif
-
 	return out_charset;
 }
 
@@ -1398,24 +1556,17 @@ const gchar *conv_get_outgoing_charset_str(void)
 	CharSet out_charset;
 	const gchar *str;
 
-	if (prefs_common.outgoing_charset) {
-		if (!isalpha((guchar)prefs_common.outgoing_charset[0])) {
-			g_free(prefs_common.outgoing_charset);
-			prefs_common.outgoing_charset = g_strdup(CS_AUTO);
-		} else if (strcmp(prefs_common.outgoing_charset, CS_AUTO) != 0)
-			return prefs_common.outgoing_charset;
-	}
-
 	out_charset = conv_get_outgoing_charset();
 	str = conv_get_charset_str(out_charset);
 
-	return str ? str : CS_US_ASCII;
+	return str ? str : CS_UTF_8;
 }
 
 gboolean conv_is_multibyte_encoding(CharSet encoding)
 {
 	switch (encoding) {
 	case C_EUC_JP:
+	case C_EUC_JP_MS:
 	case C_EUC_KR:
 	case C_EUC_TW:
 	case C_EUC_CN:
@@ -1426,6 +1577,7 @@ gboolean conv_is_multibyte_encoding(CharSet encoding)
 	case C_ISO_2022_CN:
 	case C_SHIFT_JIS:
 	case C_GB2312:
+	case C_GBK:
 	case C_BIG5:
 	case C_UTF_8:
 	case C_UTF_7:
@@ -1437,36 +1589,48 @@ gboolean conv_is_multibyte_encoding(CharSet encoding)
 
 const gchar *conv_get_current_locale(void)
 {
-	gchar *cur_locale;
+	static const gchar *cur_locale;
 
-	cur_locale = g_getenv("LC_ALL");
-	if (!cur_locale) cur_locale = g_getenv("LC_CTYPE");
-	if (!cur_locale) cur_locale = g_getenv("LANG");
-	if (!cur_locale) cur_locale = setlocale(LC_CTYPE, NULL);
+	if (!cur_locale) {
+		cur_locale = g_getenv("LC_ALL");
+		if (!cur_locale) cur_locale = g_getenv("LC_CTYPE");
+		if (!cur_locale) cur_locale = g_getenv("LANG");
+		if (!cur_locale) cur_locale = setlocale(LC_CTYPE, NULL);
 
-	debug_print("current locale: %s\n",
-		    cur_locale ? cur_locale : "(none)");
+		debug_print("current locale: %s\n",
+			    cur_locale ? cur_locale : "(none)");
+	}
 
 	return cur_locale;
 }
 
-void conv_unmime_header(gchar *outbuf, gint outlen, const gchar *str,
-			const gchar *charset)
+gchar *conv_unmime_header(const gchar *str, const gchar *default_encoding)
 {
-	CharSet cur_charset;
+	gchar *buf;
+	gchar *decoded_str;
 
-	cur_charset = conv_get_current_charset();
+	if (is_ascii_str(str))
+		return unmime_header(str);
 
-	if (cur_charset == C_EUC_JP) {
-		gchar *buf;
-		gint buflen;
+	if (default_encoding) {
+		buf = conv_codeset_strdup
+			(str, default_encoding, CS_INTERNAL);
+		if (buf) {
+			decoded_str = unmime_header(buf);
+			g_free(buf);
+			return decoded_str;
+		}
+	}
 
-		buflen = strlen(str) * 2 + 1;
-		Xalloca(buf, buflen, return);
-		conv_anytodisp(buf, buflen, str);
-		unmime_header(outbuf, outlen, buf);
-	} else
-		unmime_header(outbuf, outlen, str);
+	if (conv_get_locale_charset() == C_EUC_JP)
+		buf = conv_anytodisp(str, NULL);
+	else
+		buf = conv_localetodisp(str, NULL);
+
+	decoded_str = unmime_header(buf);
+	g_free(buf);
+
+	return decoded_str;
 }
 
 #define MAX_LINELEN		76
@@ -1499,10 +1663,10 @@ void conv_unmime_header(gchar *outbuf, gint outlen, const gchar *str,
 }
 
 void conv_encode_header(gchar *dest, gint len, const gchar *src,
-			gint header_len, gboolean addr_field)
+			gint header_len, gboolean addr_field,
+			const gchar *out_encoding)
 {
 	const gchar *cur_encoding;
-	const gchar *out_encoding;
 	gint mimestr_len;
 	gchar *mimesep_enc;
 	gint left;
@@ -1510,6 +1674,8 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 	guchar *destp = dest;
 	gboolean use_base64;
 
+	g_return_if_fail(g_utf8_validate(src, -1, NULL) == TRUE);
+
 	if (MB_CUR_MAX > 1) {
 		use_base64 = TRUE;
 		mimesep_enc = "?B?";
@@ -1518,10 +1684,9 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 		mimesep_enc = "?Q?";
 	}
 
-	cur_encoding = conv_get_current_charset_str();
-	if (!strcmp(cur_encoding, CS_US_ASCII))
-		cur_encoding = CS_ISO_8859_1;
-	out_encoding = conv_get_outgoing_charset_str();
+	cur_encoding = CS_INTERNAL;
+	if (!out_encoding)
+		out_encoding = conv_get_outgoing_charset_str();
 	if (!strcmp(out_encoding, CS_US_ASCII))
 		out_encoding = CS_ISO_8859_1;
 
@@ -1582,14 +1747,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				if (addr_field && (*p == '(' || *p == ')'))
 					break;
 
-				if (MB_CUR_MAX > 1) {
-					mb_len = mblen(p, MB_CUR_MAX);
-					if (mb_len < 0) {
-						g_warning("conv_encode_header(): invalid multibyte character encountered\n");
-						mb_len = 1;
-					}
-				} else
-					mb_len = 1;
+				mb_len = g_utf8_skip[*p];
 
 				Xstrndup_a(part_str, srcp, cur_len + mb_len, );
 				out_str = conv_codeset_strdup
@@ -1668,3 +1826,126 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 }
 
 #undef LBREAK_IF_REQUIRED
+
+gint conv_copy_file(const gchar *src, const gchar *dest, const gchar *encoding)
+{
+	FILE *src_fp, *dest_fp;
+	gchar buf[BUFFSIZE];
+	CodeConverter *conv;
+	gboolean err = FALSE;
+
+	if ((src_fp = fopen(src, "rb")) == NULL) {
+		FILE_OP_ERROR(src, "fopen");
+		return -1;
+	}
+	if ((dest_fp = fopen(dest, "wb")) == NULL) {
+		FILE_OP_ERROR(dest, "fopen");
+		fclose(src_fp);
+		return -1;
+	}
+
+	if (change_file_mode_rw(dest_fp, dest) < 0) {
+		FILE_OP_ERROR(dest, "chmod");
+		g_warning("can't change file mode\n");
+	}
+
+	conv = conv_code_converter_new(encoding, NULL);
+
+	while (fgets(buf, sizeof(buf), src_fp) != NULL) {
+		gchar *outbuf;
+
+		outbuf = conv_convert(conv, buf);
+		if (outbuf) {
+			fputs(outbuf, dest_fp);
+			g_free(outbuf);
+		} else
+			fputs(buf, dest_fp);
+	}
+
+	conv_code_converter_destroy(conv);
+
+	if (ferror(src_fp)) {
+		FILE_OP_ERROR(src, "fgets");
+		err = TRUE;
+	}
+	fclose(src_fp);
+	if (fclose(dest_fp) == EOF) {
+		FILE_OP_ERROR(dest, "fclose");
+		err = TRUE;
+	}
+	if (err) {
+		unlink(dest);
+		return -1;
+	}
+
+	return 0;
+}
+
+gint conv_copy_dir(const gchar *src, const gchar *dest, const gchar *encoding)
+{
+	DIR *dp;
+	struct dirent *d;
+	gchar *src_file;
+	gchar *dest_file;
+
+	if ((dp = opendir(src)) == NULL) {
+		FILE_OP_ERROR(src, "opendir");
+		return -1;
+	}
+
+	if (make_dir_hier(dest) < 0) {
+		closedir(dp);
+		return -1;
+	}
+
+	while ((d = readdir(dp)) != NULL) {
+		if (!strcmp(d->d_name, ".") || !strcmp(d->d_name, ".."))
+			continue;
+
+		src_file = g_strconcat(src, G_DIR_SEPARATOR_S, d->d_name, NULL);
+		dest_file = g_strconcat(dest, G_DIR_SEPARATOR_S, d->d_name,
+					NULL);
+		if (is_file_exist(src_file))
+			conv_copy_file(src_file, dest_file, encoding);
+		g_free(dest_file);
+		g_free(src_file);
+	}
+
+	closedir(dp);
+
+	return 0;
+}
+
+gchar *conv_filename_from_utf8(const gchar *utf8_file)
+{
+	gchar *fs_file;
+	GError *error = NULL;
+
+	fs_file = g_filename_from_utf8(utf8_file, -1, NULL, NULL, &error);
+	if (error) {
+		g_warning("failed to convert encoding of file name: %s\n",
+			  error->message);
+		g_error_free(error);
+	}
+	if (!fs_file)
+		fs_file = g_strdup(utf8_file);
+
+	return fs_file;
+}
+
+gchar *conv_filename_to_utf8(const gchar *fs_file)
+{
+	gchar *utf8_file;
+	GError *error = NULL;
+
+	utf8_file = g_filename_to_utf8(fs_file, -1, NULL, NULL, &error);
+	if (error) {
+		g_warning("failed to convert encoding of file name: %s\n",
+			  error->message);
+		g_error_free(error);
+	}
+	if (!utf8_file)
+		utf8_file = g_strdup(fs_file);
+
+	return utf8_file;
+}
