@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,11 +24,11 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
-#include "intl.h"
 #include "folder.h"
 #include "session.h"
 #include "imap.h"
@@ -60,13 +60,13 @@ Folder *folder_new(FolderType type, const gchar *name, const gchar *path)
 	name = name ? name : path;
 	switch (type) {
 	case F_MH:
-		folder = mh_folder_new(name, path);
+		folder = mh_get_class()->folder_new(name, path);
 		break;
 	case F_IMAP:
-		folder = imap_folder_new(name, path);
+		folder = imap_get_class()->folder_new(name, path);
 		break;
 	case F_NEWS:
-		folder = news_folder_new(name, path);
+		folder = news_get_class()->folder_new(name, path);
 		break;
 	default:
 		return NULL;
@@ -82,7 +82,6 @@ static void folder_init(Folder *folder, const gchar *name)
 	g_return_if_fail(folder != NULL);
 
 	folder_set_name(folder, name);
-	folder->type = F_UNKNOWN;
 	folder->account = NULL;
 	folder->inbox = NULL;
 	folder->outbox = NULL;
@@ -93,7 +92,7 @@ static void folder_init(Folder *folder, const gchar *name)
 	folder->ui_func_data = NULL;
 	item = folder_item_new(name, NULL);
 	item->folder = folder;
-	folder->node = g_node_new(item);
+	folder->node = item->node = g_node_new(item);
 	folder->data = NULL;
 }
 
@@ -114,20 +113,9 @@ void folder_remote_folder_init(Folder *folder, const gchar *name,
 void folder_destroy(Folder *folder)
 {
 	g_return_if_fail(folder != NULL);
+	g_return_if_fail(folder->klass->destroy != NULL);
 
-	switch (folder->type) {
-	case F_MH:
-		mh_folder_destroy(MH_FOLDER(folder));
-		break;
-	case F_IMAP:
-		imap_folder_destroy(IMAP_FOLDER(folder));
-		break;
-	case F_NEWS:
-		news_folder_destroy(NEWS_FOLDER(folder));
-		break;
-	default:
-		break;
-	}
+	folder->klass->destroy(folder);
 
 	folder_list = g_list_remove(folder_list, folder);
 
@@ -174,19 +162,27 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->stype = F_NORMAL;
 	item->name = g_strdup(name);
 	item->path = g_strdup(path);
-	item->account = NULL;
 	item->mtime = 0;
 	item->new = 0;
 	item->unread = 0;
 	item->total = 0;
+	item->unmarked_num = 0;
 	item->last_num = -1;
 	item->no_sub = FALSE;
 	item->no_select = FALSE;
 	item->collapsed = FALSE;
-	item->threaded  = TRUE;
-	item->opened    = FALSE;
+	item->threaded = TRUE;
+	item->opened = FALSE;
+	item->node = NULL;
 	item->parent = NULL;
 	item->folder = NULL;
+	item->account = NULL;
+	item->ac_apply_sub = FALSE;
+	item->auto_to = NULL;
+	item->use_auto_to_on_reply = FALSE;
+	item->auto_cc = NULL;
+	item->auto_bcc = NULL;
+	item->auto_replyto = NULL;
 	item->mark_queue = NULL;
 	item->data = NULL;
 
@@ -195,19 +191,22 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 
 void folder_item_append(FolderItem *parent, FolderItem *item)
 {
-	GNode *node;
-
 	g_return_if_fail(parent != NULL);
 	g_return_if_fail(parent->folder != NULL);
+	g_return_if_fail(parent->node != NULL);
 	g_return_if_fail(item != NULL);
 
-	node = parent->folder->node;
-	node = g_node_find(node, G_PRE_ORDER, G_TRAVERSE_ALL, parent);
-	g_return_if_fail(node != NULL);
-
 	item->parent = parent;
 	item->folder = parent->folder;
-	g_node_append_data(node, item);
+	item->node = g_node_append_data(parent->node, item);
+}
+
+static gboolean folder_item_remove_func(GNode *node, gpointer data)
+{
+	FolderItem *item = FOLDER_ITEM(node->data);
+
+	folder_item_destroy(item);
+	return FALSE;
 }
 
 void folder_item_remove(FolderItem *item)
@@ -216,26 +215,101 @@ void folder_item_remove(FolderItem *item)
 
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
+	g_return_if_fail(item->node != NULL);
 
-	node = item->folder->node;
-	node = g_node_find(node, G_PRE_ORDER, G_TRAVERSE_ALL, item);
-	g_return_if_fail(node != NULL);
+	node = item->node;
 
-	/* TODO: free all FolderItem's first */
 	if (item->folder->node == node)
 		item->folder->node = NULL;
+
+	g_node_traverse(node, G_POST_ORDER, G_TRAVERSE_ALL, -1,
+			folder_item_remove_func, NULL);
 	g_node_destroy(node);
 }
 
+void folder_item_remove_children(FolderItem *item)
+{
+	GNode *node, *next;
+
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	g_return_if_fail(item->node != NULL);
+
+	node = item->node->children;
+	while (node != NULL) {
+		next = node->next;
+		folder_item_remove(FOLDER_ITEM(node->data));
+		node = next;
+	}
+}
+
 void folder_item_destroy(FolderItem *item)
 {
+	Folder *folder;
+
 	g_return_if_fail(item != NULL);
 
+	folder = item->folder;
+	if (folder) {
+		if (folder->inbox == item)
+			folder->inbox = NULL;
+		else if (folder->outbox == item)
+			folder->outbox = NULL;
+		else if (folder->draft == item)
+			folder->draft = NULL;
+		else if (folder->queue == item)
+			folder->queue = NULL;
+		else if (folder->trash == item)
+			folder->trash = NULL;
+	}
+
 	g_free(item->name);
 	g_free(item->path);
+	g_free(item->auto_to);
+	g_free(item->auto_cc);
+	g_free(item->auto_bcc);
+	g_free(item->auto_replyto);
 	g_free(item);
 }
 
+gint folder_item_compare(FolderItem *item_a, FolderItem *item_b)
+{
+	gint ret;
+	gchar *str_a, *str_b;
+
+	if (!item_a || !item_b)
+		return 0;
+	if (!item_a->parent || !item_b->parent)
+		return 0;
+	if (!item_a->name || !item_b->name)
+		return 0;
+
+	/* if both a and b are special folders, sort them according to
+	 * their types (which is in-order). Note that this assumes that
+	 * there are no multiple folders of a special type. */
+	if (item_a->stype != F_NORMAL && item_b->stype != F_NORMAL)
+		return item_a->stype - item_b->stype;
+
+	 /* if b is normal folder, and a is not, b is smaller (ends up
+	  * lower in the list) */
+	if (item_a->stype != F_NORMAL && item_b->stype == F_NORMAL)
+		return item_b->stype - item_a->stype;
+
+	/* if b is special folder, and a is not, b is larger (ends up
+	 * higher in the list) */
+	if (item_a->stype == F_NORMAL && item_b->stype != F_NORMAL)
+		return item_b->stype - item_a->stype;
+
+	/* otherwise just compare the folder names */
+	str_a = g_utf8_casefold(item_a->name, -1);
+	str_b = g_utf8_casefold(item_b->name, -1);
+	ret = g_utf8_collate(str_a, str_b);
+	g_free(str_b);
+	g_free(str_a);
+
+	return ret;
+}
+
 void folder_set_ui_func(Folder *folder, FolderUIFunc func, gpointer data)
 {
 	g_return_if_fail(folder != NULL);
@@ -260,15 +334,10 @@ void folder_set_name(Folder *folder, const gchar *name)
 
 void folder_tree_destroy(Folder *folder)
 {
-	/* TODO: destroy all FolderItem before */
-	g_node_destroy(folder->node);
+	g_return_if_fail(folder != NULL);
 
-	folder->inbox = NULL;
-	folder->outbox = NULL;
-	folder->draft = NULL;
-	folder->queue = NULL;
-	folder->trash = NULL;
-	folder->node = NULL;
+	if (folder->node)
+		folder_item_remove(FOLDER_ITEM(folder->node->data));
 }
 
 void folder_add(Folder *folder)
@@ -281,15 +350,15 @@ void folder_add(Folder *folder)
 
 	for (i = 0, cur = folder_list; cur != NULL; cur = cur->next, i++) {
 		cur_folder = FOLDER(cur->data);
-		if (folder->type == F_MH) {
-			if (cur_folder->type != F_MH) break;
-		} else if (folder->type == F_IMAP) {
-			if (cur_folder->type != F_MH &&
-			    cur_folder->type != F_IMAP) break;
-		} else if (folder->type == F_NEWS) {
-			if (cur_folder->type != F_MH &&
-			    cur_folder->type != F_IMAP &&
-			    cur_folder->type != F_NEWS) break;
+		if (FOLDER_TYPE(folder) == F_MH) {
+			if (FOLDER_TYPE(cur_folder) != F_MH) break;
+		} else if (FOLDER_TYPE(folder) == F_IMAP) {
+			if (FOLDER_TYPE(cur_folder) != F_MH &&
+			    FOLDER_TYPE(cur_folder) != F_IMAP) break;
+		} else if (FOLDER_TYPE(folder) == F_NEWS) {
+			if (FOLDER_TYPE(cur_folder) != F_MH &&
+			    FOLDER_TYPE(cur_folder) != F_IMAP &&
+			    FOLDER_TYPE(cur_folder) != F_NEWS) break;
 		}
 	}
 
@@ -337,10 +406,10 @@ void folder_write_list(void)
 	PrefFile *pfile;
 
 	path = folder_get_list_path();
-	if ((pfile = prefs_write_open(path)) == NULL) return;
+	if ((pfile = prefs_file_open(path)) == NULL) return;
 
 	fprintf(pfile->fp, "<?xml version=\"1.0\" encoding=\"%s\"?>\n",
-		conv_get_current_charset_str());
+		CS_INTERNAL);
 	fputs("\n<folderlist>\n", pfile->fp);
 
 	for (list = folder_list; list != NULL; list = list->next) {
@@ -350,56 +419,115 @@ void folder_write_list(void)
 
 	fputs("</folderlist>\n", pfile->fp);
 
-	if (prefs_write_close(pfile) < 0)
+	if (prefs_file_close(pfile) < 0)
 		g_warning("failed to write folder list.\n");
 }
 
-struct TotalMsgCount
+struct TotalMsgStatus
 {
 	guint new;
 	guint unread;
 	guint total;
+	GString *str;
 };
 
-static gboolean folder_count_total_msgs_func(GNode *node, gpointer data)
+static gboolean folder_get_status_full_all_func(GNode *node, gpointer data)
 {
 	FolderItem *item;
-	struct TotalMsgCount *count = (struct TotalMsgCount *)data;
+	struct TotalMsgStatus *status = (struct TotalMsgStatus *)data;
+	gchar *id;
 
 	g_return_val_if_fail(node->data != NULL, FALSE);
 
 	item = FOLDER_ITEM(node->data);
-	count->new += item->new;
-	count->unread += item->unread;
-	count->total += item->total;
+
+	if (!item->path) return FALSE;
+
+	status->new += item->new;
+	status->unread += item->unread;
+	status->total += item->total;
+
+	if (status->str) {
+		id = folder_item_get_identifier(item);
+		g_string_sprintfa(status->str, "%5d %5d %5d %s\n",
+				  item->new, item->unread,
+				  item->total, id);
+		g_free(id);
+	}
 
 	return FALSE;
 }
 
-void folder_count_total_msgs(guint *new, guint *unread, guint *total)
+static void folder_get_status_full_all(GString *str, guint *new, guint *unread,
+				       guint *total)
 {
 	GList *list;
 	Folder *folder;
-	struct TotalMsgCount count;
+	struct TotalMsgStatus status;
 
-	count.new = count.unread = count.total = 0;
+	status.new = status.unread = status.total = 0;
+	status.str = str;
 
-	debug_print(_("Counting total number of messages...\n"));
+	debug_print("Counting total number of messages...\n");
 
 	for (list = folder_list; list != NULL; list = list->next) {
 		folder = FOLDER(list->data);
 		if (folder->node)
 			g_node_traverse(folder->node, G_PRE_ORDER,
 					G_TRAVERSE_ALL, -1,
-					folder_count_total_msgs_func,
-					&count);
+					folder_get_status_full_all_func,
+					&status);
 	}
 
-	*new = count.new;
-	*unread = count.unread;
-	*total = count.total;
+	*new = status.new;
+	*unread = status.unread;
+	*total = status.total;
+}
 
-	return;
+gchar *folder_get_status(GPtrArray *folders, gboolean full)
+{
+	guint new, unread, total;
+	GString *str;
+	gint i;
+	gchar *ret;
+
+	new = unread = total = 0;
+
+	str = g_string_new(NULL);
+
+	if (folders) {
+		for (i = 0; i < folders->len; i++) {
+			FolderItem *item;
+
+			item = g_ptr_array_index(folders, i);
+			new += item->new;
+			unread += item->unread;
+			total += item->total;
+
+			if (full) {
+				gchar *id;
+
+				id = folder_item_get_identifier(item);
+				g_string_sprintfa(str, "%5d %5d %5d %s\n",
+						  item->new, item->unread,
+						  item->total, id);
+				g_free(id);
+			}
+		}
+	} else {
+		folder_get_status_full_all(full ? str : NULL,
+					   &new, &unread, &total);
+	}
+
+	if (full)
+		g_string_sprintfa(str, "%5d %5d %5d\n", new, unread, total);
+	else
+		g_string_sprintfa(str, "%d %d %d\n", new, unread, total);
+
+	ret = str->str;
+	g_string_free(str, FALSE);
+
+	return ret;
 }
 
 Folder *folder_find_from_path(const gchar *path)
@@ -409,7 +537,7 @@ Folder *folder_find_from_path(const gchar *path)
 
 	for (list = folder_list; list != NULL; list = list->next) {
 		folder = list->data;
-		if (folder->type == F_MH &&
+		if (FOLDER_TYPE(folder) == F_MH &&
 		    !path_cmp(LOCAL_FOLDER(folder)->rootpath, path))
 			return folder;
 	}
@@ -424,7 +552,8 @@ Folder *folder_find_from_name(const gchar *name, FolderType type)
 
 	for (list = folder_list; list != NULL; list = list->next) {
 		folder = list->data;
-		if (folder->type == type && strcmp2(name, folder->name) == 0)
+		if (FOLDER_TYPE(folder) == type &&
+		    strcmp2(name, folder->name) == 0)
 			return folder;
 	}
 
@@ -460,6 +589,20 @@ FolderItem *folder_find_item_from_path(const gchar *path)
 	return d[1];
 }
 
+FolderItem *folder_find_child_item_by_name(FolderItem *item, const gchar *name)
+{
+	GNode *node;
+	FolderItem *child;
+
+	for (node = item->node->children; node != NULL; node = node->next) {
+		child = FOLDER_ITEM(node->data);
+		if (strcmp2(g_basename(child->path), name) == 0)
+			return child;
+	}
+
+	return NULL;
+}
+
 static const struct {
 	gchar *str;
 	FolderType type;
@@ -503,7 +646,7 @@ gchar *folder_get_identifier(Folder *folder)
 
 	g_return_val_if_fail(folder != NULL, NULL);
 
-	type_str = folder_get_type_string(folder->type);
+	type_str = folder_get_type_string(FOLDER_TYPE(folder));
 	return g_strconcat(type_str, "/", folder->name, NULL);
 }
 
@@ -642,7 +785,7 @@ void folder_set_missing_folders(void)
 
 	for (list = folder_list; list != NULL; list = list->next) {
 		folder = list->data;
-		if (folder->type != F_MH) continue;
+		if (FOLDER_TYPE(folder) != F_MH) continue;
 		rootitem = FOLDER_ITEM(folder->node->data);
 		g_return_if_fail(rootitem != NULL);
 
@@ -650,7 +793,7 @@ void folder_set_missing_folders(void)
 		    folder->queue && folder->trash)
 			continue;
 
-		if (folder->create_tree(folder) < 0) {
+		if (folder->klass->create_tree(folder) < 0) {
 			g_warning("%s: can't create the folder tree.\n",
 				  LOCAL_FOLDER(folder)->rootpath);
 			continue;
@@ -664,52 +807,104 @@ void folder_set_missing_folders(void)
 	}
 }
 
+static gboolean folder_unref_account_func(GNode *node, gpointer data)
+{
+	FolderItem *item = node->data;
+	PrefsAccount *account = data;
+
+	if (item->account == account)
+		item->account = NULL;
+
+	return FALSE;
+}
+
+void folder_unref_account_all(PrefsAccount *account)
+{
+	Folder *folder;
+	GList *list;
+
+	if (!account) return;
+
+	for (list = folder_list; list != NULL; list = list->next) {
+		folder = list->data;
+		if (folder->account == account)
+			folder->account = NULL;
+		g_node_traverse(folder->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+				folder_unref_account_func, account);
+	}
+}
+
 #undef CREATE_FOLDER_IF_NOT_EXIST
 
+gchar *folder_get_path(Folder *folder)
+{
+	gchar *path;
+
+	g_return_val_if_fail(folder != NULL, NULL);
+
+	if (FOLDER_TYPE(folder) == F_MH) {
+		path = g_filename_from_utf8(LOCAL_FOLDER(folder)->rootpath,
+					    -1, NULL, NULL, NULL);
+		if (!path) {
+			g_warning("folder_get_path: faild to convert character set\n");
+			path = g_strdup(LOCAL_FOLDER(folder)->rootpath);
+		}
+	} else if (FOLDER_TYPE(folder) == F_IMAP) {
+		g_return_val_if_fail(folder->account != NULL, NULL);
+		path = g_strconcat(get_imap_cache_dir(),
+				   G_DIR_SEPARATOR_S,
+				   folder->account->recv_server,
+				   G_DIR_SEPARATOR_S,
+				   folder->account->userid,
+				   NULL);
+	} else if (FOLDER_TYPE(folder) == F_NEWS) {
+		g_return_val_if_fail(folder->account != NULL, NULL);
+		path = g_strconcat(get_news_cache_dir(),
+				   G_DIR_SEPARATOR_S,
+				   folder->account->nntp_server,
+				   NULL);
+	} else
+		path = NULL;
+
+	return path;
+}
+
 gchar *folder_item_get_path(FolderItem *item)
 {
 	gchar *folder_path;
-	gchar *path;
+	gchar *item_path = NULL, *path;
 
 	g_return_val_if_fail(item != NULL, NULL);
 
-	if (FOLDER_TYPE(item->folder) == F_MH)
-		folder_path = g_strdup(LOCAL_FOLDER(item->folder)->rootpath);
-	else if (FOLDER_TYPE(item->folder) == F_IMAP) {
-		g_return_val_if_fail(item->folder->account != NULL, NULL);
-		folder_path = g_strconcat(get_imap_cache_dir(),
-					  G_DIR_SEPARATOR_S,
-					  item->folder->account->recv_server,
-					  G_DIR_SEPARATOR_S,
-					  item->folder->account->userid,
-					  NULL);
-	} else if (FOLDER_TYPE(item->folder) == F_NEWS) {
-		g_return_val_if_fail(item->folder->account != NULL, NULL);
-		folder_path = g_strconcat(get_news_cache_dir(),
-					  G_DIR_SEPARATOR_S,
-					  item->folder->account->nntp_server,
-					  NULL);
-	} else
-		return NULL;
-
+	folder_path = folder_get_path(item->folder);
 	g_return_val_if_fail(folder_path != NULL, NULL);
 
+	if (item->path) {
+		item_path = g_filename_from_utf8(item->path, -1,
+						 NULL, NULL, NULL);
+		if (!item_path) {
+			g_warning("folder_item_get_path: faild to convert character set\n");
+			item_path = g_strdup(item->path);
+		}
+	}
+
 	if (folder_path[0] == G_DIR_SEPARATOR) {
-		if (item->path)
+		if (item_path)
 			path = g_strconcat(folder_path, G_DIR_SEPARATOR_S,
-					   item->path, NULL);
+					   item_path, NULL);
 		else
 			path = g_strdup(folder_path);
 	} else {
-		if (item->path)
+		if (item_path)
 			path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 					   folder_path, G_DIR_SEPARATOR_S,
-					   item->path, NULL);
+					   item_path, NULL);
 		else
 			path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
 					   folder_path, NULL);
 	}
 
+	g_free(item_path);
 	g_free(folder_path);
 	return path;
 }
@@ -721,7 +916,7 @@ gint folder_item_scan(FolderItem *item)
 	g_return_val_if_fail(item != NULL, -1);
 
 	folder = item->folder;
-	return folder->scan(folder, item);
+	return folder->klass->scan(folder, item);
 }
 
 static void folder_item_scan_foreach_func(gpointer key, gpointer val,
@@ -742,7 +937,7 @@ GSList *folder_item_get_msg_list(FolderItem *item, gboolean use_cache)
 	g_return_val_if_fail(item != NULL, NULL);
 
 	folder = item->folder;
-	return folder->get_msg_list(folder, item, use_cache);
+	return folder->klass->get_msg_list(folder, item, use_cache);
 }
 
 gchar *folder_item_fetch_msg(FolderItem *item, gint num)
@@ -752,125 +947,176 @@ gchar *folder_item_fetch_msg(FolderItem *item, gint num)
 	g_return_val_if_fail(item != NULL, NULL);
 
 	folder = item->folder;
-	if (item->last_num < 0) folder->scan(folder, item);
 
-	return folder->fetch_msg(folder, item, num);
+	return folder->klass->fetch_msg(folder, item, num);
 }
 
-gint folder_item_add_msg(FolderItem *dest, const gchar *file,
+gint folder_item_fetch_all_msg(FolderItem *item)
+{
+	Folder *folder;
+	GSList *mlist;
+	GSList *cur;
+	gint num = 0;
+	gint ret = 0;
+
+	g_return_val_if_fail(item != NULL, -1);
+
+	debug_print("fetching all messages in %s ...\n", item->path);
+
+	folder = item->folder;
+
+	if (folder->ui_func)
+		folder->ui_func(folder, item, folder->ui_func_data ?
+				folder->ui_func_data : GINT_TO_POINTER(num));
+
+	mlist = folder_item_get_msg_list(item, TRUE);
+
+	for (cur = mlist; cur != NULL; cur = cur->next) {
+		MsgInfo *msginfo = (MsgInfo *)cur->data;
+		gchar *msg;
+
+		num++;
+		if (folder->ui_func)
+			folder->ui_func(folder, item,
+					folder->ui_func_data ?
+					folder->ui_func_data :
+					GINT_TO_POINTER(num));
+
+		msg = folder_item_fetch_msg(item, msginfo->msgnum);
+		if (!msg) {
+			g_warning("Can't fetch message %d. Aborting.\n",
+				  msginfo->msgnum);
+			ret = -1;
+			break;
+		}
+		g_free(msg);
+	}
+
+	procmsg_msg_list_free(mlist);
+
+	return ret;
+}
+
+MsgInfo *folder_item_get_msginfo(FolderItem *item, gint num)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(item != NULL, NULL);
+
+	folder = item->folder;
+
+	return folder->klass->get_msginfo(folder, item, num);
+}
+
+gint folder_item_add_msg(FolderItem *dest, const gchar *file, MsgFlags *flags,
 			 gboolean remove_source)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(file != NULL, -1);
-	g_return_val_if_fail(dest->folder->add_msg != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->add_msg != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->add_msg(folder, dest, file, remove_source);
-	if (num > 0) dest->last_num = num;
+	return folder->klass->add_msg(folder, dest, file, flags, remove_source);
+}
 
-	return num;
+gint folder_item_add_msgs(FolderItem *dest, GSList *file_list,
+			  gboolean remove_source, gint *first)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(file_list != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->add_msgs != NULL, -1);
+
+	folder = dest->folder;
+
+	return folder->klass->add_msgs(folder, dest, file_list, remove_source,
+				       first);
 }
 
 gint folder_item_move_msg(FolderItem *dest, MsgInfo *msginfo)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msginfo != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->move_msg != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->move_msg(folder, dest, msginfo);
-	if (num > 0) dest->last_num = num;
-
-	return num;
+	return folder->klass->move_msg(folder, dest, msginfo);
 }
 
-gint folder_item_move_msgs_with_dest(FolderItem *dest, GSList *msglist)
+gint folder_item_move_msgs(FolderItem *dest, GSList *msglist)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msglist != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->move_msgs != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->move_msgs_with_dest(folder, dest, msglist);
-	if (num > 0) dest->last_num = num;
-
-	return num;
+	return folder->klass->move_msgs(folder, dest, msglist);
 }
 
 gint folder_item_copy_msg(FolderItem *dest, MsgInfo *msginfo)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msginfo != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->copy_msg != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->copy_msg(folder, dest, msginfo);
-	if (num > 0) dest->last_num = num;
-
-	return num;
+	return folder->klass->copy_msg(folder, dest, msginfo);
 }
 
-gint folder_item_copy_msgs_with_dest(FolderItem *dest, GSList *msglist)
+gint folder_item_copy_msgs(FolderItem *dest, GSList *msglist)
 {
 	Folder *folder;
-	gint num;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msglist != NULL, -1);
+	g_return_val_if_fail(dest->folder->klass->copy_msgs != NULL, -1);
 
 	folder = dest->folder;
-	if (dest->last_num < 0) folder->scan(folder, dest);
 
-	num = folder->copy_msgs_with_dest(folder, dest, msglist);
-	if (num > 0) dest->last_num = num;
-
-	return num;
+	return folder->klass->copy_msgs(folder, dest, msglist);
 }
 
-gint folder_item_remove_msg(FolderItem *item, gint num)
+gint folder_item_remove_msg(FolderItem *item, MsgInfo *msginfo)
 {
 	Folder *folder;
-	gint ret;
 
 	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(item->folder->klass->remove_msg != NULL, -1);
 
 	folder = item->folder;
-	if (item->last_num < 0) folder->scan(folder, item);
 
-	ret = folder->remove_msg(folder, item, num);
-	if (ret == 0 && num == item->last_num)
-		folder->scan(folder, item);
-
-	return ret;
+	return folder->klass->remove_msg(folder, item, msginfo);
 }
 
 gint folder_item_remove_msgs(FolderItem *item, GSList *msglist)
 {
+	Folder *folder;
 	gint ret = 0;
 
 	g_return_val_if_fail(item != NULL, -1);
 
+	folder = item->folder;
+	if (folder->klass->remove_msgs) {
+		return folder->klass->remove_msgs(folder, item, msglist);
+	}
+
 	while (msglist != NULL) {
 		MsgInfo *msginfo = (MsgInfo *)msglist->data;
 
-		ret = folder_item_remove_msg(item, msginfo->msgnum);
+		ret = folder_item_remove_msg(item, msginfo);
 		if (ret != 0) break;
 		msglist = msglist->next;
 	}
@@ -883,11 +1129,11 @@ gint folder_item_remove_all_msg(FolderItem *item)
 	Folder *folder;
 
 	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(item->folder->klass->remove_all_msg != NULL, -1);
 
 	folder = item->folder;
-	if (item->last_num < 0) folder->scan(folder, item);
 
-	return folder->remove_all_msg(folder, item);
+	return folder->klass->remove_all_msg(folder, item);
 }
 
 gboolean folder_item_is_msg_changed(FolderItem *item, MsgInfo *msginfo)
@@ -895,9 +1141,22 @@ gboolean folder_item_is_msg_changed(FolderItem *item, MsgInfo *msginfo)
 	Folder *folder;
 
 	g_return_val_if_fail(item != NULL, FALSE);
+	g_return_val_if_fail(item->folder->klass->is_msg_changed != NULL,
+			     FALSE);
 
 	folder = item->folder;
-	return folder->is_msg_changed(folder, item, msginfo);
+	return folder->klass->is_msg_changed(folder, item, msginfo);
+}
+
+gint folder_item_close(FolderItem *item)
+{
+	Folder *folder;
+
+	g_return_val_if_fail(item != NULL, -1);
+
+	item->opened = FALSE;
+	folder = item->folder;
+	return folder->klass->close(folder, item);
 }
 
 gchar *folder_item_get_cache_file(FolderItem *item)
@@ -947,11 +1206,15 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	const gchar *path = NULL;
 	PrefsAccount *account = NULL;
 	gboolean no_sub = FALSE, no_select = FALSE, collapsed = FALSE,
-		 threaded = TRUE;
+		 threaded = TRUE, ac_apply_sub = FALSE;
 	FolderSortKey sort_key = SORT_BY_NONE;
 	FolderSortType sort_type = SORT_ASCENDING;
 	gint new = 0, unread = 0, total = 0;
 	time_t mtime = 0;
+	gboolean use_auto_to_on_reply = FALSE;
+	gchar *auto_to = NULL, *auto_cc = NULL, *auto_bcc = NULL,
+	      *auto_replyto = NULL;
+	gboolean trim_summary_subject = FALSE, trim_compose_subject = FALSE;
 
 	g_return_val_if_fail(node->data != NULL, FALSE);
 	if (!node->parent) return FALSE;
@@ -984,11 +1247,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 			name = attr->value;
 		else if (!strcmp(attr->name, "path"))
 			path = attr->value;
-		else if (!strcmp(attr->name, "account_id")) {
-			account = account_find_from_id(atoi(attr->value));
-			if (!account) g_warning("account_id: %s not found\n",
-						attr->value);
-		} else if (!strcmp(attr->name, "mtime"))
+		else if (!strcmp(attr->name, "mtime"))
 			mtime = strtoul(attr->value, NULL, 10);
 		else if (!strcmp(attr->name, "new"))
 			new = atoi(attr->value);
@@ -1027,17 +1286,41 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 				sort_key = SORT_BY_UNREAD;
 			else if (!strcmp(attr->value, "mime"))
 				sort_key = SORT_BY_MIME;
+			else if (!strcmp(attr->value, "to"))
+				sort_key = SORT_BY_TO;
 		} else if (!strcmp(attr->name, "sort_type")) {
 			if (!strcmp(attr->value, "ascending"))
 				sort_type = SORT_ASCENDING;
 			else
 				sort_type = SORT_DESCENDING;
+		} else if (!strcmp(attr->name, "account_id")) {
+			account = account_find_from_id(atoi(attr->value));
+			if (!account) g_warning("account_id: %s not found\n",
+						attr->value);
+		} else if (!strcmp(attr->name, "account_apply_sub"))
+			ac_apply_sub = *attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "to"))
+			auto_to = g_strdup(attr->value);
+		else if (!strcmp(attr->name, "use_auto_to_on_reply"))
+			use_auto_to_on_reply =
+				*attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "cc"))
+			auto_cc = g_strdup(attr->value);
+		else if (!strcmp(attr->name, "bcc"))
+			auto_bcc = g_strdup(attr->value);
+		else if (!strcmp(attr->name, "replyto"))
+			auto_replyto = g_strdup(attr->value);
+		else if (!strcmp(attr->name, "trim_summary_subject")) {
+			trim_summary_subject =
+				*attr->value == '1' ? TRUE : FALSE;
+		} else if (!strcmp(attr->name, "trim_compose_subject")) {
+			trim_compose_subject =
+				*attr->value = '1' ? TRUE : FALSE;
 		}
 	}
 
 	item = folder_item_new(name, path);
 	item->stype = stype;
-	item->account = account;
 	item->mtime = mtime;
 	item->new = new;
 	item->unread = unread;
@@ -1048,6 +1331,7 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	item->threaded  = threaded;
 	item->sort_key  = sort_key;
 	item->sort_type = sort_type;
+	item->node = node;
 	item->parent = FOLDER_ITEM(node->parent->data);
 	item->folder = folder;
 	switch (stype) {
@@ -1058,6 +1342,15 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 	case F_TRASH:  folder->trash  = item; break;
 	default:       break;
 	}
+	item->account = account;
+	item->ac_apply_sub = ac_apply_sub;
+	item->auto_to = auto_to;
+	item->use_auto_to_on_reply = use_auto_to_on_reply;
+	item->auto_cc = auto_cc;
+	item->auto_bcc = auto_bcc;
+	item->auto_replyto = auto_replyto;
+	item->trim_summary_subject = trim_summary_subject;
+	item->trim_compose_subject = trim_compose_subject;
 	node->data = item;
 	xml_free_node(xmlnode);
 
@@ -1067,13 +1360,14 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 static gboolean folder_read_folder_func(GNode *node, gpointer data)
 {
 	Folder *folder;
+	FolderItem *item;
 	XMLNode *xmlnode;
 	GList *list;
 	FolderType type = F_UNKNOWN;
 	const gchar *name = NULL;
 	const gchar *path = NULL;
 	PrefsAccount *account = NULL;
-	gboolean collapsed = FALSE, threaded = TRUE;
+	gboolean collapsed = FALSE, threaded = TRUE, ac_apply_sub = FALSE;
 
 	if (g_node_depth(node) != 2) return FALSE;
 	g_return_val_if_fail(node->data != NULL, FALSE);
@@ -1104,14 +1398,16 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 			name = attr->value;
 		else if (!strcmp(attr->name, "path"))
 			path = attr->value;
+		else if (!strcmp(attr->name, "collapsed"))
+			collapsed = *attr->value == '1' ? TRUE : FALSE;
+		else if (!strcmp(attr->name, "threaded"))
+			threaded = *attr->value == '1' ? TRUE : FALSE;
 		else if (!strcmp(attr->name, "account_id")) {
 			account = account_find_from_id(atoi(attr->value));
 			if (!account) g_warning("account_id: %s not found\n",
 						attr->value);
-		} else if (!strcmp(attr->name, "collapsed"))
-			collapsed = *attr->value == '1' ? TRUE : FALSE;
-		else if (!strcmp(attr->name, "threaded"))
-			threaded = *attr->value == '1' ? TRUE : FALSE;
+		} else if (!strcmp(attr->name, "account_apply_sub"))
+			ac_apply_sub = *attr->value == '1' ? TRUE : FALSE;
 	}
 
 	folder = folder_new(type, name, path);
@@ -1119,12 +1415,16 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 	folder->account = account;
 	if (account && (type == F_IMAP || type == F_NEWS))
 		account->folder = REMOTE_FOLDER(folder);
-	node->data = folder->node->data;
+	item = FOLDER_ITEM(folder->node->data);
+	node->data = item;
+	item->node = node;
 	g_node_destroy(folder->node);
 	folder->node = node;
 	folder_add(folder);
-	FOLDER_ITEM(node->data)->collapsed = collapsed;
-	FOLDER_ITEM(node->data)->threaded  = threaded;
+	item->collapsed = collapsed;
+	item->threaded  = threaded;
+	item->account   = account;
+	item->ac_apply_sub = ac_apply_sub;
 
 	g_node_traverse(node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 			folder_build_tree, folder);
@@ -1143,10 +1443,17 @@ static gchar *folder_get_list_path(void)
 	return filename;
 }
 
+#define PUT_ESCAPE_STR(fp, attr, str)			\
+{							\
+	fputs(" " attr "=\"", fp);			\
+	xml_file_put_escape_str(fp, str);		\
+	fputs("\"", fp);				\
+}
+
 static void folder_write_list_recursive(GNode *node, gpointer data)
 {
 	FILE *fp = (FILE *)data;
-	FolderItem *item = FOLDER_ITEM(node->data);
+	FolderItem *item;
 	gint i, depth;
 	static gchar *folder_type_str[] = {"mh", "mbox", "maildir", "imap",
 					   "news", "unknown"};
@@ -1154,8 +1461,12 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 						 "draft", "queue", "trash"};
 	static gchar *sort_key_str[] = {"none", "number", "size", "date",
 					"from", "subject", "score", "label",
-					"mark", "unread", "mime"};
+					"mark", "unread", "mime", "to"};
 
+	g_return_if_fail(node != NULL);
+	g_return_if_fail(fp != NULL);
+
+	item = FOLDER_ITEM(node->data);
 	g_return_if_fail(item != NULL);
 
 	depth = g_node_depth(node);
@@ -1164,39 +1475,28 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 	if (depth == 1) {
 		Folder *folder = item->folder;
 
-		fprintf(fp, "<folder type=\"%s\"", folder_type_str[folder->type]);
-		if (folder->name) {
-			fputs(" name=\"", fp);
-			xml_file_put_escape_str(fp, folder->name);
-			fputs("\"", fp);
-		}
-		if (folder->type == F_MH) {
-			fputs(" path=\"", fp);
-			xml_file_put_escape_str
-				(fp, LOCAL_FOLDER(folder)->rootpath);
-			fputs("\"", fp);
-		}
+		fprintf(fp, "<folder type=\"%s\"",
+			folder_type_str[FOLDER_TYPE(folder)]);
+		if (folder->name)
+			PUT_ESCAPE_STR(fp, "name", folder->name);
+		if (FOLDER_TYPE(folder) == F_MH)
+			PUT_ESCAPE_STR(fp, "path",
+				       LOCAL_FOLDER(folder)->rootpath);
+		if (item->collapsed && node->children)
+			fputs(" collapsed=\"1\"", fp);
 		if (folder->account)
 			fprintf(fp, " account_id=\"%d\"",
 				folder->account->account_id);
-		if (item->collapsed && node->children)
-			fputs(" collapsed=\"1\"", fp);
+		if (item->ac_apply_sub)
+			fputs(" account_apply_sub=\"1\"", fp);
 	} else {
 		fprintf(fp, "<folderitem type=\"%s\"",
 			folder_item_stype_str[item->stype]);
-		if (item->name) {
-			fputs(" name=\"", fp);
-			xml_file_put_escape_str(fp, item->name);
-			fputs("\"", fp);
-		}
-		if (item->path) {
-			fputs(" path=\"", fp);
-			xml_file_put_escape_str(fp, item->path);
-			fputs("\"", fp);
-		}
-		if (item->account)
-			fprintf(fp, " account_id=\"%d\"",
-				item->account->account_id);
+		if (item->name)
+			PUT_ESCAPE_STR(fp, "name", item->name);
+		if (item->path)
+			PUT_ESCAPE_STR(fp, "path", item->path);
+
 		if (item->no_sub)
 			fputs(" no_sub=\"1\"", fp);
 		if (item->no_select)
@@ -1220,6 +1520,28 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 		fprintf(fp,
 			" mtime=\"%lu\" new=\"%d\" unread=\"%d\" total=\"%d\"",
 			item->mtime, item->new, item->unread, item->total);
+
+		if (item->account)
+			fprintf(fp, " account_id=\"%d\"",
+				item->account->account_id);
+		if (item->ac_apply_sub)
+			fputs(" account_apply_sub=\"1\"", fp);
+
+		if (item->auto_to)
+			PUT_ESCAPE_STR(fp, "to", item->auto_to);
+		if (item->use_auto_to_on_reply)
+			fputs(" use_auto_to_on_reply=\"1\"", fp);
+		if (item->auto_cc)
+			PUT_ESCAPE_STR(fp, "cc", item->auto_cc);
+		if (item->auto_bcc)
+			PUT_ESCAPE_STR(fp, "bcc", item->auto_bcc);
+		if (item->auto_replyto)
+			PUT_ESCAPE_STR(fp, "replyto", item->auto_replyto);
+
+		if (item->trim_summary_subject)
+			fputs(" trim_summary_subject=\"1\"", fp);
+		if (item->trim_compose_subject)
+			fputs(" trim_compose_subject=\"1\"", fp);
 	}
 
 	if (node->children) {
@@ -1241,3 +1563,5 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 	} else
 		fputs(" />\n", fp);
 }
+
+#undef PUT_ESCAPE_STR
