@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,19 +20,19 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtknotebook.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkcontainer.h>
-#include <gtk/gtkeditable.h>
 #include <gtk/gtkwindow.h>
-#include <gtk/gtktext.h>
+#include <gtk/gtktextview.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 
-#include "intl.h"
 #include "main.h"
 #include "messageview.h"
 #include "message_search.h"
@@ -62,15 +62,19 @@
 
 static GList *messageview_list = NULL;
 
-static void messageview_change_view_type(MessageView	*messageview,
-					 MessageType	 type);
-static void messageview_destroy_cb	(GtkWidget	*widget,
-					 MessageView	*messageview);
-static void messageview_size_allocate_cb(GtkWidget	*widget,
-					 GtkAllocation	*allocation);
-static void key_pressed			(GtkWidget	*widget,
-					 GdkEventKey	*event,
-					 MessageView	*messageview);
+static void messageview_change_view_type(MessageView		*messageview,
+					 MessageType		 type);
+static void messageview_destroy_cb	(GtkWidget		*widget,
+					 MessageView		*messageview);
+static void messageview_size_allocate_cb(GtkWidget		*widget,
+					 GtkAllocation		*allocation);
+static void messageview_switch_page_cb	(GtkNotebook		*notebook,
+					 GtkNotebookPage	*page,
+					 guint			 page_num,
+					 MessageView		*messageview);
+static gboolean key_pressed		(GtkWidget		*widget,
+					 GdkEventKey		*event,
+					 MessageView		*messageview);
 
 static void save_as_cb			(gpointer	 data,
 					 guint		 action,
@@ -142,87 +146,81 @@ static GtkItemFactoryEntry msgview_entries[] =
 
 	{N_("/_View"),			NULL, NULL, 0, "<Branch>"},
 
-#define CODESET_SEPARATOR \
-	{N_("/_View/_Code set/---"),	NULL, NULL, 0, "<Separator>"}
-#define CODESET_ACTION(action) \
-	NULL, set_charset_cb, action, "/View/Code set/Auto detect"
+#define ENC_SEPARATOR \
+	{N_("/_View/Character _encoding/---"),	NULL, NULL, 0, "<Separator>"}
+#define ENC_ACTION(action) \
+	NULL, set_charset_cb, action, "/View/Character encoding/Auto detect"
 
-	{N_("/_View/_Code set"),	NULL, NULL, 0, "<Branch>"},
-	{N_("/_View/_Code set/_Auto detect"),
+	{N_("/_View/Character _encoding"),	NULL, NULL, 0, "<Branch>"},
+	{N_("/_View/Character _encoding/_Auto detect"),
 					NULL, set_charset_cb, C_AUTO, "<RadioItem>"},
-	CODESET_SEPARATOR,
-	{N_("/_View/_Code set/7bit ascii (US-ASC_II)"),
-	 CODESET_ACTION(C_US_ASCII)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/7bit ascii (US-ASC_II)"),
+	 ENC_ACTION(C_US_ASCII)},
 
-#if HAVE_ICONV
-	{N_("/_View/_Code set/Unicode (_UTF-8)"),
-	 CODESET_ACTION(C_UTF_8)},
-	CODESET_SEPARATOR,
-#endif
-	{N_("/_View/_Code set/Western European (ISO-8859-_1)"),
-	 CODESET_ACTION(C_ISO_8859_1)},
-	{N_("/_View/_Code set/Western European (ISO-8859-15)"),
-	 CODESET_ACTION(C_ISO_8859_15)},
-	CODESET_SEPARATOR,
-#if HAVE_ICONV
-	{N_("/_View/_Code set/Central European (ISO-8859-_2)"),
-	 CODESET_ACTION(C_ISO_8859_2)},
-	CODESET_SEPARATOR,
-	{N_("/_View/_Code set/_Baltic (ISO-8859-13)"),
-	 CODESET_ACTION(C_ISO_8859_13)},
-	{N_("/_View/_Code set/Baltic (ISO-8859-_4)"),
-	 CODESET_ACTION(C_ISO_8859_4)},
-	CODESET_SEPARATOR,
-	{N_("/_View/_Code set/Greek (ISO-8859-_7)"),
-	 CODESET_ACTION(C_ISO_8859_7)},
-	CODESET_SEPARATOR,
-	{N_("/_View/_Code set/Turkish (ISO-8859-_9)"),
-	 CODESET_ACTION(C_ISO_8859_9)},
-	CODESET_SEPARATOR,
-	{N_("/_View/_Code set/Cyrillic (ISO-8859-_5)"),
-	 CODESET_ACTION(C_ISO_8859_5)},
-	{N_("/_View/_Code set/Cyrillic (KOI8-_R)"),
-	 CODESET_ACTION(C_KOI8_R)},
-	{N_("/_View/_Code set/Cyrillic (KOI8-U)"),
-	 CODESET_ACTION(C_KOI8_U)},
-	{N_("/_View/_Code set/Cyrillic (Windows-1251)"),
-	 CODESET_ACTION(C_CP1251)},
-	CODESET_SEPARATOR,
-#endif
-	{N_("/_View/_Code set/Japanese (ISO-2022-_JP)"),
-	 CODESET_ACTION(C_ISO_2022_JP)},
-#if HAVE_ICONV
-	{N_("/_View/_Code set/Japanese (ISO-2022-JP-2)"),
-	 CODESET_ACTION(C_ISO_2022_JP_2)},
-#endif
-	{N_("/_View/_Code set/Japanese (_EUC-JP)"),
-	 CODESET_ACTION(C_EUC_JP)},
-	{N_("/_View/_Code set/Japanese (_Shift__JIS)"),
-	 CODESET_ACTION(C_SHIFT_JIS)},
-#if HAVE_ICONV
-	CODESET_SEPARATOR,
-	{N_("/_View/_Code set/Simplified Chinese (_GB2312)"),
-	 CODESET_ACTION(C_GB2312)},
-	{N_("/_View/_Code set/Traditional Chinese (_Big5)"),
-	 CODESET_ACTION(C_BIG5)},
-	{N_("/_View/_Code set/Traditional Chinese (EUC-_TW)"),
-	 CODESET_ACTION(C_EUC_TW)},
-	{N_("/_View/_Code set/Chinese (ISO-2022-_CN)"),
-	 CODESET_ACTION(C_ISO_2022_CN)},
-	CODESET_SEPARATOR,
-	{N_("/_View/_Code set/Korean (EUC-_KR)"),
-	 CODESET_ACTION(C_EUC_KR)},
-	{N_("/_View/_Code set/Korean (ISO-2022-KR)"),
-	 CODESET_ACTION(C_ISO_2022_KR)},
-	CODESET_SEPARATOR,
-	{N_("/_View/_Code set/Thai (TIS-620)"),
-	 CODESET_ACTION(C_TIS_620)},
-	{N_("/_View/_Code set/Thai (Windows-874)"),
-	 CODESET_ACTION(C_WINDOWS_874)},
-#endif
+	{N_("/_View/Character _encoding/Unicode (_UTF-8)"),
+	 ENC_ACTION(C_UTF_8)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Western European (ISO-8859-_1)"),
+	 ENC_ACTION(C_ISO_8859_1)},
+	{N_("/_View/Character _encoding/Western European (ISO-8859-15)"),
+	 ENC_ACTION(C_ISO_8859_15)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Central European (ISO-8859-_2)"),
+	 ENC_ACTION(C_ISO_8859_2)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/_Baltic (ISO-8859-13)"),
+	 ENC_ACTION(C_ISO_8859_13)},
+	{N_("/_View/Character _encoding/Baltic (ISO-8859-_4)"),
+	 ENC_ACTION(C_ISO_8859_4)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Greek (ISO-8859-_7)"),
+	 ENC_ACTION(C_ISO_8859_7)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Turkish (ISO-8859-_9)"),
+	 ENC_ACTION(C_ISO_8859_9)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Cyrillic (ISO-8859-_5)"),
+	 ENC_ACTION(C_ISO_8859_5)},
+	{N_("/_View/Character _encoding/Cyrillic (KOI8-_R)"),
+	 ENC_ACTION(C_KOI8_R)},
+	{N_("/_View/Character _encoding/Cyrillic (KOI8-U)"),
+	 ENC_ACTION(C_KOI8_U)},
+	{N_("/_View/Character _encoding/Cyrillic (Windows-1251)"),
+	 ENC_ACTION(C_CP1251)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Japanese (ISO-2022-_JP)"),
+	 ENC_ACTION(C_ISO_2022_JP)},
+	{N_("/_View/Character _encoding/Japanese (ISO-2022-JP-2)"),
+	 ENC_ACTION(C_ISO_2022_JP_2)},
+	{N_("/_View/Character _encoding/Japanese (_EUC-JP)"),
+	 ENC_ACTION(C_EUC_JP)},
+	{N_("/_View/Character _encoding/Japanese (_Shift__JIS)"),
+	 ENC_ACTION(C_SHIFT_JIS)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Simplified Chinese (_GB2312)"),
+	 ENC_ACTION(C_GB2312)},
+	{N_("/_View/Character _encoding/Simplified Chinese (GBK)"),
+	 ENC_ACTION(C_GBK)},
+	{N_("/_View/Character _encoding/Traditional Chinese (_Big5)"),
+	 ENC_ACTION(C_BIG5)},
+	{N_("/_View/Character _encoding/Traditional Chinese (EUC-_TW)"),
+	 ENC_ACTION(C_EUC_TW)},
+	{N_("/_View/Character _encoding/Chinese (ISO-2022-_CN)"),
+	 ENC_ACTION(C_ISO_2022_CN)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Korean (EUC-_KR)"),
+	 ENC_ACTION(C_EUC_KR)},
+	{N_("/_View/Character _encoding/Korean (ISO-2022-KR)"),
+	 ENC_ACTION(C_ISO_2022_KR)},
+	ENC_SEPARATOR,
+	{N_("/_View/Character _encoding/Thai (TIS-620)"),
+	 ENC_ACTION(C_TIS_620)},
+	{N_("/_View/Character _encoding/Thai (Windows-874)"),
+	 ENC_ACTION(C_WINDOWS_874)},
 
-#undef CODESET_SEPARATOR
-#undef CODESET_ACTION
+#undef ENC_SEPARATOR
+#undef ENC_ACTION
 
 	{N_("/_View/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/Mess_age source"),	NULL, view_source_cb, 0, NULL},
@@ -274,6 +272,7 @@ MessageView *messageview_create(void)
 {
 	MessageView *messageview;
 	GtkWidget *vbox;
+	GtkWidget *notebook;
 	HeaderView *headerview;
 	TextView *textview;
 	MimeView *mimeview;
@@ -295,23 +294,45 @@ MessageView *messageview_create(void)
 	mimeview->imageview->messageview = messageview;
 	mimeview->messageview = messageview;
 
+	notebook = gtk_notebook_new();
+	gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);
+	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
+	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);
+	gtk_widget_show(notebook);
+
+	gtk_container_add(GTK_CONTAINER(notebook), GTK_WIDGET_PTR(textview));
+	gtk_notebook_set_tab_label_text
+		(GTK_NOTEBOOK(notebook), GTK_WIDGET_PTR(textview), _("Text"));
+
+	gtk_container_add(GTK_CONTAINER(notebook), GTK_WIDGET_PTR(mimeview));
+	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
+					GTK_WIDGET_PTR(mimeview),
+					_("Attachments"));
+
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
+	gtk_widget_show_all(notebook);
+
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET_PTR(headerview),
 			   FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET_PTR(textview),
-			   TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
 	gtk_widget_show(vbox);
 
 	/* to remove without destroyed */
-	gtk_widget_ref(GTK_WIDGET_PTR(textview));
-	gtk_widget_ref(GTK_WIDGET_PTR(mimeview));
 	gtk_widget_ref(GTK_WIDGET_PTR(mimeview->textview));
 	gtk_widget_ref(GTK_WIDGET_PTR(mimeview->imageview));
 
+	g_signal_connect(G_OBJECT(notebook), "switch_page",
+			 G_CALLBACK(messageview_switch_page_cb), messageview);
+
 	messageview->vbox        = vbox;
+	messageview->notebook    = notebook;
+
 	messageview->new_window  = FALSE;
 	messageview->window      = NULL;
 	messageview->window_vbox = NULL;
+	messageview->body_vbox   = NULL;
+
 	messageview->headerview  = headerview;
 	messageview->textview    = textview;
 	messageview->mimeview    = mimeview;
@@ -319,6 +340,8 @@ MessageView *messageview_create(void)
 	messageview->statusbar     = NULL;
 	messageview->statusbar_cid = 0;
 
+	messageview->current_page = 0;
+
 	return messageview;
 }
 
@@ -337,27 +360,30 @@ MessageView *messageview_create_with_new_window(void)
 	gtk_window_set_title(GTK_WINDOW(window), _("Sylpheed - Message View"));
 	gtk_window_set_wmclass(GTK_WINDOW(window), "message_view", "Sylpheed");
 	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
-	gtk_widget_set_usize(window, prefs_common.msgwin_width,
-			     prefs_common.msgwin_height);
+	gtk_widget_set_size_request(window, prefs_common.msgwin_width,
+				    prefs_common.msgwin_height);
 
 	msgview = messageview_create();
 
 	window_vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(window), window_vbox);
 
-	gtk_signal_connect(GTK_OBJECT(window), "size_allocate",
-			   GTK_SIGNAL_FUNC(messageview_size_allocate_cb),
-			   msgview);
-	gtk_signal_connect(GTK_OBJECT(window), "destroy",
-			   GTK_SIGNAL_FUNC(messageview_destroy_cb), msgview);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(key_pressed), msgview);
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(messageview_size_allocate_cb),
+			 msgview);
+	g_signal_connect(G_OBJECT(window), "destroy",
+			 G_CALLBACK(messageview_destroy_cb), msgview);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(key_pressed), msgview);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	n_menu_entries = sizeof(msgview_entries) / sizeof (msgview_entries[0]);
 	menubar = menubar_create(window, msgview_entries, n_menu_entries,
 				 "<MessageView>", msgview);
+#warning FIXME_GTK2
+#if 0
 	menu_factory_copy_rc("<Main>", "<MessageView>");
+#endif
 	gtk_box_pack_start(GTK_BOX(window_vbox), menubar, FALSE, TRUE, 0);
 
 	body_vbox = gtk_vbox_new(FALSE, BORDER_WIDTH);
@@ -432,6 +458,8 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	if (messageview->msginfo != msginfo) {
 		procmsg_msginfo_free(messageview->msginfo);
 		messageview->msginfo = procmsg_msginfo_get_full_info(msginfo);
+		if (!messageview->msginfo)
+			messageview->msginfo = procmsg_msginfo_copy(msginfo);
 	}
 	headerview_show(messageview->headerview, messageview->msginfo);
 
@@ -456,31 +484,21 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 static void messageview_change_view_type(MessageView *messageview,
 					 MessageType type)
 {
-	TextView *textview = messageview->textview;
-	MimeView *mimeview = messageview->mimeview;
+	GtkWidget *notebook = messageview->notebook;
 
 	if (messageview->type == type) return;
 
 	if (type == MVIEW_MIME) {
-		gtkut_container_remove
-			(GTK_CONTAINER(GTK_WIDGET_PTR(messageview)),
-			 GTK_WIDGET_PTR(textview));
-		gtk_box_pack_start(GTK_BOX(messageview->vbox),
-				   GTK_WIDGET_PTR(mimeview), TRUE, TRUE, 0);
-		gtk_container_add(GTK_CONTAINER(mimeview->vbox),
-				  GTK_WIDGET_PTR(textview));
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), TRUE);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook),
+					      messageview->current_page);
 	} else if (type == MVIEW_TEXT) {
-		gtkut_container_remove
-			(GTK_CONTAINER(GTK_WIDGET_PTR(messageview)),
-			 GTK_WIDGET_PTR(mimeview));
-		mimeview_clear(mimeview);
+		gint current_page = messageview->current_page;
 
-		if (mimeview->vbox == GTK_WIDGET_PTR(textview)->parent)
-			gtkut_container_remove(GTK_CONTAINER(mimeview->vbox),
-			 		       GTK_WIDGET_PTR(textview));
-
-		gtk_box_pack_start(GTK_BOX(messageview->vbox),
-				   GTK_WIDGET_PTR(textview), TRUE, TRUE, 0);
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
+		messageview->current_page = current_page;
+		mimeview_clear(messageview->mimeview);
 	} else
 		return;
 
@@ -499,9 +517,8 @@ void messageview_clear(MessageView *messageview)
 
 void messageview_destroy(MessageView *messageview)
 {
-	GtkWidget *textview  = GTK_WIDGET_PTR(messageview->textview);
+	GtkWidget *textview  = GTK_WIDGET_PTR(messageview->mimeview->textview);
 	GtkWidget *imageview = GTK_WIDGET_PTR(messageview->mimeview->imageview);
-	GtkWidget *mimeview  = GTK_WIDGET_PTR(messageview->mimeview);
 
 	messageview_list = g_list_remove(messageview_list, messageview);
 
@@ -515,7 +532,6 @@ void messageview_destroy(MessageView *messageview)
 
 	gtk_widget_unref(textview);
 	gtk_widget_unref(imageview);
-	gtk_widget_unref(mimeview);
 }
 
 void messageview_quote_color_set(void)
@@ -535,7 +551,7 @@ TextView *messageview_get_current_textview(MessageView *messageview)
 		text = messageview->textview;
 	else if (messageview->type == MVIEW_MIME) {
 		if (gtk_notebook_get_current_page
-			(GTK_NOTEBOOK(messageview->mimeview->notebook)) == 0)
+			(GTK_NOTEBOOK(messageview->notebook)) == 0)
 			text = messageview->textview;
 		else if (messageview->mimeview->type == MIMEVIEW_TEXT)
 			text = messageview->mimeview->textview;
@@ -557,8 +573,15 @@ void messageview_copy_clipboard(MessageView *messageview)
 	TextView *text;
 
 	text = messageview_get_current_textview(messageview);
-	if (text)
-		gtk_editable_copy_clipboard(GTK_EDITABLE(text->text));
+	if (text) {
+		GtkTextView *textview = GTK_TEXT_VIEW(text->text);
+		GtkTextBuffer *buffer;
+		GtkClipboard *clipboard;
+
+		buffer = gtk_text_view_get_buffer(textview);
+		clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+		gtk_text_buffer_copy_clipboard(buffer, clipboard);
+	}
 }
 
 void messageview_select_all(MessageView *messageview)
@@ -566,8 +589,15 @@ void messageview_select_all(MessageView *messageview)
 	TextView *text;
 
 	text = messageview_get_current_textview(messageview);
-	if (text)
-		gtk_editable_select_region(GTK_EDITABLE(text->text), 0, -1);
+	if (text) {
+		GtkTextView *textview = GTK_TEXT_VIEW(text->text);
+		GtkTextBuffer *buffer;
+		GtkTextIter start, end;
+
+		buffer = gtk_text_view_get_buffer(textview);
+		gtk_text_buffer_get_bounds(buffer, &start, &end);
+		gtk_text_buffer_select_range(buffer, &start, &end);
+	}
 }
 
 void messageview_set_position(MessageView *messageview, gint pos)
@@ -609,16 +639,9 @@ void messageview_save_as(MessageView *messageview)
 		Xstrdup_a(filename, msginfo->subject, return);
 		subst_for_filename(filename);
 	}
-	dest = filesel_select_file(_("Save as"), filename);
-	if (!dest) return;
-	if (is_file_exist(dest)) {
-		AlertValue aval;
 
-		aval = alertpanel(_("Overwrite"),
-				  _("Overwrite existing file?"),
-				  _("OK"), _("Cancel"), NULL);
-		if (G_ALERTDEFAULT != aval) return;
-	}
+	dest = filesel_save_as(filename);
+	if (!dest) return;
 
 	src = procmsg_get_message_file(msginfo);
 	if (copy_file(src, dest, TRUE) < 0) {
@@ -626,6 +649,8 @@ void messageview_save_as(MessageView *messageview)
 				 g_basename(dest));
 	}
 	g_free(src);
+
+	g_free(dest);
 }
 
 static void messageview_destroy_cb(GtkWidget *widget, MessageView *messageview)
@@ -642,11 +667,19 @@ static void messageview_size_allocate_cb(GtkWidget *widget,
 	prefs_common.msgwin_height = allocation->height;
 }
 
-static void key_pressed(GtkWidget *widget, GdkEventKey *event,
-			MessageView *messageview)
+static void messageview_switch_page_cb(GtkNotebook *notebook,
+				       GtkNotebookPage *page, guint page_num,
+				       MessageView *messageview)
+{
+	messageview->current_page = page_num;
+}
+
+static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
+			    MessageView *messageview)
 {
 	if (event && event->keyval == GDK_Escape && messageview->window)
 		gtk_widget_destroy(messageview->window);
+	return FALSE;
 }
 
 static void save_as_cb(gpointer data, guint action, GtkWidget *widget)
@@ -773,8 +806,8 @@ static void reply_cb(gpointer data, guint action, GtkWidget *widget)
 	msginfo = messageview->msginfo;
 	mlist = g_slist_append(NULL, msginfo);
 
-	text = gtkut_editable_get_selection
-		(GTK_EDITABLE(messageview->textview->text));
+	text = gtkut_text_view_get_selection
+		(GTK_TEXT_VIEW(messageview->textview->text));
 	if (text && *text == '\0') {
 		g_free(text);
 		text = NULL;
@@ -854,7 +887,7 @@ static void create_filter_cb(gpointer data, guint action, GtkWidget *widget)
 
 	procmsg_get_filter_keyword(messageview->msginfo, &header, &key,
 				   (PrefsFilterType)action);
-	prefs_filter_open(messageview->msginfo, header, key);
+	prefs_filter_open(messageview->msginfo, header);
 
 	g_free(header);
 	g_free(key);
