@@ -21,7 +21,10 @@
 #  include "config.h"
 #endif
 
+#include "defs.h"
+
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
@@ -30,7 +33,6 @@
 #include <time.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "pop.h"
 #include "md5.h"
 #include "prefs_account.h"
@@ -247,10 +249,11 @@ static gint pop3_getrange_uidl_recv(Pop3Session *session, const gchar *data,
 		p = newline + 1;
 		if (p < lastp && *p == '\n') p++;
 
-		if (sscanf(buf, "%d %" Xstr(IDLEN) "s", &num, id) != 2)
-			return -1;
-		if (num <= 0 || num > session->count)
-			return -1;
+		if (sscanf(buf, "%d %" Xstr(IDLEN) "s", &num, id) != 2 ||
+		    num <= 0 || num > session->count) {
+			log_warning(_("invalid UIDL response: %s\n"), buf);
+			continue;
+		}
 
 		session->msg[num].uidl = g_strdup(id);
 
@@ -332,8 +335,8 @@ static gint pop3_retr_recv(Pop3Session *session, const gchar *data, guint len)
 		return -1;
 	}
 
-	/* drop_ok: 0: success 1: don't receive -1: error */
 	drop_ok = session->drop_message(session, file);
+	unlink(file);
 	g_free(file);
 	if (drop_ok < 0) {
 		session->error_val = PS_IOERR;
@@ -346,7 +349,9 @@ static gint pop3_retr_recv(Pop3Session *session, const gchar *data, guint len)
 
 	session->msg[session->cur_msg].received = TRUE;
 	session->msg[session->cur_msg].recv_time =
-		drop_ok == 1 ? RECV_TIME_KEEP : session->current_time;
+		drop_ok == DROP_DONT_RECEIVE ? RECV_TIME_KEEP
+			: drop_ok == DROP_DELETE ? RECV_TIME_DELETE
+			: session->current_time;
 
 	return PS_SUCCESS;
 }
@@ -451,19 +456,12 @@ GHashTable *pop3_get_uidl_table(PrefsAccount *ac_prefs)
 	table = g_hash_table_new(g_str_hash, g_str_equal);
 
 	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
-			   "uidl", G_DIR_SEPARATOR_S, ac_prefs->recv_server,
+			   UIDL_DIR, G_DIR_SEPARATOR_S, ac_prefs->recv_server,
 			   "-", ac_prefs->userid, NULL);
 	if ((fp = fopen(path, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(path, "fopen");
 		g_free(path);
-		path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
-				   "uidl-", ac_prefs->recv_server,
-				   "-", ac_prefs->userid, NULL);
-		if ((fp = fopen(path, "rb")) == NULL) {
-			if (ENOENT != errno) FILE_OP_ERROR(path, "fopen");
-			g_free(path);
-			return table;
-		}
+		return table;
 	}
 	g_free(path);
 
@@ -539,7 +537,8 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 	 * ^data              ^prev            ^cur             data+len-1^ */
 
 	prev = data;
-	while ((cur = memchr(prev, '\r', len - (prev - data))) != NULL) {
+	while ((cur = (gchar *)my_memmem(prev, len - (prev - data), "\r\n", 2))
+	       != NULL) {
 		if ((cur > prev && fwrite(prev, cur - prev, 1, fp) < 1) ||
 		    fputc('\n', fp) == EOF) {
 			FILE_OP_ERROR(file, "fwrite");
@@ -608,11 +607,12 @@ static Pop3State pop3_lookup_next(Pop3Session *session)
 		     ac->size_limit > 0 &&
 		     size > ac->size_limit * 1024);
 
-		if (ac->rmmail &&
-		    msg->recv_time != RECV_TIME_NONE &&
-		    msg->recv_time != RECV_TIME_KEEP &&
-		    session->current_time - msg->recv_time >=
-		    ac->msg_leave_time * 24 * 60 * 60) {
+		if (msg->recv_time == RECV_TIME_DELETE ||
+		    (ac->rmmail &&
+		     msg->recv_time != RECV_TIME_NONE &&
+		     msg->recv_time != RECV_TIME_KEEP &&
+		     session->current_time - msg->recv_time >=
+		     ac->msg_leave_time * 24 * 60 * 60)) {
 			log_print(_("POP3: Deleting expired message %d\n"),
 				  session->cur_msg);
 			pop3_delete_send(session);
@@ -726,7 +726,7 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 			pop3_stls_send(pop3_session);
 		else
 #endif
-		if (pop3_session->ac_prefs->protocol == A_APOP)
+		if (pop3_session->ac_prefs->use_apop_auth)
 			pop3_getauth_apop_send(pop3_session);
 		else
 			pop3_getauth_user_send(pop3_session);
@@ -735,7 +735,7 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 	case POP3_STLS:
 		if (pop3_stls_recv(pop3_session) != PS_SUCCESS)
 			return -1;
-		if (pop3_session->ac_prefs->protocol == A_APOP)
+		if (pop3_session->ac_prefs->use_apop_auth)
 			pop3_getauth_apop_send(pop3_session);
 		else
 			pop3_getauth_user_send(pop3_session);
@@ -833,10 +833,12 @@ static gint pop3_session_recv_data_finished(Session *session, guchar *data,
 		if (pop3_retr_recv(pop3_session, data, len) < 0)
 			return -1;
 
-		if (pop3_session->ac_prefs->rmmail &&
-		    pop3_session->ac_prefs->msg_leave_time == 0 &&
-		    pop3_session->msg[pop3_session->cur_msg].recv_time
-		    != RECV_TIME_KEEP)
+		if (pop3_session->msg[pop3_session->cur_msg].recv_time
+		    == RECV_TIME_DELETE ||
+		    (pop3_session->ac_prefs->rmmail &&
+		     pop3_session->ac_prefs->msg_leave_time == 0 &&
+		     pop3_session->msg[pop3_session->cur_msg].recv_time
+		     != RECV_TIME_KEEP))
 			pop3_delete_send(pop3_session);
 		else if (pop3_session->cur_msg == pop3_session->count)
 			pop3_logout_send(pop3_session);
