@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 
 #include "defs.h"
 
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
@@ -32,7 +34,6 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "intl.h"
 #include "main.h"
 #include "prefs.h"
 #include "prefs_account.h"
@@ -491,7 +492,7 @@ static void prefs_account_edit_custom_header	(void);
 static gint prefs_account_deleted		(GtkWidget	*widget,
 						 GdkEventAny	*event,
 						 gpointer	 data);
-static void prefs_account_key_pressed		(GtkWidget	*widget,
+static gboolean prefs_account_key_pressed	(GtkWidget	*widget,
 						 GdkEventKey	*event,
 						 gpointer	 data);
 static void prefs_account_ok			(void);
@@ -514,13 +515,18 @@ PrefsAccount *prefs_account_new(void)
 void prefs_account_read_config(PrefsAccount *ac_prefs, const gchar *label)
 {
 	const guchar *p = label;
+	gchar *rcpath;
 	gint id;
 
 	g_return_if_fail(ac_prefs != NULL);
 	g_return_if_fail(label != NULL);
 
 	memset(&tmp_ac_prefs, 0, sizeof(PrefsAccount));
-	prefs_read_config(param, label, ACCOUNT_RC);
+
+	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, ACCOUNT_RC, NULL);
+	prefs_read_config(param, label, rcpath, NULL);
+	g_free(rcpath);
+
 	*ac_prefs = tmp_ac_prefs;
 	while (*p && !isdigit(*p)) p++;
 	id = atoi(p);
@@ -615,7 +621,7 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 	}
 
 	manage_window_set_transient(GTK_WINDOW(dialog.window));
-	gtk_notebook_set_page(GTK_NOTEBOOK(dialog.notebook), 0);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(dialog.notebook), 0);
 	gtk_widget_grab_focus(dialog.ok_btn);
 
 	tmp_ac_prefs = *ac_prefs;
@@ -676,18 +682,18 @@ static void prefs_account_create(void)
 
 	/* create dialog */
 	prefs_dialog_create(&dialog);
-	gtk_signal_connect(GTK_OBJECT(dialog.window), "delete_event",
-			   GTK_SIGNAL_FUNC(prefs_account_deleted), NULL);
-	gtk_signal_connect(GTK_OBJECT(dialog.window), "key_press_event",
-			   GTK_SIGNAL_FUNC(prefs_account_key_pressed), NULL);
+	g_signal_connect(G_OBJECT(dialog.window), "delete_event",
+			 G_CALLBACK(prefs_account_deleted), NULL);
+	g_signal_connect(G_OBJECT(dialog.window), "key_press_event",
+			 G_CALLBACK(prefs_account_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(dialog.window);
 
-	gtk_signal_connect(GTK_OBJECT(dialog.ok_btn), "clicked",
-	 		   GTK_SIGNAL_FUNC(prefs_account_ok), NULL);
-	gtk_signal_connect(GTK_OBJECT(dialog.apply_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_account_apply), NULL);
-	gtk_signal_connect(GTK_OBJECT(dialog.cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_account_cancel), NULL);
+	g_signal_connect(G_OBJECT(dialog.ok_btn), "clicked",
+	 		 G_CALLBACK(prefs_account_ok), NULL);
+	g_signal_connect(G_OBJECT(dialog.apply_btn), "clicked",
+			 G_CALLBACK(prefs_account_apply), NULL);
+	g_signal_connect(G_OBJECT(dialog.cancel_btn), "clicked",
+			 G_CALLBACK(prefs_account_cancel), NULL);
 
 	prefs_account_basic_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Basic"), page++);
@@ -711,9 +717,8 @@ static void prefs_account_create(void)
 
 #define SET_ACTIVATE(menuitem) \
 { \
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", \
-			   GTK_SIGNAL_FUNC(prefs_account_protocol_activated), \
-			   NULL); \
+	g_signal_connect(G_OBJECT(menuitem), "activate", \
+			 G_CALLBACK(prefs_account_protocol_activated), NULL); \
 }
 
 static void prefs_account_basic_create(void)
@@ -762,7 +767,7 @@ static void prefs_account_basic_create(void)
 
 	acname_entry = gtk_entry_new ();
 	gtk_widget_show (acname_entry);
-	gtk_widget_set_usize (acname_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (acname_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), acname_entry, TRUE, TRUE, 0);
 
 	default_chkbtn = gtk_check_button_new_with_label (_("Set as default"));
@@ -880,14 +885,14 @@ static void prefs_account_basic_create(void)
 
 	uid_entry = gtk_entry_new ();
 	gtk_widget_show (uid_entry);
-	gtk_widget_set_usize (uid_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (uid_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_table_attach (GTK_TABLE (serv_table), uid_entry, 1, 2, 5, 6,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	pass_entry = gtk_entry_new ();
 	gtk_widget_show (pass_entry);
-	gtk_widget_set_usize (pass_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (pass_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_table_attach (GTK_TABLE (serv_table), pass_entry, 3, 4, 5, 6,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
@@ -1002,7 +1007,7 @@ static void prefs_account_receive_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox1), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	leave_time_label = gtk_label_new (_("Remove after"));
 	gtk_widget_show (leave_time_label);
@@ -1010,7 +1015,7 @@ static void prefs_account_receive_create(void)
 
 	leave_time_entry = gtk_entry_new ();
 	gtk_widget_show (leave_time_entry);
-	gtk_widget_set_usize (leave_time_entry, 64, -1);
+	gtk_widget_set_size_request (leave_time_entry, 64, -1);
 	gtk_box_pack_start (GTK_BOX (hbox1), leave_time_entry, FALSE, FALSE, 0);
 
 	leave_time_label = gtk_label_new (_("days"));
@@ -1028,7 +1033,7 @@ static void prefs_account_receive_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox1), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	leave_time_label = gtk_label_new (_("(0 days: remove immediately)"));
 	gtk_widget_show (leave_time_label);
@@ -1047,7 +1052,7 @@ static void prefs_account_receive_create(void)
 
 	size_limit_entry = gtk_entry_new ();
 	gtk_widget_show (size_limit_entry);
-	gtk_widget_set_usize (size_limit_entry, 64, -1);
+	gtk_widget_set_size_request (size_limit_entry, 64, -1);
 	gtk_box_pack_start (GTK_BOX (hbox1), size_limit_entry, FALSE, FALSE, 0);
 
 	label = gtk_label_new (_("KB"));
@@ -1071,15 +1076,15 @@ static void prefs_account_receive_create(void)
 
 	inbox_entry = gtk_entry_new ();
 	gtk_widget_show (inbox_entry);
-	gtk_widget_set_usize (inbox_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (inbox_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox1), inbox_entry, TRUE, TRUE, 0);
 
 	inbox_btn = gtk_button_new_with_label (_(" Select... "));
 	gtk_widget_show (inbox_btn);
 	gtk_box_pack_start (GTK_BOX (hbox1), inbox_btn, FALSE, FALSE, 0);
-	gtk_signal_connect (GTK_OBJECT (inbox_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_account_select_folder_cb),
-			    inbox_entry);
+	g_signal_connect (G_OBJECT (inbox_btn), "clicked",
+			  G_CALLBACK (prefs_account_select_folder_cb),
+			  inbox_entry);
 
 	PACK_VSPACER(vbox2, vbox3, VSPACING_NARROW_2);
 
@@ -1092,6 +1097,7 @@ static void prefs_account_receive_create(void)
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtkut_widget_set_small_font_size (label);
 
 	PACK_FRAME (vbox1, imap_frame, _("IMAP4"));
 
@@ -1191,9 +1197,9 @@ static void prefs_account_send_create(void)
 	gtk_widget_show (customhdr_edit_btn);
 	gtk_box_pack_start (GTK_BOX (hbox), customhdr_edit_btn,
 			    FALSE, FALSE, 0);
-	gtk_signal_connect (GTK_OBJECT (customhdr_edit_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_account_edit_custom_header),
-			    NULL);
+	g_signal_connect (G_OBJECT (customhdr_edit_btn), "clicked",
+			  G_CALLBACK (prefs_account_edit_custom_header),
+			  NULL);
 
 	SET_TOGGLE_SENSITIVITY (customhdr_chkbtn, customhdr_edit_btn);
 
@@ -1218,7 +1224,7 @@ static void prefs_account_send_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	label = gtk_label_new (_("Authentication method"));
 	gtk_widget_show (label);
@@ -1231,6 +1237,7 @@ static void prefs_account_send_create(void)
 	optmenu_menu = gtk_menu_new ();
 
 	MENUITEM_ADD (optmenu_menu, menuitem, _("Automatic"), 0);
+	MENUITEM_ADD (optmenu_menu, menuitem, "PLAIN", SMTPAUTH_PLAIN);
 	MENUITEM_ADD (optmenu_menu, menuitem, "LOGIN", SMTPAUTH_LOGIN);
 	MENUITEM_ADD (optmenu_menu, menuitem, "CRAM-MD5", SMTPAUTH_CRAM_MD5);
 	MENUITEM_ADD (optmenu_menu, menuitem, "DIGEST-MD5", SMTPAUTH_DIGEST_MD5);
@@ -1247,7 +1254,7 @@ static void prefs_account_send_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	label = gtk_label_new (_("User ID"));
 	gtk_widget_show (label);
@@ -1255,7 +1262,7 @@ static void prefs_account_send_create(void)
 
 	smtp_uid_entry = gtk_entry_new ();
 	gtk_widget_show (smtp_uid_entry);
-	gtk_widget_set_usize (smtp_uid_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (smtp_uid_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), smtp_uid_entry, TRUE, TRUE, 0);
 
 	label = gtk_label_new (_("Password"));
@@ -1264,7 +1271,7 @@ static void prefs_account_send_create(void)
 
 	smtp_pass_entry = gtk_entry_new ();
 	gtk_widget_show (smtp_pass_entry);
-	gtk_widget_set_usize (smtp_pass_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (smtp_pass_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), smtp_pass_entry, TRUE, TRUE, 0);
 	gtk_entry_set_visibility (GTK_ENTRY (smtp_pass_entry), FALSE);
 
@@ -1277,14 +1284,16 @@ static void prefs_account_send_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	label = gtk_label_new
-		(_("If you leave these entries empty, the same\n"
+		(_("If you leave these entries empty, the same "
 		   "user ID and password as receiving will be used."));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtkut_widget_set_small_font_size (label);
 
 	SET_TOGGLE_SENSITIVITY (smtp_auth_chkbtn, vbox4);
 
@@ -1342,16 +1351,16 @@ static void prefs_account_compose_create(void)
 	gtk_widget_show (sigfile_radiobtn);
 	gtk_box_pack_start (GTK_BOX (sig_hbox), sigfile_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (sigfile_radiobtn),
-				  GINT_TO_POINTER (SIG_FILE));
+	g_object_set_data (G_OBJECT (sigfile_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIG_FILE));
 
 	sigcmd_radiobtn = gtk_radio_button_new_with_label_from_widget
 		(GTK_RADIO_BUTTON(sigfile_radiobtn), _("Command output"));
 	gtk_widget_show (sigcmd_radiobtn);
 	gtk_box_pack_start (GTK_BOX (sig_hbox), sigcmd_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (sigcmd_radiobtn),
-				  GINT_TO_POINTER (SIG_COMMAND));
+	g_object_set_data (G_OBJECT (sigcmd_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIG_COMMAND));
 
 	sigpath_entry = gtk_entry_new ();
 	gtk_widget_show (sigpath_entry);
@@ -1451,8 +1460,9 @@ static void prefs_account_privacy_create(void)
 			   _("Use ASCII-armored format for encryption"));
 	PACK_CHECK_BUTTON (vbox2, clearsign_chkbtn,
 			   _("Use clear text signature"));
-	gtk_signal_connect (GTK_OBJECT (ascii_armored_chkbtn), "toggled",
-			    prefs_account_ascii_armored_warning, NULL);
+	g_signal_connect (G_OBJECT (ascii_armored_chkbtn), "toggled",
+			  G_CALLBACK (prefs_account_ascii_armored_warning),
+			  NULL);
 
 	PACK_FRAME (vbox1, frame1, _("Sign key"));
 
@@ -1466,8 +1476,8 @@ static void prefs_account_privacy_create(void)
 	gtk_widget_show (defaultkey_radiobtn);
 	gtk_box_pack_start (GTK_BOX (vbox2), defaultkey_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (defaultkey_radiobtn),
-				  GINT_TO_POINTER (SIGN_KEY_DEFAULT));
+	g_object_set_data (G_OBJECT (defaultkey_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIGN_KEY_DEFAULT));
 
 	emailkey_radiobtn = gtk_radio_button_new_with_label_from_widget
 		(GTK_RADIO_BUTTON (defaultkey_radiobtn),
@@ -1475,8 +1485,8 @@ static void prefs_account_privacy_create(void)
 	gtk_widget_show (emailkey_radiobtn);
 	gtk_box_pack_start (GTK_BOX (vbox2), emailkey_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (emailkey_radiobtn),
-				  GINT_TO_POINTER (SIGN_KEY_BY_FROM));
+	g_object_set_data (G_OBJECT (emailkey_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIGN_KEY_BY_FROM));
 
 	customkey_radiobtn = gtk_radio_button_new_with_label_from_widget
 		(GTK_RADIO_BUTTON (defaultkey_radiobtn),
@@ -1484,8 +1494,8 @@ static void prefs_account_privacy_create(void)
 	gtk_widget_show (customkey_radiobtn);
 	gtk_box_pack_start (GTK_BOX (vbox2), customkey_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (customkey_radiobtn),
-				  GINT_TO_POINTER (SIGN_KEY_CUSTOM));
+	g_object_set_data (G_OBJECT (customkey_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIGN_KEY_CUSTOM));
 
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox1);
@@ -1494,7 +1504,7 @@ static void prefs_account_privacy_create(void)
 	label = gtk_label_new ("");
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_widget_set_usize (label, 16, -1);
+	gtk_widget_set_size_request (label, 16, -1);
 
 	label = gtk_label_new (_("User or key ID:"));
 	gtk_widget_show (label);
@@ -1526,8 +1536,8 @@ static void prefs_account_privacy_create(void)
 		(GTK_RADIO_BUTTON (btn_p), label);			\
 	gtk_widget_show (btn);						\
 	gtk_box_pack_start (GTK_BOX (box), btn, FALSE, FALSE, 0);	\
-	gtk_object_set_user_data (GTK_OBJECT (btn),			\
-				  GINT_TO_POINTER (data));		\
+	g_object_set_data (G_OBJECT (btn), MENU_VAL_ID,			\
+			   GINT_TO_POINTER (data));			\
 }
 
 #define CREATE_RADIO_BUTTONS(box,					\
@@ -1538,8 +1548,8 @@ static void prefs_account_privacy_create(void)
 	btn1 = gtk_radio_button_new_with_label(NULL, btn1_label);	\
 	gtk_widget_show (btn1);						\
 	gtk_box_pack_start (GTK_BOX (box), btn1, FALSE, FALSE, 0);	\
-	gtk_object_set_user_data (GTK_OBJECT (btn1),			\
-				  GINT_TO_POINTER (btn1_data));		\
+	g_object_set_data (G_OBJECT (btn1), MENU_VAL_ID,		\
+			   GINT_TO_POINTER (btn1_data));		\
 									\
 	CREATE_RADIO_BUTTON(box, btn2, btn1, btn2_label, btn2_data);	\
 	CREATE_RADIO_BUTTON(box, btn3, btn1, btn3_label, btn3_data);	\
@@ -1628,8 +1638,8 @@ static void prefs_account_ssl_create(void)
 	gtk_widget_show (nntp_nossl_radiobtn);
 	gtk_box_pack_start (GTK_BOX (vbox4), nntp_nossl_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (nntp_nossl_radiobtn),
-				  GINT_TO_POINTER (SSL_NONE));
+	g_object_set_data (G_OBJECT (nntp_nossl_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SSL_NONE));
 
 	CREATE_RADIO_BUTTON(vbox4, nntp_ssltunnel_radiobtn, nntp_nossl_radiobtn,
 			    _("Use SSL for NNTP connection"), SSL_TUNNEL);
@@ -1665,12 +1675,13 @@ static void prefs_account_ssl_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 16, -1);
+	gtk_widget_set_size_request (hbox_spc, 16, -1);
 
 	label = gtk_label_new
 		(_("(Turn this off if you have problems in SSL connection)"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtkut_widget_set_small_font_size (label);
 
 	ssl.pop_frame               = pop_frame;
 	ssl.pop_nossl_radiobtn      = pop_nossl_radiobtn;
@@ -1739,10 +1750,11 @@ static void prefs_account_advanced_create(void)
 
 #define PACK_PORT_ENTRY(box, entry) \
 { \
-	entry = gtk_entry_new_with_max_length (5); \
+	entry = gtk_entry_new (); \
+	gtk_entry_set_max_length (GTK_ENTRY(entry), 5); \
 	gtk_widget_show (entry); \
 	gtk_box_pack_start (GTK_BOX (box), entry, FALSE, FALSE, 0); \
-	gtk_widget_set_usize (entry, 64, -1); \
+	gtk_widget_set_size_request (entry, 64, -1); \
 }
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
@@ -1841,9 +1853,9 @@ static void prefs_account_advanced_create(void)
 	gtk_widget_show (button);					\
 	gtk_table_attach (GTK_TABLE (table), button,			\
 			  2, 3, n, n + 1, GTK_FILL, 0, 0, 0);		\
-	gtk_signal_connect						\
-		(GTK_OBJECT (button), "clicked",			\
-		 GTK_SIGNAL_FUNC (prefs_account_select_folder_cb),	\
+	g_signal_connect						\
+		(G_OBJECT (button), "clicked",				\
+		 G_CALLBACK (prefs_account_select_folder_cb),		\
 		 entry);						\
 									\
 	SET_TOGGLE_SENSITIVITY (chkbtn, entry);				\
@@ -1889,11 +1901,12 @@ static gint prefs_account_deleted(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static void prefs_account_key_pressed(GtkWidget *widget, GdkEventKey *event,
-				      gpointer data)
+static gboolean prefs_account_key_pressed(GtkWidget *widget, GdkEventKey *event,
+					  gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_account_cancel();
+	return FALSE;
 }
 
 static void prefs_account_ok(void)
@@ -1911,7 +1924,7 @@ static gint prefs_account_apply(void)
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(basic.protocol_optmenu));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	protocol = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 
 	if (*gtk_entry_get_text(GTK_ENTRY(basic.acname_entry)) == '\0') {
 		alertpanel_error(_("Account name is not entered."));
@@ -1983,12 +1996,13 @@ static void prefs_account_enum_set_data_from_radiobtn(PrefParam *pparam)
 	GSList *group;
 
 	radiobtn = GTK_RADIO_BUTTON (*pparam->widget);
-	group = gtk_radio_button_group (radiobtn);
+	group = gtk_radio_button_get_group (radiobtn);
 	while (group != NULL) {
 		GtkToggleButton *btn = GTK_TOGGLE_BUTTON (group->data);
 		if (gtk_toggle_button_get_active (btn)) {
 			*((gint *)pparam->data) = GPOINTER_TO_INT
-				(gtk_object_get_user_data (GTK_OBJECT (btn)));
+				(g_object_get_data (G_OBJECT (btn),
+						    MENU_VAL_ID));
 			break;
 		}
 		group = group->next;
@@ -2003,10 +2017,12 @@ static void prefs_account_enum_set_radiobtn(PrefParam *pparam)
 
 	data = GINT_TO_POINTER (*((gint *)pparam->data));
 	radiobtn = GTK_RADIO_BUTTON (*pparam->widget);
-	group = gtk_radio_button_group (radiobtn);
+	group = gtk_radio_button_get_group (radiobtn);
 	while (group != NULL) {
 		GtkToggleButton *btn = GTK_TOGGLE_BUTTON (group->data);
-		gpointer data1 = gtk_object_get_user_data (GTK_OBJECT (btn));
+		gpointer data1;
+
+		data1 = g_object_get_data (G_OBJECT (btn), MENU_VAL_ID);
 		if (data1 == data) {
 			gtk_toggle_button_set_active (btn, TRUE);
 			break;
@@ -2036,7 +2052,7 @@ static void prefs_account_protocol_set_data_from_optmenu(PrefParam *pparam)
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	*((RecvProtocol *)pparam->data) = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 }
 
 static void prefs_account_protocol_set_optmenu(PrefParam *pparam)
@@ -2069,7 +2085,7 @@ static void prefs_account_imap_auth_type_set_data_from_optmenu(PrefParam *pparam
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	*((RecvProtocol *)pparam->data) = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 }
 
 static void prefs_account_imap_auth_type_set_optmenu(PrefParam *pparam)
@@ -2104,7 +2120,7 @@ static void prefs_account_smtp_auth_type_set_data_from_optmenu(PrefParam *pparam
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	*((RecvProtocol *)pparam->data) = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 }
 
 static void prefs_account_smtp_auth_type_set_optmenu(PrefParam *pparam)
@@ -2115,15 +2131,18 @@ static void prefs_account_smtp_auth_type_set_optmenu(PrefParam *pparam)
 	GtkWidget *menuitem;
 
 	switch (type) {
-	case SMTPAUTH_LOGIN:
+	case SMTPAUTH_PLAIN:
 		gtk_option_menu_set_history(optmenu, 1);
 		break;
-	case SMTPAUTH_CRAM_MD5:
+	case SMTPAUTH_LOGIN:
 		gtk_option_menu_set_history(optmenu, 2);
 		break;
-	case SMTPAUTH_DIGEST_MD5:
+	case SMTPAUTH_CRAM_MD5:
 		gtk_option_menu_set_history(optmenu, 3);
 		break;
+	case SMTPAUTH_DIGEST_MD5:
+		gtk_option_menu_set_history(optmenu, 4);
+		break;
 	case 0:
 	default:
 		gtk_option_menu_set_history(optmenu, 0);
@@ -2140,7 +2159,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 	gboolean active;
 
 	protocol = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 
 	switch(protocol) {
 	case A_NNTP:
