@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
@@ -31,7 +32,6 @@
 #include <string.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "main.h"
 #include "prefs.h"
 #include "prefs_filter.h"
@@ -53,7 +53,9 @@
 static struct FilterRuleListWindow {
 	GtkWidget *window;
 
-	GtkWidget *clist;
+	GtkWidget *treeview;
+	GtkListStore *store;
+	GtkTreeSelection *selection;
 
 	GtkWidget *add_btn;
 	GtkWidget *edit_btn;
@@ -69,15 +71,17 @@ static struct FilterRuleListWindow {
 	GtkWidget *close_btn;
 } rule_list_window;
 
-static GdkPixmap *markxpm;
-static GdkBitmap *markxpmmask;
+enum {
+	COL_ENABLED,
+	COL_NAME,
+	COL_FILTER_RULE,
+	N_COLS
+};
 
 static void prefs_filter_create			(void);
 
-static void prefs_filter_read_old_config	(void);
-
 static void prefs_filter_set_dialog		(void);
-static void prefs_filter_set_list_row		(gint		 row,
+static void prefs_filter_set_list_row		(GtkTreeIter	*iter,
 						 FilterRule	*rule,
 						 gboolean	 move_view);
 
@@ -97,18 +101,29 @@ static void prefs_filter_up		(void);
 static void prefs_filter_down		(void);
 static void prefs_filter_bottom		(void);
 
-static void prefs_filter_select		(GtkCList	*clist,
-					 gint		 row,
-					 gint		 column,
-					 GdkEvent	*event);
-static void prefs_filter_row_move	(GtkCList	*clist,
-					 gint		 source_row,
-					 gint		 dest_row);
+static gboolean prefs_filter_select	(GtkTreeSelection	*selection,
+					 GtkTreeModel		*model,
+					 GtkTreePath		*path,
+					 gboolean		 cur_selected,
+					 gpointer		 data);
+static void prefs_filter_enable_toggled	(GtkCellRenderer	*cell,
+					 gchar			*path,
+					 gpointer		 data);
+
+static void prefs_filter_row_activated	(GtkTreeView		*treeview,
+					 GtkTreePath		*path,
+					 GtkTreeViewColumn	*column,
+					 gpointer		 data);
+static void prefs_filter_row_reordered	(GtkTreeModel		*model,
+					 GtkTreePath		*path,
+					 GtkTreeIter		*iter,
+					 gpointer		 data,
+					 gpointer		 user_data);
 
 static gint prefs_filter_deleted	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	 data);
-static gint prefs_filter_key_pressed	(GtkWidget	*widget,
+static gboolean prefs_filter_key_pressed(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 gpointer	 data);
 static void prefs_filter_close		(void);
@@ -136,7 +151,7 @@ void prefs_filter_open(MsgInfo *msginfo, const gchar *header)
 		rule = prefs_filter_edit_open(NULL, header);
 
 		if (rule) {
-			prefs_filter_set_list_row(-1, rule, TRUE);
+			prefs_filter_set_list_row(NULL, rule, TRUE);
 			prefs_filter_set_list();
 		}
 	}
@@ -151,7 +166,11 @@ static void prefs_filter_create(void)
 
 	GtkWidget *hbox;
 	GtkWidget *scrolledwin;
-	GtkWidget *clist;
+	GtkWidget *treeview;
+	GtkListStore *store;
+	GtkTreeSelection *selection;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
 
 	GtkWidget *btn_vbox;
 	GtkWidget *spc_vbox;
@@ -166,13 +185,11 @@ static void prefs_filter_create(void)
 	GtkWidget *copy_btn;
 	GtkWidget *del_btn;
 
-	gchar *title[2];
-
 	debug_print("Creating filter setting window...\n");
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
-	gtk_widget_set_usize(window, 540, 360);
+	gtk_widget_set_size_request(window, 540, 360);
 	gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
@@ -181,21 +198,22 @@ static void prefs_filter_create(void)
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-	gtkut_button_set_create(&confirm_area, &close_btn, _("Close"),
-				NULL, NULL, NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area,
+				      &close_btn, GTK_STOCK_CLOSE,
+				      NULL, NULL, NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(close_btn);
 
 	gtk_window_set_title(GTK_WINDOW(window),
 			     _("Filter setting"));
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(prefs_filter_deleted), NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(prefs_filter_key_pressed), NULL);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(prefs_filter_deleted), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(prefs_filter_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT (window);
-	gtk_signal_connect(GTK_OBJECT(close_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_close), NULL);
+	g_signal_connect(G_OBJECT(close_btn), "clicked",
+			 G_CALLBACK(prefs_filter_close), NULL);
 
 	/* Rule list */
 
@@ -205,30 +223,49 @@ static void prefs_filter_create(void)
 
 	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(scrolledwin);
-	gtk_widget_set_usize(scrolledwin, -1, 150);
+	gtk_widget_set_size_request(scrolledwin, -1, 150);
 	gtk_box_pack_start(GTK_BOX(hbox), scrolledwin, TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
 				       GTK_POLICY_AUTOMATIC,
 				       GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
+					    GTK_SHADOW_IN);
 
-	title[0] = _("Enabled");
-	title[1] = _("Name");
-	clist = gtk_clist_new_with_titles(2, title);
-	gtk_widget_show(clist);
-	gtk_container_add (GTK_CONTAINER(scrolledwin), clist);
-	gtk_clist_set_column_width(GTK_CLIST(clist), 0, 64);
-	gtk_clist_set_column_justification(GTK_CLIST(clist), 0,
-					   GTK_JUSTIFY_CENTER);
-	gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
-	GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist)->column[0].button,
-			       GTK_CAN_FOCUS);
-	GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist)->column[1].button,
-			       GTK_CAN_FOCUS);
-	gtk_signal_connect(GTK_OBJECT(clist), "select_row",
-			   GTK_SIGNAL_FUNC(prefs_filter_select), NULL);
-	gtk_signal_connect_after(GTK_OBJECT(clist), "row_move",
-				 GTK_SIGNAL_FUNC(prefs_filter_row_move),
-				 NULL);
+	store = gtk_list_store_new
+		(N_COLS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_POINTER);
+
+	treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+	g_object_unref(G_OBJECT(store));
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), TRUE);
+	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
+	gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeview), COL_NAME);
+	gtk_tree_view_set_reorderable(GTK_TREE_VIEW(treeview), TRUE);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
+	gtk_tree_selection_set_select_function(selection, prefs_filter_select,
+					       NULL, NULL);
+
+	renderer = gtk_cell_renderer_toggle_new();
+	g_signal_connect(renderer, "toggled",
+			 G_CALLBACK(prefs_filter_enable_toggled), NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Enabled"), renderer, "active", COL_ENABLED, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
+
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Name"), renderer, "text", COL_NAME, NULL);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
+
+	gtk_widget_show(treeview);
+	gtk_container_add(GTK_CONTAINER(scrolledwin), treeview);
+
+	g_signal_connect(G_OBJECT(treeview), "row-activated",
+			 G_CALLBACK(prefs_filter_row_activated), NULL);
+	g_signal_connect_after(G_OBJECT(store), "rows-reordered",
+			       G_CALLBACK(prefs_filter_row_reordered), NULL);
 
 	/* Up / Down */
 
@@ -236,33 +273,33 @@ static void prefs_filter_create(void)
 	gtk_widget_show(btn_vbox);
 	gtk_box_pack_start(GTK_BOX(hbox), btn_vbox, FALSE, FALSE, 0);
 
-	top_btn = gtk_button_new_with_label(_("Top"));
+	top_btn = gtk_button_new_from_stock(GTK_STOCK_GOTO_TOP);
 	gtk_widget_show(top_btn);
 	gtk_box_pack_start(GTK_BOX(btn_vbox), top_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(top_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_top), NULL);
+	g_signal_connect(G_OBJECT(top_btn), "clicked",
+			 G_CALLBACK(prefs_filter_top), NULL);
 
 	PACK_VSPACER(btn_vbox, spc_vbox, VSPACING_NARROW_2);
 
-	up_btn = gtk_button_new_with_label(_("Up"));
+	up_btn = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
 	gtk_widget_show(up_btn);
 	gtk_box_pack_start(GTK_BOX(btn_vbox), up_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(up_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_up), NULL);
+	g_signal_connect(G_OBJECT(up_btn), "clicked",
+			 G_CALLBACK(prefs_filter_up), NULL);
 
-	down_btn = gtk_button_new_with_label(_("Down"));
+	down_btn = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
 	gtk_widget_show(down_btn);
 	gtk_box_pack_start(GTK_BOX(btn_vbox), down_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(down_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_down), NULL);
+	g_signal_connect(G_OBJECT(down_btn), "clicked",
+			 G_CALLBACK(prefs_filter_down), NULL);
 
 	PACK_VSPACER(btn_vbox, spc_vbox, VSPACING_NARROW_2);
 
-	bottom_btn = gtk_button_new_with_label(_("Bottom"));
+	bottom_btn = gtk_button_new_from_stock(GTK_STOCK_GOTO_BOTTOM);
 	gtk_widget_show(bottom_btn);
 	gtk_box_pack_start(GTK_BOX(btn_vbox), bottom_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(bottom_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_bottom), NULL);
+	g_signal_connect(G_OBJECT(bottom_btn), "clicked",
+			 G_CALLBACK(prefs_filter_bottom), NULL);
 
 	/* add / edit / copy / delete */
 
@@ -274,42 +311,47 @@ static void prefs_filter_create(void)
 	gtk_widget_show(btn_hbox);
 	gtk_box_pack_start(GTK_BOX(hbox), btn_hbox, FALSE, FALSE, 0);
 
-	add_btn = gtk_button_new_with_label(_("Add"));
+	add_btn = gtk_button_new_from_stock(GTK_STOCK_ADD);
 	gtk_widget_show(add_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), add_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(add_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_add_cb), NULL);
+	g_signal_connect(G_OBJECT(add_btn), "clicked",
+			 G_CALLBACK(prefs_filter_add_cb), NULL);
 
 	edit_btn = gtk_button_new_with_label(_("Edit"));
 	gtk_widget_show(edit_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), edit_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(edit_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_edit_cb), NULL);
+	g_signal_connect(G_OBJECT(edit_btn), "clicked",
+			 G_CALLBACK(prefs_filter_edit_cb), NULL);
 
-	copy_btn = gtk_button_new_with_label(_("Copy"));
+	copy_btn = gtk_button_new_from_stock(GTK_STOCK_COPY);
 	gtk_widget_show(copy_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), copy_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(copy_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_copy_cb), NULL);
+	g_signal_connect(G_OBJECT(copy_btn), "clicked",
+			 G_CALLBACK(prefs_filter_copy_cb), NULL);
 
-	del_btn = gtk_button_new_with_label(_(" Delete "));
+	del_btn = gtk_button_new_from_stock(GTK_STOCK_DELETE);
 	gtk_widget_show(del_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), del_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(del_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_filter_delete_cb), NULL);
+	g_signal_connect(G_OBJECT(del_btn), "clicked",
+			 G_CALLBACK(prefs_filter_delete_cb), NULL);
 
 	gtk_widget_show_all(window);
 
-	stock_pixmap_gdk(clist, STOCK_PIXMAP_MARK, &markxpm, &markxpmmask);
-
 	rule_list_window.window = window;
 	rule_list_window.close_btn = close_btn;
 
-	rule_list_window.clist = clist;
+	rule_list_window.treeview = treeview;
+	rule_list_window.store = store;
+	rule_list_window.selection = selection;
 
-	rule_list_window.default_hdr_list  = NULL;
-	rule_list_window.user_hdr_list  = NULL;
-	rule_list_window.msg_hdr_list  = NULL;
+	rule_list_window.add_btn = add_btn;
+	rule_list_window.edit_btn = edit_btn;
+	rule_list_window.copy_btn = copy_btn;
+	rule_list_window.del_btn = del_btn;
+
+	rule_list_window.default_hdr_list = NULL;
+	rule_list_window.user_hdr_list = NULL;
+	rule_list_window.msg_hdr_list = NULL;
 	rule_list_window.msg_hdr_table = NULL;
 }
 
@@ -332,7 +374,6 @@ void prefs_filter_read_config(void)
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, FILTER_LIST,
 			     NULL);
 	if (!is_file_exist(rcpath)) {
-		prefs_filter_read_old_config();
 		g_free(rcpath);
 		return;
 	}
@@ -350,35 +391,6 @@ void prefs_filter_read_config(void)
 	xml_free_tree(node);
 }
 
-static void prefs_filter_read_old_config(void)
-{
-	gchar *rcpath;
-	FILE *fp;
-	gchar buf[PREFSBUFSIZE];
-	FilterRule *rule;
-
-	debug_print("Reading old filter configuration...\n");
-
-	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, FILTER_RC, NULL);
-	if ((fp = fopen(rcpath, "rb")) == NULL) {
-		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
-		g_free(rcpath);
-		return;
-	}
-	g_free(rcpath);
-
-	while (fgets(buf, sizeof(buf), fp) != NULL) {
-		g_strchomp(buf);
-		rule = filter_read_str(buf);
-		if (rule) {
-			prefs_common.fltlist =
-				g_slist_append(prefs_common.fltlist, rule);
-		}
-	}
-
-	fclose(fp);
-}
-
 void prefs_filter_write_config(void)
 {
 	filter_write_config(prefs_common.fltlist);
@@ -423,63 +435,67 @@ void prefs_filter_delete_path(const gchar *path)
 
 static void prefs_filter_set_dialog(void)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
 	GSList *cur;
 
-	gtk_clist_freeze(clist);
-	gtk_clist_clear(clist);
+	gtk_list_store_clear(rule_list_window.store);
 
 	for (cur = prefs_common.fltlist; cur != NULL; cur = cur->next) {
 		FilterRule *rule = (FilterRule *)cur->data;
-		prefs_filter_set_list_row(-1, rule, FALSE);
+		prefs_filter_set_list_row(NULL, rule, FALSE);
 	}
-
-	gtk_clist_thaw(clist);
 }
 
-static void prefs_filter_set_list_row(gint row, FilterRule *rule,
+static void prefs_filter_set_list_row(GtkTreeIter *iter, FilterRule *rule,
 				      gboolean move_view)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
-	gchar *cond_str[2] = {"", NULL};
-
-	if (!rule)
-		rule = gtk_clist_get_row_data(clist, row);
+	GtkListStore *store = rule_list_window.store;
+	gchar *rule_name;
+	GtkTreeIter iter_;
 
 	g_return_if_fail(rule != NULL);
 
 	if (rule->name && *rule->name)
-		cond_str[1] = g_strdup(rule->name);
-	else {
-		cond_str[1] = filter_get_str(rule);
-	}
-
-	if (row < 0)
-		row = gtk_clist_append(clist, cond_str);
-	else {
-		FilterRule *prev_rule;
-
-		prev_rule = gtk_clist_get_row_data(clist, row);
-		if (rule == prev_rule)
-			gtk_clist_set_text(clist, row, 1, cond_str[1]);
-		else if (prev_rule) {
-			gtk_clist_set_text(clist, row, 1, cond_str[1]);
-			filter_rule_free(prev_rule);
-		} else
-			row = gtk_clist_append(clist, cond_str);
-	}
-
-	if (rule->enabled)
-		gtk_clist_set_pixmap(clist, row, 0, markxpm, markxpmmask);
+		rule_name = g_strdup(rule->name);
 	else
-		gtk_clist_set_text(clist, row, 0, "");
+		rule_name = filter_get_str(rule);
 
-	gtk_clist_set_row_data(clist, row, rule);
-	g_free(cond_str[1]);
+	if (!iter) {
+		gtk_list_store_append(store, &iter_);
+		gtk_list_store_set(store, &iter_,
+				   COL_ENABLED, rule->enabled,
+				   COL_NAME, rule_name,
+				   COL_FILTER_RULE, rule, -1);
+	} else {
+		FilterRule *prev_rule = NULL;
 
-	if (move_view &&
-	    gtk_clist_row_is_visible(clist, row) != GTK_VISIBILITY_FULL)
-		gtk_clist_moveto(clist, row, -1, 0.5, 0.0);
+		iter_ = *iter;
+		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter_,
+				   COL_FILTER_RULE, &prev_rule, -1);
+		if (!prev_rule) {
+			g_warning("rule at the row not found\n");
+			gtk_list_store_append(store, &iter_);
+		}
+
+		gtk_list_store_set(store, &iter_,
+				   COL_ENABLED, rule->enabled,
+				   COL_NAME, rule_name,
+				   COL_FILTER_RULE, rule, -1);
+
+		if (prev_rule && prev_rule != rule)
+			filter_rule_free(prev_rule);
+	}
+
+	g_free(rule_name);
+
+	if (move_view) {
+		GtkTreePath *path;
+
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter_);
+		gtk_tree_view_scroll_to_cell
+			(GTK_TREE_VIEW(rule_list_window.treeview),
+			 path, NULL, TRUE, 0.5, 0.0);
+		gtk_tree_path_free(path);
+	}
 }
 
 #define APPEND_HDR_LIST(hdr_list)					  \
@@ -651,18 +667,22 @@ static void prefs_filter_write_user_header_list(void)
 
 static void prefs_filter_set_list(void)
 {
-	gint row = 0;
 	FilterRule *rule;
+	GtkTreeIter iter;
+	GtkTreeModel *model = GTK_TREE_MODEL(rule_list_window.store);
 
 	g_slist_free(prefs_common.fltlist);
 	prefs_common.fltlist = NULL;
 
-	while ((rule = gtk_clist_get_row_data
-		(GTK_CLIST(rule_list_window.clist), row)) != NULL) {
-		prefs_common.fltlist = g_slist_append(prefs_common.fltlist,
-						      rule);
-		row++;
-	}
+	if (!gtk_tree_model_get_iter_first(model, &iter))
+		return;
+
+	do {
+		gtk_tree_model_get(model, &iter, COL_FILTER_RULE, &rule, -1);
+		if (rule)
+			prefs_common.fltlist =
+				g_slist_append(prefs_common.fltlist, rule);
+	} while (gtk_tree_model_iter_next(model, &iter));
 }
 
 static void prefs_filter_add_cb(void)
@@ -672,145 +692,187 @@ static void prefs_filter_add_cb(void)
 	rule = prefs_filter_edit_open(NULL, NULL);
 
 	if (rule) {
-		prefs_filter_set_list_row(-1, rule, TRUE);
+		prefs_filter_set_list_row(NULL, rule, TRUE);
 		prefs_filter_set_list();
 	}
 }
 
 static void prefs_filter_edit_cb(void)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
+	GtkTreeIter iter;
 	FilterRule *rule, *new_rule;
-	gint row;
 
-	if (!clist->selection) return;
+	if (!gtk_tree_selection_get_selected(rule_list_window.selection,
+					     NULL, &iter))
+		return;
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-
-	rule = gtk_clist_get_row_data(clist, row);
+	gtk_tree_model_get(GTK_TREE_MODEL(rule_list_window.store), &iter,
+			   COL_FILTER_RULE, &rule, -1);
 	g_return_if_fail(rule != NULL);
 
 	new_rule = prefs_filter_edit_open(rule, NULL);
 
 	if (new_rule) {
-		prefs_filter_set_list_row(row, new_rule, TRUE);
+		prefs_filter_set_list_row(&iter, new_rule, TRUE);
 		prefs_filter_set_list();
 	}
 }
 
 static void prefs_filter_copy_cb(void)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
+	GtkTreeIter iter;
 	FilterRule *rule, *new_rule;
-	gint row;
 
-	if (!clist->selection) return;
+	if (!gtk_tree_selection_get_selected(rule_list_window.selection,
+					     NULL, &iter))
+		return;
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-
-	rule = gtk_clist_get_row_data(clist, row);
+	gtk_tree_model_get(GTK_TREE_MODEL(rule_list_window.store), &iter,
+			   COL_FILTER_RULE, &rule, -1);
 	g_return_if_fail(rule != NULL);
 
 	new_rule = prefs_filter_edit_open(rule, NULL);
 
 	if (new_rule) {
-		prefs_filter_set_list_row(-1, new_rule, TRUE);
+		prefs_filter_set_list_row(NULL, new_rule, TRUE);
 		prefs_filter_set_list();
 	}
 }
 
 static void prefs_filter_delete_cb(void)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
+	GtkTreeIter iter;
 	FilterRule *rule;
-	gint row;
+	gchar buf[BUFFSIZE];
+	gboolean valid;
 
-	if (!clist->selection) return;
-	row = GPOINTER_TO_INT(clist->selection->data);
-
-	if (alertpanel(_("Delete rule"),
-		       _("Do you really want to delete this rule?"),
-		       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
+	if (!gtk_tree_selection_get_selected(rule_list_window.selection,
+					     NULL, &iter))
 		return;
 
-	rule = gtk_clist_get_row_data(clist, row);
-	filter_rule_free(rule);
-	gtk_clist_remove(clist, row);
+	gtk_tree_model_get(GTK_TREE_MODEL(rule_list_window.store), &iter,
+			   COL_FILTER_RULE, &rule, -1);
+	g_return_if_fail(rule != NULL);
+
+	g_snprintf(buf, sizeof(buf),
+		   _("Do you really want to delete the rule '%s'?"),
+		   rule->name ? rule->name : _("(Untitled)"));
+	if (alertpanel(_("Delete rule"), buf,
+		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+		return;
+
+	valid = gtk_list_store_remove(rule_list_window.store, &iter);
+	if (valid)
+		gtk_tree_selection_select_iter(rule_list_window.selection,
+					       &iter);
+
 	prefs_common.fltlist = g_slist_remove(prefs_common.fltlist, rule);
-	if (!clist->selection)
-		gtk_clist_select_row(clist, row - 1, -1);
+	filter_rule_free(rule);
 }
 
 static void prefs_filter_top(void)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
-	gint row;
+	GtkTreeIter iter;
 
-	if (!clist->selection) return;
+	if (!gtk_tree_selection_get_selected(rule_list_window.selection,
+					     NULL, &iter))
+		return;
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row > 0)
-		gtk_clist_row_move(clist, row, 0);
+	gtk_list_store_move_after(rule_list_window.store, &iter, NULL);
 }
 
 static void prefs_filter_up(void)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
-	gint row;
+	GtkTreeModel *model = GTK_TREE_MODEL(rule_list_window.store);
+	GtkTreeIter iter, prev;
+	GtkTreePath *path;
 
-	if (!clist->selection) return;
+	if (!gtk_tree_selection_get_selected(rule_list_window.selection,
+					     NULL, &iter))
+		return;
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row > 0)
-		gtk_clist_row_move(clist, row, row - 1);
+	path = gtk_tree_model_get_path(model, &iter);
+	if (gtk_tree_path_prev(path)) {
+		gtk_tree_model_get_iter(model, &prev, path);
+		gtk_list_store_swap(rule_list_window.store, &iter, &prev);
+	}
+	gtk_tree_path_free(path);
 }
 
 static void prefs_filter_down(void)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
-	gint row;
+	GtkTreeIter iter, next;
 
-	if (!clist->selection) return;
+	if (!gtk_tree_selection_get_selected(rule_list_window.selection,
+					     NULL, &iter))
+		return;
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row < clist->rows - 1)
-		gtk_clist_row_move(clist, row, row + 1);
+	next = iter;
+	if (gtk_tree_model_iter_next(GTK_TREE_MODEL(rule_list_window.store),
+				     &next))
+		gtk_list_store_swap(rule_list_window.store, &iter, &next);
 }
 
 static void prefs_filter_bottom(void)
 {
-	GtkCList *clist = GTK_CLIST(rule_list_window.clist);
-	gint row;
+	GtkTreeIter iter;
 
-	if (!clist->selection) return;
-
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row < clist->rows - 1)
-		gtk_clist_row_move(clist, row, clist->rows - 1);
-}
-
-static void prefs_filter_select(GtkCList *clist, gint row, gint column,
-				GdkEvent *event)
-{
-	if (event && event->type == GDK_2BUTTON_PRESS) {
-		prefs_filter_edit_cb();
+	if (!gtk_tree_selection_get_selected(rule_list_window.selection,
+					     NULL, &iter))
 		return;
-	}
 
-	if (column == 0) {
-		FilterRule *rule;
-		rule = gtk_clist_get_row_data(clist, row);
-		rule->enabled ^= TRUE;
-		prefs_filter_set_list_row(row, rule, FALSE);
-	}
+	gtk_list_store_move_before(rule_list_window.store, &iter, NULL);
 }
 
-static void prefs_filter_row_move(GtkCList *clist, gint source_row,
-				  gint dest_row)
+static gboolean prefs_filter_select(GtkTreeSelection *selection,
+				    GtkTreeModel *model, GtkTreePath *path,
+				    gboolean cur_selected, gpointer data)
 {
-	prefs_filter_set_list();
-	if (gtk_clist_row_is_visible(clist, dest_row) != GTK_VISIBILITY_FULL)
-		gtk_clist_moveto(clist, dest_row, -1, 0.5, 0.0);
+	return TRUE;
+}
+
+static void prefs_filter_enable_toggled(GtkCellRenderer *cell, gchar *path_str,
+					gpointer data)
+{
+	FilterRule *rule;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	path = gtk_tree_path_new_from_string(path_str);
+	gtk_tree_model_get_iter(GTK_TREE_MODEL(rule_list_window.store),
+				&iter, path);
+	gtk_tree_path_free(path);
+	gtk_tree_model_get(GTK_TREE_MODEL(rule_list_window.store), &iter,
+			   COL_FILTER_RULE, &rule, -1);
+
+	rule->enabled ^= TRUE;
+
+	gtk_list_store_set(rule_list_window.store, &iter,
+			   COL_ENABLED, rule->enabled, -1);
+}
+
+static void prefs_filter_row_activated(GtkTreeView *treeview, GtkTreePath *path,
+				       GtkTreeViewColumn *column,
+				       gpointer data)
+{
+	gtk_button_clicked(GTK_BUTTON(rule_list_window.edit_btn));
+}
+
+static void prefs_filter_row_reordered(GtkTreeModel *model,
+				       GtkTreePath *path, GtkTreeIter *iter,
+				       gpointer data, gpointer user_data)
+{
+	GtkTreeIter iter_;
+	GtkTreePath *path_;
+
+	if (!gtk_tree_selection_get_selected(rule_list_window.selection,
+					     NULL, &iter_))
+		return;
+	path_ = gtk_tree_model_get_path
+		(GTK_TREE_MODEL(rule_list_window.store), &iter_);
+	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(rule_list_window.treeview),
+				     path_, NULL, FALSE, 0.0, 0.0);
+	gtk_tree_path_free(path_);
 }
 
 static gint prefs_filter_deleted(GtkWidget *widget, GdkEventAny *event,
@@ -820,20 +882,21 @@ static gint prefs_filter_deleted(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static gint prefs_filter_key_pressed(GtkWidget *widget, GdkEventKey *event,
-				     gpointer data)
+static gboolean prefs_filter_key_pressed(GtkWidget *widget, GdkEventKey *event,
+					 gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_filter_close();
-	return TRUE;
+	return FALSE;
 }
 
 static void prefs_filter_close(void)
 {
 	prefs_filter_set_msg_header_list(NULL);
 	prefs_filter_write_user_header_list();
+	prefs_filter_set_list();
 	filter_write_config(prefs_common.fltlist);
 	gtk_widget_hide(rule_list_window.window);
-	gtk_clist_clear(GTK_CLIST(rule_list_window.clist));
+	gtk_list_store_clear(rule_list_window.store);
 	inc_unlock();
 }
