@@ -24,10 +24,10 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "intl.h"
 #include "folder.h"
 #include "prefs.h"
 #include "prefs_folder_item.h"
@@ -80,7 +80,7 @@ static void prefs_folder_item_cancel_cb	(GtkWidget		*widget,
 static gint prefs_folder_item_delete_cb	(GtkWidget		*widget,
 					 GdkEventAny		*event, 
 					 PrefsFolderItemDialog	*dialog);
-static void prefs_folder_item_key_press_cb
+static gboolean prefs_folder_item_key_press_cb
 					(GtkWidget		*widget,
 					 GdkEventKey		*event,
 					 PrefsFolderItemDialog	*dialog);
@@ -114,23 +114,18 @@ PrefsFolderItemDialog *prefs_folder_item_create(FolderItem *item)
 
 	gtk_window_set_title(GTK_WINDOW(dialog->window), _("Folder properties"));
 	gtk_widget_realize(dialog->window);
-	gtk_signal_connect(GTK_OBJECT(dialog->window), "delete_event",
-			   GTK_SIGNAL_FUNC(prefs_folder_item_delete_cb),
-			   new_dialog);
-	gtk_signal_connect(GTK_OBJECT(dialog->window), "key_press_event",
-			   GTK_SIGNAL_FUNC(prefs_folder_item_key_press_cb),
-			   new_dialog);
+	g_signal_connect(G_OBJECT(dialog->window), "delete_event",
+			 G_CALLBACK(prefs_folder_item_delete_cb), new_dialog);
+	g_signal_connect(G_OBJECT(dialog->window), "key_press_event",
+			 G_CALLBACK(prefs_folder_item_key_press_cb), new_dialog);
 	MANAGE_WINDOW_SIGNALS_CONNECT(dialog->window);
 
-	gtk_signal_connect(GTK_OBJECT(dialog->ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_folder_item_ok_cb),
-			   new_dialog);
-	gtk_signal_connect(GTK_OBJECT(dialog->apply_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_folder_item_apply_cb),
-			   new_dialog);
-	gtk_signal_connect(GTK_OBJECT(dialog->cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_folder_item_cancel_cb),
-			   new_dialog);
+	g_signal_connect(G_OBJECT(dialog->ok_btn), "clicked",
+			 G_CALLBACK(prefs_folder_item_ok_cb), new_dialog);
+	g_signal_connect(G_OBJECT(dialog->apply_btn), "clicked",
+			 G_CALLBACK(prefs_folder_item_apply_cb), new_dialog);
+	g_signal_connect(G_OBJECT(dialog->cancel_btn), "clicked",
+			 G_CALLBACK(prefs_folder_item_cancel_cb), new_dialog);
 
 	new_dialog->dialog = dialog;
 	new_dialog->item = item;
@@ -178,8 +173,8 @@ static void prefs_folder_item_general_create(PrefsFolderItemDialog *dialog)
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
 	name_entry = gtk_entry_new();
-	gtk_entry_set_editable(GTK_ENTRY(name_entry), FALSE);
-	gtk_widget_set_usize(name_entry, 200, -1);
+	gtk_editable_set_editable(GTK_EDITABLE(name_entry), FALSE);
+	gtk_widget_set_size_request(name_entry, 200, -1);
 	gtk_widget_set_style(name_entry, style);
 	gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, 0, 1,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -191,8 +186,8 @@ static void prefs_folder_item_general_create(PrefsFolderItemDialog *dialog)
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
 	path_entry = gtk_entry_new();
-	gtk_entry_set_editable(GTK_ENTRY(path_entry), FALSE);
-	gtk_widget_set_usize(path_entry, 200, -1);
+	gtk_editable_set_editable(GTK_EDITABLE(path_entry), FALSE);
+	gtk_widget_set_size_request(path_entry, 200, -1);
 	gtk_widget_set_style(path_entry, style);
 	gtk_table_attach(GTK_TABLE(table), path_entry, 1, 2, 1, 2,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -322,7 +317,7 @@ static void prefs_folder_item_compose_create(PrefsFolderItemDialog *dialog)
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	to_entry = gtk_entry_new();
-	gtk_widget_set_usize(to_entry, 200, -1);
+	gtk_widget_set_size_request(to_entry, 200, -1);
 	gtk_box_pack_start(GTK_BOX(hbox), to_entry, TRUE, TRUE, 0);
 
 	PACK_CHECK_BUTTON(hbox, on_reply_chkbtn, _("use also on reply"));
@@ -333,7 +328,7 @@ static void prefs_folder_item_compose_create(PrefsFolderItemDialog *dialog)
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
 	cc_entry = gtk_entry_new();
-	gtk_widget_set_usize(cc_entry, 200, -1);
+	gtk_widget_set_size_request(cc_entry, 200, -1);
 	gtk_table_attach(GTK_TABLE(table), cc_entry, 1, 2, 1, 2,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
@@ -344,7 +339,7 @@ static void prefs_folder_item_compose_create(PrefsFolderItemDialog *dialog)
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
 	bcc_entry = gtk_entry_new();
-	gtk_widget_set_usize(bcc_entry, 200, -1);
+	gtk_widget_set_size_request(bcc_entry, 200, -1);
 	gtk_table_attach(GTK_TABLE(table), bcc_entry, 1, 2, 2, 3,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
@@ -355,7 +350,7 @@ static void prefs_folder_item_compose_create(PrefsFolderItemDialog *dialog)
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
 	replyto_entry = gtk_entry_new();
-	gtk_widget_set_usize(replyto_entry, 200, -1);
+	gtk_widget_set_size_request(replyto_entry, 200, -1);
 	gtk_table_attach(GTK_TABLE(table), replyto_entry, 1, 2, 3, 4,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
@@ -398,7 +393,7 @@ static void prefs_folder_item_set_dialog(PrefsFolderItemDialog *dialog)
 	     cur != NULL; cur = cur->next, n++) {
 		menuitem = GTK_WIDGET(cur->data);
 		type = (SpecialFolderItemType)
-			gtk_object_get_user_data(GTK_OBJECT(menuitem));
+			g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID);
 		if (type != F_NORMAL &&
 		    FOLDER_TYPE(dialog->item->folder) == F_NEWS)
 			gtk_widget_set_sensitive(menuitem, FALSE);
@@ -480,13 +475,13 @@ static void prefs_folder_item_apply_cb(GtkWidget *widget,
 	Folder *folder = item->folder;
 	FolderItem *prev_item = NULL;
 	gint account_id;
-	gchar *entry_str;
+	const gchar *entry_str;
 
 	optmenu = GTK_OPTION_MENU(dialog->type_optmenu);
 	menu = gtk_option_menu_get_menu(optmenu);
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	type = (SpecialFolderItemType)
-		gtk_object_get_user_data(GTK_OBJECT(menuitem));
+		g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID);
 
 	if (item->stype != type) {
 		switch (type) {
@@ -541,7 +536,7 @@ static void prefs_folder_item_apply_cb(GtkWidget *widget,
 	menu = gtk_option_menu_get_menu(optmenu);
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	account_id = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 	if (account_id >= 0)
 		item->account = account_find_from_id(account_id);
 	else
@@ -574,10 +569,11 @@ static gint prefs_folder_item_delete_cb(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static void prefs_folder_item_key_press_cb(GtkWidget *widget,
+static gboolean prefs_folder_item_key_press_cb(GtkWidget *widget,
 					   GdkEventKey *event,
 					   PrefsFolderItemDialog *dialog)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_folder_item_cancel_cb(widget, dialog);
+	return FALSE;
 }
