@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -42,7 +43,6 @@
 #include <dirent.h>
 #include <time.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "socket.h"
 #include "statusbar.h"
@@ -93,6 +93,25 @@ void hash_free_value_mem(GHashTable *table)
 	g_hash_table_foreach(table, hash_free_value_mem_func, NULL);
 }
 
+gint str_case_equal(gconstpointer v, gconstpointer v2)
+{
+	return strcasecmp((const gchar *)v, (const gchar *)v2) == 0;
+}
+
+guint str_case_hash(gconstpointer key)
+{
+	const gchar *p = key;
+	guint h = *p;
+
+	if (h) {
+		h = tolower(h);
+		for (p += 1; *p != '\0'; p++)
+			h = (h << 5) - h + tolower(*p);
+	}
+
+	return h;
+}
+
 void ptr_array_free_strings(GPtrArray *array)
 {
 	gint i;
@@ -106,9 +125,29 @@ void ptr_array_free_strings(GPtrArray *array)
 	}
 }
 
+gboolean str_find(const gchar *haystack, const gchar *needle)
+{
+	return strstr(haystack, needle) != NULL ? TRUE : FALSE;
+}
+
+gboolean str_case_find(const gchar *haystack, const gchar *needle)
+{
+	return strcasestr(haystack, needle) != NULL ? TRUE : FALSE;
+}
+
+gboolean str_find_equal(const gchar *haystack, const gchar *needle)
+{
+	return strcmp(haystack, needle) == 0;
+}
+
+gboolean str_case_find_equal(const gchar *haystack, const gchar *needle)
+{
+	return strcasecmp(haystack, needle) == 0;
+}
+
 gint to_number(const gchar *nstr)
 {
-	register const gchar *p;
+	register const guchar *p;
 
 	if (*nstr == '\0') return -1;
 
@@ -139,13 +178,13 @@ gchar *to_human_readable(off_t size)
 	static gchar str[10];
 
 	if (size < 1024)
-		g_snprintf(str, sizeof(str), "%dB", (gint)size);
+		g_snprintf(str, sizeof(str), _("%dB"), (gint)size);
 	else if (size >> 10 < 1024)
-		g_snprintf(str, sizeof(str), "%.1fKB", (gfloat)size / (1 << 10));
+		g_snprintf(str, sizeof(str), _("%.1fKB"), (gfloat)size / (1 << 10));
 	else if (size >> 20 < 1024)
-		g_snprintf(str, sizeof(str), "%.2fMB", (gfloat)size / (1 << 20));
+		g_snprintf(str, sizeof(str), _("%.2fMB"), (gfloat)size / (1 << 20));
 	else
-		g_snprintf(str, sizeof(str), "%.2fGB", (gfloat)size / (1 << 30));
+		g_snprintf(str, sizeof(str), _("%.2fGB"), (gfloat)size / (1 << 30));
 
 	return str;
 }
@@ -244,23 +283,40 @@ gchar *strcasestr(const gchar *haystack, const gchar *needle)
 	return NULL;
 }
 
+gpointer my_memmem(gconstpointer haystack, size_t haystacklen,
+		   gconstpointer needle, size_t needlelen)
+{
+	const gchar *haystack_ = (const gchar *)haystack;
+	const gchar *needle_ = (const gchar *)needle;
+	const gchar *haystack_cur = (const gchar *)haystack;
+
+	if (needlelen == 1)
+		return memchr(haystack_, *needle_, haystacklen);
+
+	while ((haystack_cur = memchr(haystack_cur, *needle_, haystacklen))
+	       != NULL) {
+		if (haystacklen - (haystack_cur - haystack_) < needlelen)
+			break;
+		if (memcmp(haystack_cur + 1, needle_ + 1, needlelen - 1) == 0)
+			return (gpointer)haystack_cur;
+		else
+			haystack_cur++;
+	}
+
+	return NULL;
+}
+
 /* Copy no more than N characters of SRC to DEST, with NULL terminating.  */
 gchar *strncpy2(gchar *dest, const gchar *src, size_t n)
 {
-	register gchar c;
-	gchar *s = dest;
+	register const gchar *s = src;
+	register gchar *d = dest;
 
-	do {
-		if (--n == 0) {
-			*dest = '\0';
-			return s;
-		}
-		c = *src++;
-		*dest++ = c;
-	} while (c != '\0');
+	while (--n && *s)
+		*d++ = *s++;
+	*d = '\0';
 
-	/* don't do zero fill */
-	return s;
+	return dest;
 }
 
 #if !HAVE_ISWALNUM
@@ -446,53 +502,52 @@ wchar_t *wcscasestr(const wchar_t *haystack, const wchar_t *needle)
 	return NULL;
 }
 
+gint get_mbs_len(const gchar *s)
+{
+	const gchar *p = s;
+	gint mb_len;
+	gint len = 0;
+
+	if (!p)
+		return -1;
+
+	while (*p != '\0') {
+		mb_len = g_utf8_skip[*(guchar *)p];
+		if (mb_len == 0)
+			break;
+		else
+			len++;
+
+		p += mb_len;
+	}
+
+	return len;
+}
+
 /* Examine if next block is non-ASCII string */
-gboolean is_next_nonascii(const wchar_t *s)
+gboolean is_next_nonascii(const guchar *s)
 {
-	const wchar_t *wp;
+	const guchar *p;
 
 	/* skip head space */
-	for (wp = s; *wp != (wchar_t)0 && iswspace(*wp); wp++)
+	for (p = s; *p != '\0' && isspace(*p); p++)
 		;
-	for (; *wp != (wchar_t)0 && !iswspace(*wp); wp++) {
-		if (*wp > 127)
+	for (; *p != '\0' && !isspace(*p); p++) {
+		if (*p > 127 || *p < 32)
 			return TRUE;
 	}
 
 	return FALSE;
 }
 
-/* Examine if next block is multi-byte string */
-gboolean is_next_mbs(const wchar_t *s)
+gint get_next_word_len(const guchar *s)
 {
-	gint mbl;
-	const wchar_t *wp;
-	gchar tmp[MB_LEN_MAX];
+	gint len = 0;
 
-	/* skip head space */
-	for (wp = s; *wp != (wchar_t)0 && iswspace(*wp); wp++)
+	for (; *s != '\0' && !isspace(*s); s++, len++)
 		;
-	for (; *wp != (wchar_t)0 && !iswspace(*wp); wp++) {
-		mbl = wctomb(tmp, *wp);
-		if (mbl > 1)
-			return TRUE;
-	}
 
-	return FALSE;
-}
-
-wchar_t *find_wspace(const wchar_t *s)
-{
-	const wchar_t *wp;
-
-	for (wp = s; *wp != (wchar_t)0 && iswspace(*wp); wp++)
-		;
-	for (; *wp != (wchar_t)0; wp++) {
-		if (iswspace(*wp))
-			return (wchar_t *)wp;
-	}
-
-	return NULL;
+	return len;
 }
 
 /* compare subjects */
@@ -506,17 +561,32 @@ gint subject_compare(const gchar *s1, const gchar *s2)
 	Xstrdup_a(str1, s1, return -1);
 	Xstrdup_a(str2, s2, return -1);
 
-	trim_subject(str1);
-	trim_subject(str2);
+	trim_subject_for_compare(str1);
+	trim_subject_for_compare(str2);
 
 	if (!*str1 || !*str2) return -1;
 
 	return strcmp(str1, str2);
 }
 
-void trim_subject(gchar *str)
+gint subject_compare_for_sort(const gchar *s1, const gchar *s2)
 {
-	gchar *srcp;
+	gchar *str1, *str2;
+
+	if (!s1 || !s2) return -1;
+
+	Xstrdup_a(str1, s1, return -1);
+	Xstrdup_a(str2, s2, return -1);
+
+	trim_subject_for_sort(str1);
+	trim_subject_for_sort(str2);
+
+	return strcasecmp(str1, str2);
+}
+
+void trim_subject_for_compare(gchar *str)
+{
+	guchar *srcp;
 
 	eliminate_parenthesis(str, '[', ']');
 	eliminate_parenthesis(str, '(', ')');
@@ -529,9 +599,58 @@ void trim_subject(gchar *str)
 	}
 }
 
+void trim_subject_for_sort(gchar *str)
+{
+	guchar *srcp;
+
+	g_strstrip(str);
+
+	while (!strncasecmp(str, "Re:", 3)) {
+		srcp = str + 3;
+		while (isspace(*srcp)) srcp++;
+		memmove(str, srcp, strlen(srcp) + 1);
+	}
+}
+
+void trim_subject(gchar *str)
+{
+	register guchar *srcp, *destp;
+	gchar op, cl;
+	gint in_brace;
+
+	destp = str;
+	while (!strncasecmp(destp, "Re:", 3)) {
+		destp += 3;
+		while (isspace(*destp)) destp++;
+	}
+
+	if (*destp == '[') {
+		op = '[';
+		cl = ']';
+	} else if (*destp == '(') {
+		op = '(';
+		cl = ')';
+	} else
+		return;
+
+	srcp = destp + 1;
+	in_brace = 1;
+	while (*srcp) {
+		if (*srcp == op)
+			in_brace++;
+		else if (*srcp == cl)
+			in_brace--;
+		srcp++;
+		if (in_brace == 0)
+			break;
+	}
+	while (isspace(*srcp)) srcp++;
+	memmove(destp, srcp, strlen(srcp) + 1);
+}
+
 void eliminate_parenthesis(gchar *str, gchar op, gchar cl)
 {
-	register gchar *srcp, *destp;
+	register guchar *srcp, *destp;
 	gint in_brace;
 
 	srcp = destp = str;
@@ -580,35 +699,6 @@ void extract_parenthesis(gchar *str, gchar op, gchar cl)
 	*destp = '\0';
 }
 
-void extract_one_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
-					     gchar op, gchar cl)
-{
-	register gchar *srcp, *destp;
-	gint in_brace;
-	gboolean in_quote = FALSE;
-
-	srcp = destp = str;
-
-	if ((srcp = strchr_with_skip_quote(destp, quote_chr, op))) {
-		memmove(destp, srcp + 1, strlen(srcp));
-		in_brace = 1;
-		while(*destp) {
-			if (*destp == op && !in_quote)
-				in_brace++;
-			else if (*destp == cl && !in_quote)
-				in_brace--;
-			else if (*destp == quote_chr)
-				in_quote ^= TRUE;
-
-			if (in_brace == 0)
-				break;
-
-			destp++;
-		}
-	}
-	*destp = '\0';
-}
-
 void extract_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
 					 gchar op, gchar cl)
 {
@@ -642,7 +732,7 @@ void extract_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
 
 void eliminate_quote(gchar *str, gchar quote_chr)
 {
-	register gchar *srcp, *destp;
+	register guchar *srcp, *destp;
 
 	srcp = destp = str;
 
@@ -672,7 +762,7 @@ void extract_quote(gchar *str, gchar quote_chr)
 
 void eliminate_address_comment(gchar *str)
 {
-	register gchar *srcp, *destp;
+	register guchar *srcp, *destp;
 	gint in_brace;
 
 	srcp = destp = str;
@@ -751,6 +841,13 @@ void extract_address(gchar *str)
 	g_strstrip(str);
 }
 
+void extract_list_id_str(gchar *str)
+{
+	if (strchr_with_skip_quote(str, '"', '<'))
+		extract_parenthesis_with_skip_quote(str, '"', '<', '>');
+	g_strstrip(str);
+}
+
 GSList *address_list_append(GSList *addr_list, const gchar *str)
 {
 	gchar *work;
@@ -786,7 +883,7 @@ GSList *address_list_append(GSList *addr_list, const gchar *str)
 	return addr_list;
 }
 
-GSList *references_list_append(GSList *msgid_list, const gchar *str)
+GSList *references_list_prepend(GSList *msgid_list, const gchar *str)
 {
 	const gchar *strp;
 
@@ -806,7 +903,7 @@ GSList *references_list_append(GSList *msgid_list, const gchar *str)
 		msgid = g_strndup(start + 1, end - start - 1);
 		g_strstrip(msgid);
 		if (*msgid)
-			msgid_list = g_slist_append(msgid_list, msgid);
+			msgid_list = g_slist_prepend(msgid_list, msgid);
 		else
 			g_free(msgid);
 
@@ -816,6 +913,17 @@ GSList *references_list_append(GSList *msgid_list, const gchar *str)
 	return msgid_list;
 }
 
+GSList *references_list_append(GSList *msgid_list, const gchar *str)
+{
+	GSList *list;
+
+	list = references_list_prepend(NULL, str);
+	list = g_slist_reverse(list);
+	msgid_list = g_slist_concat(msgid_list, list);
+
+	return msgid_list;
+}
+
 GSList *newsgroup_list_append(GSList *group_list, const gchar *str)
 {
 	gchar *work;
@@ -886,7 +994,7 @@ void remove_return(gchar *str)
 
 void remove_space(gchar *str)
 {
-	register gchar *p = str;
+	register guchar *p = str;
 	register gint spc;
 
 	while (*p) {
@@ -902,7 +1010,7 @@ void remove_space(gchar *str)
 
 void unfold_line(gchar *str)
 {
-	register gchar *p = str;
+	register guchar *p = str;
 	register gint spc;
 
 	while (*p) {
@@ -942,7 +1050,7 @@ void subst_chars(gchar *str, gchar *orig, gchar subst)
 
 void subst_for_filename(gchar *str)
 {
-	subst_chars(str, " \t\r\n\"/\\", '_');
+	subst_chars(str, " \t\r\n\"'/\\", '_');
 }
 
 gboolean is_header_line(const gchar *str)
@@ -973,15 +1081,15 @@ gboolean is_ascii_str(const guchar *str)
 
 gint get_quote_level(const gchar *str)
 {
-	const gchar *first_pos;
-	const gchar *last_pos;
-	const gchar *p = str;
+	const guchar *first_pos;
+	const guchar *last_pos;
+	const guchar *p = str;
 	gint quote_level = -1;
 
 	/* speed up line processing by only searching to the last '>' */
 	if ((first_pos = strchr(str, '>')) != NULL) {
 		/* skip a line if it contains a '<' before the initial '>' */
-		if (memchr(str, '<', first_pos - str) != NULL)
+		if (memchr(str, '<', first_pos - (const guchar *)str) != NULL)
 			return -1;
 		last_pos = strrchr(first_pos, '>');
 	} else
@@ -1112,7 +1220,7 @@ gchar **strsplit_parenthesis(const gchar *str, gchar op, gchar cl,
 			n++;
 			str = s_cl + 1;
 
-			while (*str && isspace(*str)) str++;
+			while (*str && isspace(*(guchar *)str)) str++;
 			if (*str != op) {
 				string_list = g_slist_prepend(string_list,
 							      g_strdup(""));
@@ -1199,18 +1307,20 @@ gchar **strsplit_with_quote(const gchar *str, const gchar *delim,
 	return str_array;
 }
 
-gchar *get_abbrev_newsgroup_name(const gchar *group)
+gchar *get_abbrev_newsgroup_name(const gchar *group, gint len)
 {
 	gchar *abbrev_group;
 	gchar *ap;
 	const gchar *p = group;
+	const gchar *last;
 
+	last = group + strlen(group);
 	abbrev_group = ap = g_malloc(strlen(group) + 1);
 
 	while (*p) {
 		while (*p == '.')
 			*ap++ = *p++;
-		if (strchr(p, '.')) {
+		if ((ap - abbrev_group) + (last - p) > len && strchr(p, '.')) {
 			*ap++ = *p++;
 			while (*p != '.') p++;
 		} else {
@@ -1233,17 +1343,17 @@ gchar *trim_string(const gchar *str, gint len)
 	if (!str) return NULL;
 	if (strlen(str) <= len)
 		return g_strdup(str);
+	if (g_utf8_validate(str, -1, NULL) == FALSE)
+		return g_strdup(str);
 
 	while (*p != '\0') {
-		mb_len = mblen(p, MB_LEN_MAX);
+		mb_len = g_utf8_skip[*(guchar *)p];
 		if (mb_len == 0)
 			break;
-		else if (mb_len < 0)
-			return g_strdup(str);
 		else if (new_len + mb_len > len)
 			break;
-		else
-			new_len += mb_len;
+
+		new_len += mb_len;
 		p += mb_len;
 	}
 
@@ -1251,10 +1361,37 @@ gchar *trim_string(const gchar *str, gint len)
 	return g_strconcat(new_str, "...", NULL);
 }
 
+gchar *trim_string_before(const gchar *str, gint len)
+{
+	const gchar *p = str;
+	gint mb_len;
+	gint new_len;
+
+	if (!str) return NULL;
+	if ((new_len = strlen(str)) <= len)
+		return g_strdup(str);
+	if (g_utf8_validate(str, -1, NULL) == FALSE)
+		return g_strdup(str);
+
+	while (*p != '\0') {
+		mb_len = g_utf8_skip[*(guchar *)p];
+		if (mb_len == 0)
+			break;
+
+		new_len -= mb_len;
+		p += mb_len;
+
+		if (new_len <= len)
+			break;
+	}
+
+	return g_strconcat("...", p, NULL);
+}
+
 GList *uri_list_extract_filenames(const gchar *uri_list)
 {
 	GList *result = NULL;
-	const gchar *p, *q;
+	const guchar *p, *q;
 	gchar *file;
 
 	p = uri_list;
@@ -1312,6 +1449,41 @@ static gint axtoi(const gchar *hex_str)
 	return (hi << 4) + lo;
 }
 
+gboolean is_uri_string(const gchar *str)
+{
+	return (g_strncasecmp(str, "http://", 7) == 0 ||
+		g_strncasecmp(str, "https://", 8) == 0 ||
+		g_strncasecmp(str, "ftp://", 6) == 0 ||
+		g_strncasecmp(str, "www.", 4) == 0);
+}
+
+gchar *get_uri_path(const gchar *uri)
+{
+	if (g_strncasecmp(uri, "http://", 7) == 0)
+		return (gchar *)(uri + 7);
+	else if (g_strncasecmp(uri, "https://", 8) == 0)
+		return (gchar *)(uri + 8);
+	else if (g_strncasecmp(uri, "ftp://", 6) == 0)
+		return (gchar *)(uri + 6);
+	else
+		return (gchar *)uri;
+}
+
+gint get_uri_len(const gchar *str)
+{
+	const gchar *p;
+
+	if (is_uri_string(str)) {
+		for (p = str; *p != '\0'; p++) {
+			if (!g_ascii_isgraph(*p) || strchr("()<>\"", *p))
+				break;
+		}
+		return p - str;
+	}
+
+	return 0;
+}
+
 /* Decodes URL-Encoded strings (i.e. strings in which spaces are replaced by
  * plusses, and escape characters are used)
  */
@@ -1323,7 +1495,8 @@ void decode_uri(gchar *decoded_uri, const gchar *encoded_uri)
 	while (*enc) {
 		if (*enc == '%') {
 			enc++;
-			if (isxdigit(enc[0]) && isxdigit(enc[1])) {
+			if (isxdigit((guchar)enc[0]) &&
+			    isxdigit((guchar)enc[1])) {
 				*dec = axtoi(enc);
 				dec++;
 				enc += 2;
@@ -1341,13 +1514,70 @@ void decode_uri(gchar *decoded_uri, const gchar *encoded_uri)
 	*dec = '\0';
 }
 
+gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
+		     gchar **subject, gchar **body)
+{
+	gchar *tmp_mailto;
+	gchar *p;
+
+	Xstrdup_a(tmp_mailto, mailto, return -1);
+
+	if (!strncmp(tmp_mailto, "mailto:", 7))
+		tmp_mailto += 7;
+
+	p = strchr(tmp_mailto, '?');
+	if (p) {
+		*p = '\0';
+		p++;
+	}
+
+	if (to && !*to)
+		*to = g_strdup(tmp_mailto);
+
+	while (p) {
+		gchar *field, *value;
+
+		field = p;
+
+		p = strchr(p, '=');
+		if (!p) break;
+		*p = '\0';
+		p++;
+
+		value = p;
+
+		p = strchr(p, '&');
+		if (p) {
+			*p = '\0';
+			p++;
+		}
+
+		if (*value == '\0') continue;
+
+		if (cc && !*cc && !g_strcasecmp(field, "cc")) {
+			*cc = g_strdup(value);
+		} else if (bcc && !*bcc && !g_strcasecmp(field, "bcc")) {
+			*bcc = g_strdup(value);
+		} else if (subject && !*subject &&
+			   !g_strcasecmp(field, "subject")) {
+			*subject = g_malloc(strlen(value) + 1);
+			decode_uri(*subject, value);
+		} else if (body && !*body && !g_strcasecmp(field, "body")) {
+			*body = g_malloc(strlen(value) + 1);
+			decode_uri(*body, value);
+		}
+	}
+
+	return 0;
+}
+
 /*
  * We need this wrapper around g_get_home_dir(), so that
  * we can fix some Windoze things here.  Should be done in glibc of course
  * but as long as we are not able to do our own extensions to glibc, we do 
  * it here.
  */
-gchar *get_home_dir(void)
+const gchar *get_home_dir(void)
 {
 #if HAVE_DOSISH_SYSTEM
     static gchar *home_dir;
@@ -1372,7 +1602,7 @@ gchar *get_home_dir(void)
 #endif
 }
 
-gchar *get_rc_dir(void)
+const gchar *get_rc_dir(void)
 {
 	static gchar *rc_dir = NULL;
 
@@ -1383,7 +1613,18 @@ gchar *get_rc_dir(void)
 	return rc_dir;
 }
 
-gchar *get_news_cache_dir(void)
+const gchar *get_old_rc_dir(void)
+{
+	static gchar *old_rc_dir = NULL;
+
+	if (!old_rc_dir)
+		old_rc_dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
+					 OLD_RC_DIR, NULL);
+
+	return old_rc_dir;
+}
+
+const gchar *get_news_cache_dir(void)
 {
 	static gchar *news_cache_dir = NULL;
 
@@ -1394,7 +1635,7 @@ gchar *get_news_cache_dir(void)
 	return news_cache_dir;
 }
 
-gchar *get_imap_cache_dir(void)
+const gchar *get_imap_cache_dir(void)
 {
 	static gchar *imap_cache_dir = NULL;
 
@@ -1405,7 +1646,7 @@ gchar *get_imap_cache_dir(void)
 	return imap_cache_dir;
 }
 
-gchar *get_mime_tmp_dir(void)
+const gchar *get_mime_tmp_dir(void)
 {
 	static gchar *mime_tmp_dir = NULL;
 
@@ -1416,7 +1657,7 @@ gchar *get_mime_tmp_dir(void)
 	return mime_tmp_dir;
 }
 
-gchar *get_template_dir(void)
+const gchar *get_template_dir(void)
 {
 	static gchar *template_dir = NULL;
 
@@ -1427,7 +1668,7 @@ gchar *get_template_dir(void)
 	return template_dir;
 }
 
-gchar *get_tmp_dir(void)
+const gchar *get_tmp_dir(void)
 {
 	static gchar *tmp_dir = NULL;
 
@@ -1449,7 +1690,7 @@ gchar *get_tmp_file(void)
 	return tmp_file;
 }
 
-gchar *get_domain_name(void)
+const gchar *get_domain_name(void)
 {
 	static gchar *domain_name = NULL;
 
@@ -1545,6 +1786,9 @@ gboolean file_exist(const gchar *file, gboolean allow_fifo)
 {
 	struct stat s;
 
+	if (file == NULL)
+		return FALSE;
+
 	if (stat(file, &s) < 0) {
 		if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
 		return FALSE;
@@ -1560,6 +1804,9 @@ gboolean is_dir_exist(const gchar *dir)
 {
 	struct stat s;
 
+	if (dir == NULL)
+		return FALSE;
+
 	if (stat(dir, &s) < 0) {
 		if (ENOENT != errno) FILE_OP_ERROR(dir, "stat");
 		return FALSE;
@@ -1575,6 +1822,9 @@ gboolean is_file_entry_exist(const gchar *file)
 {
 	struct stat s;
 
+	if (file == NULL)
+		return FALSE;
+
 	if (stat(file, &s) < 0) {
 		if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
 		return FALSE;
@@ -1583,6 +1833,34 @@ gboolean is_file_entry_exist(const gchar *file)
 	return TRUE;
 }
 
+gboolean dirent_is_regular_file(struct dirent *d)
+{
+	struct stat s;
+
+#ifdef HAVE_DIRENT_D_TYPE
+	if (d->d_type == DT_REG)
+		return TRUE;
+	else if (d->d_type != DT_UNKNOWN)
+		return FALSE;
+#endif
+
+	return (stat(d->d_name, &s) == 0 && S_ISREG(s.st_mode));
+}
+
+gboolean dirent_is_directory(struct dirent *d)
+{
+	struct stat s;
+
+#ifdef HAVE_DIRENT_D_TYPE
+	if (d->d_type == DT_DIR)
+		return TRUE;
+	else if (d->d_type != DT_UNKNOWN)
+		return FALSE;
+#endif
+
+	return (stat(d->d_name, &s) == 0 && S_ISDIR(s.st_mode));
+}
+
 gint change_dir(const gchar *dir)
 {
 	gchar *prevdir = NULL;
@@ -1655,11 +1933,13 @@ gint remove_all_files(const gchar *dir)
 
 	if (chdir(dir) < 0) {
 		FILE_OP_ERROR(dir, "chdir");
+		g_free(prev_dir);
 		return -1;
 	}
 
 	if ((dp = opendir(".")) == NULL) {
 		FILE_OP_ERROR(dir, "opendir");
+		g_free(prev_dir);
 		return -1;
 	}
 
@@ -1690,23 +1970,25 @@ gint remove_numbered_files(const gchar *dir, guint first, guint last)
 	DIR *dp;
 	struct dirent *d;
 	gchar *prev_dir;
-	gint fileno;
+	gint file_no;
 
 	prev_dir = g_get_current_dir();
 
 	if (chdir(dir) < 0) {
 		FILE_OP_ERROR(dir, "chdir");
+		g_free(prev_dir);
 		return -1;
 	}
 
 	if ((dp = opendir(".")) == NULL) {
 		FILE_OP_ERROR(dir, "opendir");
+		g_free(prev_dir);
 		return -1;
 	}
 
 	while ((d = readdir(dp)) != NULL) {
-		fileno = to_number(d->d_name);
-		if (fileno >= 0 && first <= fileno && fileno <= last) {
+		file_no = to_number(d->d_name);
+		if (file_no > 0 && first <= file_no && file_no <= last) {
 			if (is_dir_exist(d->d_name))
 				continue;
 			if (unlink(d->d_name) < 0)
@@ -1738,18 +2020,20 @@ gint remove_expired_files(const gchar *dir, guint hours)
 	struct dirent *d;
 	struct stat s;
 	gchar *prev_dir;
-	gint fileno;
+	gint file_no;
 	time_t mtime, now, expire_time;
 
 	prev_dir = g_get_current_dir();
 
 	if (chdir(dir) < 0) {
 		FILE_OP_ERROR(dir, "chdir");
+		g_free(prev_dir);
 		return -1;
 	}
 
 	if ((dp = opendir(".")) == NULL) {
 		FILE_OP_ERROR(dir, "opendir");
+		g_free(prev_dir);
 		return -1;
 	}
 
@@ -1757,8 +2041,8 @@ gint remove_expired_files(const gchar *dir, guint hours)
 	expire_time = hours * 60 * 60;
 
 	while ((d = readdir(dp)) != NULL) {
-		fileno = to_number(d->d_name);
-		if (fileno >= 0) {
+		file_no = to_number(d->d_name);
+		if (file_no > 0) {
 			if (stat(d->d_name, &s) < 0) {
 				FILE_OP_ERROR(d->d_name, "stat");
 				continue;
@@ -1841,14 +2125,9 @@ gint remove_dir_recursive(const gchar *dir)
 		    !strcmp(d->d_name, ".."))
 			continue;
 
-		if (stat(d->d_name, &s) < 0) {
-			FILE_OP_ERROR(d->d_name, "stat");
-			continue;
-		}
-
 		/* g_print("removing %s\n", d->d_name); */
 
-		if (S_ISDIR(s.st_mode)) {
+		if (dirent_is_directory(d)) {
 			if (remove_dir_recursive(d->d_name) < 0) {
 				g_warning("can't remove directory\n");
 				return -1;
@@ -1960,6 +2239,41 @@ gint copy_file(const gchar *src, const gchar *dest, gboolean keep_backup)
 	return 0;
 }
 
+gint copy_dir(const gchar *src, const gchar *dest)
+{
+	DIR *dp;
+	struct dirent *d;
+	gchar *src_file;
+	gchar *dest_file;
+
+	if ((dp = opendir(src)) == NULL) {
+		FILE_OP_ERROR(src, "opendir");
+		return -1;
+	}
+
+	if (make_dir_hier(dest) < 0) {
+		closedir(dp);
+		return -1;
+	}
+
+	while ((d = readdir(dp)) != NULL) {
+		if (!strcmp(d->d_name, ".") || !strcmp(d->d_name, ".."))
+			continue;
+
+		src_file = g_strconcat(src, G_DIR_SEPARATOR_S, d->d_name, NULL);
+		dest_file = g_strconcat(dest, G_DIR_SEPARATOR_S, d->d_name,
+					NULL);
+		if (is_file_exist(src_file))
+			copy_file(src_file, dest_file, FALSE);
+		g_free(dest_file);
+		g_free(src_file);
+	}
+
+	closedir(dp);
+
+	return 0;
+}
+
 gint move_file(const gchar *src, const gchar *dest, gboolean overwrite)
 {
 	if (overwrite == FALSE && is_file_exist(dest)) {
@@ -1981,9 +2295,100 @@ gint move_file(const gchar *src, const gchar *dest, gboolean overwrite)
 	return 0;
 }
 
+gint copy_file_part(FILE *fp, off_t offset, size_t length, const gchar *dest)
+{
+	FILE *dest_fp;
+	gint n_read;
+	gint bytes_left, to_read;
+	gchar buf[BUFSIZ];
+	gboolean err = FALSE;
+
+	if (fseek(fp, offset, SEEK_SET) < 0) {
+		perror("fseek");
+		return -1;
+	}
+
+	if ((dest_fp = fopen(dest, "wb")) == NULL) {
+		FILE_OP_ERROR(dest, "fopen");
+		return -1;
+	}
+
+	if (change_file_mode_rw(dest_fp, dest) < 0) {
+		FILE_OP_ERROR(dest, "chmod");
+		g_warning("can't change file mode\n");
+	}
+
+	bytes_left = length;
+	to_read = MIN(bytes_left, sizeof(buf));
+
+	while ((n_read = fread(buf, sizeof(gchar), to_read, fp)) > 0) {
+		if (n_read < to_read && ferror(fp))
+			break;
+		if (fwrite(buf, n_read, 1, dest_fp) < 1) {
+			g_warning(_("writing to %s failed.\n"), dest);
+			fclose(dest_fp);
+			unlink(dest);
+			return -1;
+		}
+		bytes_left -= n_read;
+		if (bytes_left == 0)
+			break;
+		to_read = MIN(bytes_left, sizeof(buf));
+	}
+
+	if (ferror(fp)) {
+		perror("fread");
+		err = TRUE;
+	}
+	if (fclose(dest_fp) == EOF) {
+		FILE_OP_ERROR(dest, "fclose");
+		err = TRUE;
+	}
+
+	if (err) {
+		unlink(dest);
+		return -1;
+	}
+
+	return 0;
+}
+
 /* convert line endings into CRLF. If the last line doesn't end with
  * linebreak, add it.
  */
+gchar *canonicalize_str(const gchar *str)
+{
+	const gchar *p;
+	guint new_len = 0;
+	gchar *out, *outp;
+
+	for (p = str; *p != '\0'; ++p) {
+		if (*p != '\r') {
+			++new_len;
+			if (*p == '\n')
+				++new_len;
+		}
+	}
+	if (p == str || *(p - 1) != '\n')
+		new_len += 2;
+
+	out = outp = g_malloc(new_len + 1);
+	for (p = str; *p != '\0'; ++p) {
+		if (*p != '\r') {
+			if (*p == '\n')
+				*outp++ = '\r';
+			*outp++ = *p;
+		}
+	}
+	if (p == str || *(p - 1) != '\n') {
+		*outp++ = '\r';
+		*outp++ = '\n';
+	}
+	*outp = '\0';
+
+	return out;
+}
+
 gint canonicalize_file(const gchar *src, const gchar *dest)
 {
 	FILE *src_fp, *dest_fp;
@@ -2045,7 +2450,7 @@ gint canonicalize_file(const gchar *src, const gchar *dest)
 	}
 
 	if (ferror(src_fp)) {
-		FILE_OP_ERROR(src, "fread");
+		FILE_OP_ERROR(src, "fgets");
 		err = TRUE;
 	}
 	fclose(src_fp);
@@ -2084,6 +2489,195 @@ gint canonicalize_file_replace(const gchar *file)
 	return 0;
 }
 
+gint uncanonicalize_file(const gchar *src, const gchar *dest)
+{
+	FILE *src_fp, *dest_fp;
+	gchar buf[BUFFSIZE];
+	gboolean err = FALSE;
+
+	if ((src_fp = fopen(src, "rb")) == NULL) {
+		FILE_OP_ERROR(src, "fopen");
+		return -1;
+	}
+
+	if ((dest_fp = fopen(dest, "wb")) == NULL) {
+		FILE_OP_ERROR(dest, "fopen");
+		fclose(src_fp);
+		return -1;
+	}
+
+	if (change_file_mode_rw(dest_fp, dest) < 0) {
+		FILE_OP_ERROR(dest, "chmod");
+		g_warning("can't change file mode\n");
+	}
+
+	while (fgets(buf, sizeof(buf), src_fp) != NULL) {
+		strcrchomp(buf);
+		if (fputs(buf, dest_fp) == EOF) {
+			g_warning("writing to %s failed.\n", dest);
+			fclose(dest_fp);
+			fclose(src_fp);
+			unlink(dest);
+			return -1;
+		}
+	}
+
+	if (ferror(src_fp)) {
+		FILE_OP_ERROR(src, "fgets");
+		err = TRUE;
+	}
+	fclose(src_fp);
+	if (fclose(dest_fp) == EOF) {
+		FILE_OP_ERROR(dest, "fclose");
+		err = TRUE;
+	}
+
+	if (err) {
+		unlink(dest);
+		return -1;
+	}
+
+	return 0;
+}
+
+gint uncanonicalize_file_replace(const gchar *file)
+{
+	gchar *tmp_file;
+
+	tmp_file = get_tmp_file();
+
+	if (uncanonicalize_file(file, tmp_file) < 0) {
+		g_free(tmp_file);
+		return -1;
+	}
+
+	if (move_file(tmp_file, file, TRUE) < 0) {
+		g_warning("can't replace %s .\n", file);
+		unlink(tmp_file);
+		g_free(tmp_file);
+		return -1;
+	}
+
+	g_free(tmp_file);
+	return 0;
+}
+
+gchar *normalize_newlines(const gchar *str)
+{
+	const gchar *p = str;
+	gchar *out, *outp;
+
+	out = outp = g_malloc(strlen(str) + 1);
+	for (p = str; *p != '\0'; ++p) {
+		if (*p == '\r') {
+			if (*(p + 1) != '\n')
+				*outp++ = '\n';
+		} else
+			*outp++ = *p;
+	}
+
+	*outp = '\0';
+
+	return out;
+}
+
+gchar *get_outgoing_rfc2822_str(FILE *fp)
+{
+	gchar buf[BUFFSIZE];
+	GString *str;
+	gchar *ret;
+
+	str = g_string_new(NULL);
+
+	/* output header part */
+	while (fgets(buf, sizeof(buf), fp) != NULL) {
+		strretchomp(buf);
+		if (!g_strncasecmp(buf, "Bcc:", 4)) {
+			gint next;
+
+			for (;;) {
+				next = fgetc(fp);
+				if (next == EOF)
+					break;
+				else if (next != ' ' && next != '\t') {
+					ungetc(next, fp);
+					break;
+				}
+				if (fgets(buf, sizeof(buf), fp) == NULL)
+					break;
+			}
+		} else {
+			g_string_append(str, buf);
+			g_string_append(str, "\r\n");
+			if (buf[0] == '\0')
+				break;
+		}
+	}
+
+	/* output body part */
+	while (fgets(buf, sizeof(buf), fp) != NULL) {
+		strretchomp(buf);
+		if (buf[0] == '.')
+			g_string_append_c(str, '.');
+		g_string_append(str, buf);
+		g_string_append(str, "\r\n");
+	}
+
+	ret = str->str;
+	g_string_free(str, FALSE);
+
+	return ret;
+}
+
+/*
+ * Create a new boundary in a way that it is very unlikely that this
+ * will occur in the following text.  It would be easy to ensure
+ * uniqueness if everything is either quoted-printable or base64
+ * encoded (note that conversion is allowed), but because MIME bodies
+ * may be nested, it may happen that the same boundary has already
+ * been used. We avoid scanning the message for conflicts and hope the
+ * best.
+ *
+ *   boundary := 0*69<bchars> bcharsnospace
+ *   bchars := bcharsnospace / " "
+ *   bcharsnospace := DIGIT / ALPHA / "'" / "(" / ")" /
+ *                    "+" / "_" / "," / "-" / "." /
+ *                    "/" / ":" / "=" / "?"
+ *
+ * some special characters removed because of buggy MTAs
+ */
+
+gchar *generate_mime_boundary(const gchar *prefix)
+{
+	static gchar tbl[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+			     "abcdefghijklmnopqrstuvwxyz"
+			     "1234567890+_./=";
+	gchar buf_uniq[17];
+	gchar buf_date[64];
+	gint i;
+	gint pid;
+
+	pid = getpid();
+
+	/* We make the boundary depend on the pid, so that all running
+	 * processes generate different values even when they have been
+	 * started within the same second and srandom(time(NULL)) has been
+	 * used.  I can't see whether this is really an advantage but it
+	 * doesn't do any harm.
+	 */
+	for (i = 0; i < sizeof(buf_uniq) - 1; i++)
+		buf_uniq[i] = tbl[(random() ^ pid) % (sizeof(tbl) - 1)];
+	buf_uniq[i] = '\0';
+
+	get_rfc822_date(buf_date, sizeof(buf_date));
+	subst_char(buf_date, ' ', '_');
+	subst_char(buf_date, ',', '_');
+	subst_char(buf_date, ':', '_');
+
+	return g_strdup_printf("%s=_%s_%s", prefix ? prefix : "Multipart",
+			       buf_date, buf_uniq);
+}
+
 gint change_file_mode_rw(FILE *fp, const gchar *file)
 {
 #if HAVE_FCHMOD
@@ -2196,10 +2790,7 @@ gint str_write_to_file(const gchar *str, const gchar *file)
 
 gchar *file_read_to_str(const gchar *file)
 {
-	GByteArray *array;
 	FILE *fp;
-	gchar buf[BUFSIZ];
-	gint n_read;
 	gchar *str;
 
 	g_return_val_if_fail(file != NULL, NULL);
@@ -2209,6 +2800,22 @@ gchar *file_read_to_str(const gchar *file)
 		return NULL;
 	}
 
+	str = file_read_stream_to_str(fp);
+
+	fclose(fp);
+
+	return str;
+}
+
+gchar *file_read_stream_to_str(FILE *fp)
+{
+	GByteArray *array;
+	gchar buf[BUFSIZ];
+	gint n_read;
+	gchar *str;
+
+	g_return_val_if_fail(fp != NULL, NULL);
+
 	array = g_byte_array_new();
 
 	while ((n_read = fread(buf, sizeof(gchar), sizeof(buf), fp)) > 0) {
@@ -2218,14 +2825,11 @@ gchar *file_read_to_str(const gchar *file)
 	}
 
 	if (ferror(fp)) {
-		FILE_OP_ERROR(file, "fread");
-		fclose(fp);
+		FILE_OP_ERROR("file stream", "fread");
 		g_byte_array_free(array, TRUE);
 		return NULL;
 	}
 
-	fclose(fp);
-
 	buf[0] = '\0';
 	g_byte_array_append(array, buf, 1);
 	str = (gchar *)array->data;
@@ -2237,6 +2841,7 @@ gchar *file_read_to_str(const gchar *file)
 gint execute_async(gchar *const argv[])
 {
 	pid_t pid;
+	gint status;
 
 	if ((pid = fork()) < 0) {
 		perror("fork");
@@ -2261,14 +2866,18 @@ gint execute_async(gchar *const argv[])
 		_exit(0);
 	}
 
-	waitpid(pid, NULL, 0);
+	waitpid(pid, &status, 0);
 
-	return 0;
+	if (WIFEXITED(status))
+		return WEXITSTATUS(status);
+	else
+		return -1;
 }
 
 gint execute_sync(gchar *const argv[])
 {
 	pid_t pid;
+	gint status;
 
 	if ((pid = fork()) < 0) {
 		perror("fork");
@@ -2282,9 +2891,12 @@ gint execute_sync(gchar *const argv[])
 		_exit(1);
 	}
 
-	waitpid(pid, NULL, 0);
+	waitpid(pid, &status, 0);
 
-	return 0;
+	if (WIFEXITED(status))
+		return WEXITSTATUS(status);
+	else
+		return -1;
 }
 
 gint execute_command_line(const gchar *cmdline, gboolean async)
@@ -2292,20 +2904,49 @@ gint execute_command_line(const gchar *cmdline, gboolean async)
 	gchar **argv;
 	gint ret;
 
+	debug_print("executing: %s\n", cmdline);
+
 	argv = strsplit_with_quote(cmdline, " ", 0);
 
 	if (async)
 		ret = execute_async(argv);
 	else
 		ret = execute_sync(argv);
+
 	g_strfreev(argv);
 
 	return ret;
 }
 
+gchar *get_command_output(const gchar *cmdline)
+{
+	gchar buf[BUFFSIZE];
+	FILE *fp;
+	GString *str;
+	gchar *ret;
+
+	g_return_val_if_fail(cmdline != NULL, NULL);
+
+	if ((fp = popen(cmdline, "r")) == NULL) {
+		FILE_OP_ERROR(cmdline, "popen");
+		return NULL;
+	}
+
+	str = g_string_new("");
+
+	while (fgets(buf, sizeof(buf), fp) != NULL)
+		g_string_append(str, buf);
+
+	pclose(fp);
+
+	ret = str->str;
+	g_string_free(str, FALSE);
+
+	return ret;
+}
+
 gint open_uri(const gchar *uri, const gchar *cmdline)
 {
-	static gchar *default_cmdline = "netscape -remote openURL(%s,raise)";
 	gchar buf[BUFFSIZE];
 	gchar *p;
 
@@ -2317,9 +2958,10 @@ gint open_uri(const gchar *uri, const gchar *cmdline)
 		g_snprintf(buf, sizeof(buf), cmdline, uri);
 	else {
 		if (cmdline)
-			g_warning(_("Open URI command line is invalid: `%s'"),
+			g_warning("Open URI command line is invalid "
+				  "(there must be only one '%%s'): %s",
 				  cmdline);
-		g_snprintf(buf, sizeof(buf), default_cmdline, uri);
+		g_snprintf(buf, sizeof(buf), DEFAULT_BROWSER_CMD, uri);
 	}
 
 	execute_command_line(buf, TRUE);
@@ -2349,11 +2991,16 @@ time_t remote_tzoffset_sec(const gchar *zone)
 	} else if (!strncmp(zone, "UT" , 2) ||
 		   !strncmp(zone, "GMT", 2)) {
 		remoteoffset = 0;
-	} else if (strlen(zone3) == 3 &&
-		   (p = strstr(ustzstr, zone3)) != NULL &&
-		   (p - ustzstr) % 3 == 0) {
-		iustz = ((gint)(p - ustzstr) / 3 + 1) / 2 - 8;
-		remoteoffset = iustz * 3600;
+	} else if (strlen(zone3) == 3) {
+		for (p = ustzstr; *p != '\0'; p += 3) {
+			if (!strncasecmp(p, zone3, 3)) {
+				iustz = ((gint)(p - ustzstr) / 3 + 1) / 2 - 8;
+				remoteoffset = iustz * 3600;
+				break;
+			}
+		}
+		if (*p == '\0')
+			return -1;
 	} else if (strlen(zone3) == 1) {
 		switch (zone[0]) {
 		case 'Z': remoteoffset =   0; break;
@@ -2384,7 +3031,8 @@ time_t remote_tzoffset_sec(const gchar *zone)
 		default:  remoteoffset =   0; break;
 		}
 		remoteoffset = remoteoffset * 3600;
-	}
+	} else
+		return -1;
 
 	return remoteoffset;
 }
@@ -2412,10 +3060,6 @@ time_t tzoffset_sec(time_t *now)
 		off = 23 * 60 + 59;	/* if not, insert silly value */
 	if (off <= -24 * 60)
 		off = -(23 * 60 + 59);
-	if (off > 12 * 60)
-		off -= 24 * 60;
-	if (off < -12 * 60)
-		off += 24 * 60;
 
 	return off * 60;
 }
@@ -2510,7 +3154,7 @@ void debug_print(const gchar *format, ...)
 	g_vsnprintf(buf, sizeof(buf), format, args);
 	va_end(args);
 
-	fputs(buf, stdout);
+	g_print("%s", buf);
 }
 
 #define TIME_LEN	11
